@@ -148,7 +148,7 @@ proc asdeno_prof { fen } {
    set n [llength $lmod]
    for {set i 0} {$i < $n} {incr i} {
       set astk::profil($var,fich,$i,nom)      [lindex $lmod $i]
-      set astk::profil($var,fich,$i,serv)     $astk::config($astk::agla(num_serv),nom)
+      set astk::profil($var,fich,$i,serv)     $astk::local_server
       set astk::profil($var,fich,$i,type)     $type
       set astk::profil($var,fich,$i,UL)       0
       set astk::profil($var,fich,$i,donnee)   1
