@@ -59,6 +59,10 @@ proc def_onglet { } {
    set astk::ihm(tit,$n,$i) [ashare::mess ihm 144]
    set astk::ihm(ctx,$n,$i) 287
    incr i
+   set astk::ihm(ong,$n,$i) "asdenot"
+   set astk::ihm(tit,$n,$i) [ashare::mess ihm 428]
+   set astk::ihm(ctx,$n,$i) 287
+   incr i
    set astk::ihm(ong,$n,$i) "asquit"
    set astk::ihm(tit,$n,$i) [ashare::mess ihm 152]
    set astk::ihm(ctx,$n,$i) 288
@@ -167,7 +171,7 @@ proc fenEEC { fenetre_parent nongl } {
 # n'est plus utilise avec la case  cocher
 #################################################################
 proc fenEEC2 {fenetre_parent chaine {vari titre} {coch 0}} {
-   pack [frame $fenetre_parent.titre -relief solid -bd 1 -background $astk::ihm(couleur,onglet_inactif)] 
+   pack [frame $fenetre_parent.titre -relief solid -bd 1 -background $astk::ihm(couleur,onglet_inactif)]
    label $fenetre_parent.titre.label -font $astk::ihm(font,labbout) -text $chaine  -background $astk::ihm(couleur,onglet_inactif)
    pack $fenetre_parent.titre.label -side left
    if { $coch } {
@@ -207,13 +211,13 @@ proc affiche_onglet { nom } {
          }
       }
    }
-   
+
    $astk::ihm(fenetre).onglet.$ongP_off.titre.songl configure -relief raised -background $astk::ihm(couleur,onglet_inactif) -activebackground $astk::ihm(couleur,onglet_inactif)
    # on s'arrete l si none
    if { $nom == "none" } { return }
    set astk::profil(onglet_actif) $nom
    $astk::ihm(fenetre).onglet.$ongP_on.titre.songl configure -relief sunken -background $astk::ihm(couleur,onglet_actif) -activebackground $astk::ihm(couleur,onglet_actif)
-   
+
    # status
    change_status [ashare::mess ihm 12]
 #
@@ -421,7 +425,11 @@ proc mod_option { m opt } {
    }
 # demande la valeur
    set last $astk::profil(opt_val,$opt)
-   getValue_fen . 24 267 {} $last vide 1
+   if { [lindex $lst 0 ] == "OUI" } {
+       getValue_chb . 24 $opt $lst $last
+   } else {
+       getValue_fen . 24 267 {} $last vide 1
+   }
    tkwait window .fen_getv
    if { $getValue_val == "" || [lsearch -exact $lst $getValue_val] > -1 || [llength $lst] == 0 } {
       set astk::profil(opt_val,$opt) $getValue_val
