@@ -26,6 +26,7 @@ namespace eval astk {
    variable recup
    variable rcdir_mod
    variable rcdir
+   variable rcdir_arg
    variable tmpdir
    variable local_server
    variable fic_serveurs
@@ -45,6 +46,7 @@ namespace eval astk {
    # Structures de donnes
    # (voir DOC/sdd_'nom de la sdd'.txt pour le dtail)
    variable UL
+   variable UL_ref
    variable profil
    variable config
    variable agla
@@ -160,12 +162,16 @@ proc init_gene { root } {
    set astk::config(-1,etat) "on"
 
 # chemin des ressources
+# rcdir_arg contiendra par exemple .astkrc_salomeXX, c'est
+# lui qui est pass  as_run
+   set astk::rcdir_arg $astk::rcdir
    set astk::rcdir [file join $astk::config(-1,home) $astk::rcdir]
 
 # rpertoire temporaire
    set astk::tmpdir "/tmp/astk_$astk::config(-1,login)"
 
    file mkdir "$astk::rcdir"
+   catch { file delete [file join "$astk::tmpdir" *] }
    file mkdir "$astk::tmpdir"
 
    #  dit si on rcupre ou pas les infos de config
@@ -226,7 +232,9 @@ proc init_gene { root } {
    set astk::ihm(font,labbout) "-Adobe-Helvetica-Bold-R-Normal--*-120-*-*-*-*-*-*"
    # fichier des logs
    set ashare::fic_log [file join $astk::rcdir log]
-   
+   set ashare::fic_progress [file join $astk::rcdir logprogress]
+   catch { file delete $ashare::fic_progress }
+
    # init des listes arguments, paramtres
    set astk::ihm(l_arg) { dbgjeveux  non 261 \
                           rep_outils oui 262 \
@@ -239,14 +247,15 @@ proc init_gene { root } {
                           facmtps      266 {} \
                           corefilesize 293 {} \
                         }
+   # on se sert du "OUI" en premier pour proposer une liste cochable oui/non
    set astk::ihm(l_opt) { ncpus        44 {} \
                           mpi_nbcpu   300 {} \
                           mpi_nbnoeud 370 {} \
                           classe      334 {} \
                           depart      335 {} \
-                          distrib     386 {OUI YES oui yes} \
-                          flashdir    387 {} \
+                          distrib     386 {OUI YES oui yes NON NO non no} \
                           exectool    385 {} \
+                          multiple    413 {OUI YES oui yes NON NO non no}
                         }
 }
 
@@ -390,7 +399,7 @@ proc init_icon { {theme crystal} } {
                folder empty up reload openterm \
                help help salome \
                1uparrow 1downarrow }
-   
+
    if { $theme == "vaio" } {
       set prefix "vaio_"
       set ficon { openRS openS newS deleteS editS \
@@ -412,7 +421,7 @@ proc init_icon { {theme crystal} } {
    # add iconwindow
    append icons "iconwindow"
    append ficon "iconwindow"
-   
+
    set nmod [llength $icons]
    for {set i 0} {$i < $nmod} {incr i} {
       set iconi [lindex $icons $i]
@@ -586,7 +595,7 @@ proc init_font { } {
       # valeurs individuelles
       set name($ff) [lindex $astk::ihm(style,font_$ff) 0]
       set size($ff) [lindex $astk::ihm(style,font_$ff) 1]
-      set weig($ff) "" 
+      set weig($ff) ""
       catch {set weig($ff) [lindex $astk::ihm(style,font_$ff) 2]}
    }
 # dfinition des fonts utilises
@@ -626,13 +635,15 @@ proc init_mots { } {
 # fichier de messages
    ashare::init_mots_msg
 # fichier prefs
-   set ashare::mots(MCS_pref) { nom_user email org def_vers xterm editeur \
-                                nb_reman langue dbglevel freq_actu nb_ligne \
-                                flag_maj isdhcp ipdhcp nom_domaine bip \
-                                remote_copy_protocol remote_shell_protocol \
-                                couleur font_main font_fixe font_bsf \
-                                Geom_astk Geom_asjob Geom_bsf Geom_rex Geom_log \
-                                browser theme forced_display }
+   set ashare::mots(MCS_pref) { nom_user email org def_vers xterm editeur
+                                nb_reman langue dbglevel freq_actu nb_ligne
+                                flag_maj isdhcp ipdhcp nom_domaine bip
+                                remote_copy_protocol remote_shell_protocol
+                                couleur font_main font_fixe font_bsf
+                                Geom_astk Geom_asjob Geom_bsf Geom_rex Geom_log
+                                browser theme forced_display
+                                listul user_listul
+                                }
 # fichier lastprof
    set ashare::mots(MCF_prof) { profil }
    set ashare::mots(SSF_prof) { serv }
@@ -660,7 +671,7 @@ proc init_pltf { } {
    set astk::ihm(pltf,LINUX32) "Linux x86"
    set astk::ihm(pltf,WIN64) "Windows 64 bits"
    set astk::ihm(pltf,WIN32) "Windows"
-   
+
    set astk::ihm(pltf,P_LINUX) "Linux x86"
    set astk::ihm(pltf,TRU64) "OSF1 TRU64"
    set astk::ihm(pltf,SOLARIS) "Solaris"
@@ -678,7 +689,7 @@ proc init_ihm { } {
    init_font
    init_pltf
    init_lang
-   
+
    if { $ashare::noinitwarn == 1 } {
       set ashare::show_dialog 0
    }
@@ -821,16 +832,19 @@ proc update_opt { opt auth_value {REINI "APPEND"} } {
    for {set i 0} {$i < $npar} {incr i} {
       set var [lindex $astk::ihm(l_opt) [expr $i * 3]]
       if { $var == $opt } {
+         set prev [lindex $astk::ihm(l_opt) [expr $i * 3 + 2]]
+         set new {}
          set trouv 1
          if { $REINI == "APPEND" } {
             # on vite les doublons
+            set new $prev
             foreach val $auth_value {
-               if { [lsearch $val [lindex $astk::ihm(l_opt) [expr $i * 3 + 2]]] > -1 } {
-                  set auth_value "[lindex $astk::ihm(l_opt) [expr $i * 3 + 2]] $val"
+               if { [lsearch $prev $val] < 0 } {
+                  set new "$new $val"
                }
             }
          }
-         lset astk::ihm(l_opt) [expr $i * 3 + 2] $auth_value
+         lset astk::ihm(l_opt) [expr $i * 3 + 2] $new
          break
       }
    }
@@ -840,4 +854,3 @@ proc update_opt { opt auth_value {REINI "APPEND"} } {
       ashare::mess "info" 48 $opt $auth_value
    }
 }
-
