@@ -18,7 +18,7 @@
 # ==============================================================================
 
 """
-Definition of ASTER_CALCUL class and its derivated.
+Definition of AsterCalcul class and its derivated.
 """
 
 import os
@@ -26,18 +26,23 @@ import os.path as osp
 import re
 import time
 
-from asrun.installation     import aster_root, confdir
+from asrun.installation     import confdir
 from asrun.common.i18n      import _
+from asrun.core             import magic
 from asrun.mystring         import convert, print3, ufmt
 from asrun.job              import Func_actu, Func_tail, Del
-from asrun.profil           import ASTER_PROFIL
+from asrun.build            import AsterBuild
+from asrun.config           import build_AsterConfig
+from asrun.profil           import AsterProfil
 from asrun.system           import shell_cmd
 from asrun.execution        import build_test_export
-from asrun.profile_modifier import ModifierFactory, SEPAR
-from asrun.common_func      import get_tmpname
+from asrun.batch            import BatchSystemFactory
+from asrun.profile_modifier import apply_special_service
+from asrun.common_func      import get_tmpname, flash_filename, is_localhost2, same_hosts2
 from asrun.common.rcfile    import get_nodepara
-from asrun.common.utils     import find_command, YES_VALUES, NO_VALUES
-from asrun.common.sysutils  import is_localhost
+from asrun.common.utils     import find_command, YES_VALUES, NO_VALUES, get_plugin
+
+from asrun.backward_compatibility import bwc_deprecate_class
 
 
 msg_pb_version = u"""######################################################################
@@ -45,12 +50,17 @@ msg_pb_version = u"""###########################################################
 WARNING : ASTK Server version %s differs from
           ASTK Client version %s.
           An update from Server or Client is necessary.
-          This execution may fail ! 
+          This execution may fail !
           If it works results should be correct.
 
 ######################################################################
 """
 
+msg_cli_version = u"""
+No ASTK Client version found in the profile.
+Can not check compatibility with the ASTK Server (version %s).
+"""
+
 msg_info = u"""###############################################
            Client name : %(mcli)s
               Username : %(ucli)s
@@ -67,6 +77,7 @@ msg_info = u"""###############################################
   Number of processors : %(nbp)s   (OpenMP)
        Number of nodes : %(mpi_nbnoeud)s   (MPI)
   Number of processors : %(mpi_nbcpu)s   (MPI)
+                  Mode : %(mode)s
 %(sep)s
             Debug mode : %(dbg)s"""
 
@@ -79,26 +90,107 @@ msg_vers  = u"""   Version ASTK Server : %s
 """
 
 
-class ASTER_CALCUL:
+class BaseCalcul(object):
+    """Common part between AsterCalcul and AsterCalcHandler objects."""
+
+    def __init__(self):
+        self.prof = None
+        self.scheduler = None
+        self.jobid = -1
+        self.studyid = -1
+        self.queue = "unknown"
+        self.name = "unnamed"
+        self.diag = '?'
+        self.start_time = self.end_time = None
+
+    def error(self, msg):
+        """Print an error msg and exit."""
+        magic.run.Mess(msg, '<F>_ERROR')
+
+    def start(self, *args, **kwargs):
+        self.is_starting()
+        raise NotImplementedError('must be defined in a derivated class')
+
+    def is_starting(self):
+        """Mark as started."""
+        self.start_time = time.time()
+
+    def finish(self, state):
+        """Mark as ended if state is ENDED."""
+        if state == 'ENDED':
+            self.end_time = time.time()
+
+    def is_ended(self):
+        """Tell if the calculation is ended."""
+        return self.end_time is not None
+
+    def get_state(self):
+        """Return current state of the job."""
+        # should call finish() to fill end_time
+        raise NotImplementedError('must be defined in a derivated class')
+
+    def tail(self, **kwargs):
+        """Return tail of current output of the job."""
+        raise NotImplementedError('must be defined in a derivated class')
+
+    def wait(self, refresh_delay=1.):
+        """Wait for job completion."""
+        state = '_'
+        while state != 'ENDED':
+            time.sleep(refresh_delay)
+            res = self.get_state()
+            state, diag = res[0:2]
+        self.diag = diag
+
+    def get_diag(self):
+        """Return diagnostic of the execution based on the output file."""
+        raise NotImplementedError('must be defined in a derivated class')
+
+    def kill(self):
+        """Kill the job (if it is running) and delete of its files."""
+        raise NotImplementedError('must be defined in a derivated class')
+
+    def request(self, key):
+        """Return the value to request."""
+        if key == 'cpu':
+            res = max(int(self.prof['ncpus'][0] or 1), int(self.prof['mpi_nbcpu'][0] or 1))
+        elif key == 'mem':
+            res = float(self.prof['memjob'][0] or 0.) / 1024.
+        else:
+            res = 0
+        return res
+
+    def on_host(self, serv, node):
+        """Change submission serv/node.
+        """
+        if serv is not None:
+            self.prof['serveur'] = serv
+        if node is not None:
+            self.prof['noeud'] = node
+
+
+class AsterCalcul(BaseCalcul):
     """This class read user's profile and (if needed) call as_run through or not
     a terminal, or just write a btc file...
     """
-    _supported_services     = ('study', 'parametric_study', 'meshtool', 'stanley',
-                                        'convbase', 'distribution', 'exectool')
-    _available_services_ref = ('study',                     'meshtool', 'stanley',
-                                        'convbase',                 'exectool')
+    _supported_services     = ('study', 'parametric_study', 'testcase', 'meshtool',
+                               'stanley', 'convbase', 'distribution', 'exectool',
+                               'multiple', 'agla')
+    _available_services_ref = ('agla',)
+    _agla_actions =           ('asno', 'asdeno', 'asdenot', 'asquit', 'asverif', 'pre_eda', 'asrest')
 
-    def __init__(self, run, filename=None, prof=None, pid=None):
+
+    def __init__(self, run, filename=None, prof=None, pid=None, differ_init=False):
         """Initializations
         """
+        BaseCalcul.__init__(self)
         assert filename or prof, 'none of (filename, prof) provided!'
         self.run = run
         if pid is None:
             self.pid = self.run['num_job']
         else:
             self.pid = pid
-        self.jobid = -1
-        self.queue = "unknown"
+        self.studyid = self.pid
 
         if prof is not None:
             self.prof = prof
@@ -107,68 +199,73 @@ class ASTER_CALCUL:
             fprof = get_tmpname(self.run, self.run['tmp_user'], basename='profil_astk')
             self.run.ToDelete(fprof)
             kret = self.run.Copy(fprof, filename, niverr='<F>_PROFILE_COPY')
-            self.prof = ASTER_PROFIL(fprof, self.run)
+            self.prof = AsterProfil(fprof, self.run)
         if self.prof['nomjob'][0] == '':
             self.prof['nomjob'] = 'unnamed'
 
-        # ----- decode service called
-        self.decode_special_service()
-
         # attributes
         self.dict_info   = None
-        self.as_exec_ref = osp.join(aster_root, 'ASTK', 'ASTK_SERV', 'bin', 'as_exec_ref')
+        self.as_exec_ref = self.run.get('as_exec_ref')
         self.diag        = '?'
+        self.__initialized = False
+
+        if not differ_init:
+            self.finalize_init()
+
+
+    def finalize_init(self):
+        """Finalize initialization.
+        Allow to adapt prof object before customization."""
+        # decode service called
+        self.decode_special_service()
+
+        # allow customization of calcul object
+        if self.run['schema_calcul']:
+            schem = get_plugin(self.run['schema_calcul'])
+            self.run.DBG("calling plugin : %s" % self.run['schema_calcul'])
+            self.prof = schem(self)
+
+        self.__initialized = True
 
-    def error(self, msg):
-        """Print an error msg and exit.
-        """
-        self.run.Mess(msg, '<F>_ERROR')
 
     def decode_special_service(self):
         """Return the profile modified for the "special" service.
         """
-        self.serv = self.prof['special'][0].split(SEPAR)[0]
+        self.serv, self.prof = apply_special_service(self.prof, self.run)
         if self.serv == '':
-            if self.prof['distrib'][0] != '':
-                self.serv = 'distribution'
-            elif self.prof['exectool'][0] != '':
-                self.serv = 'exectool'
-            elif self.prof['parent'][0] == 'parametric':
+            if self.prof['parent'][0] == 'parametric':
                 self.serv = 'parametric_study'
+            elif self.prof['parent'][0] == 'astout':
+                self.serv = 'testcase'
             else:
                 self.serv = 'study'
+                for act in self.prof['actions']:
+                    if act in self._agla_actions:
+                        self.serv = 'agla'
+                        break
+
         self.prof['service'] = self.serv
         self.run.DBG("service name : %s" % self.serv)
         if self.serv not in self._supported_services:
             self.error(_(u'Unknown service : %s') % self.serv)
-        elif self.serv in ('study', 'parametric_study'):
-            return
-        
-        modifier = ModifierFactory(self.serv, self.prof, self.run)
-        modifier.modify()
-        self.prof =  modifier.return_profile()
+
 
     def on_mach_ref(self):
-        """Are we on reference server ?
+        """Are we on the reference server ?
         """
-        return osp.exists(self.as_exec_ref) and os.access(self.as_exec_ref, os.X_OK)
+        return self.as_exec_ref and osp.exists(self.as_exec_ref)\
+            and os.access(self.as_exec_ref, os.X_OK)
 
-    def flash(self, typ, num_job=None):
-        """If typ='o', return something like .../flasheur/nomjob.o1234
-        """
-        assert self.prof is not None
-        if num_job is None:
-            num_job = self.jobid
-        return osp.join(self.run['flasheur'],
-                '%s.%s%s' % (self.prof['nomjob'][0], typ, num_job))
 
     def build_dict_info(self, opts):
         """Build a dictionnary grouping all parameters.
         """
         sep = "-----------------------------------------------"
         self.mode = self.prof['mode'][0]
-        if not self.mode:
+        if not self.mode or self.run.get(self.mode) not in YES_VALUES:
             self.mode = self.prof['mode'] = "interactif"
+        if self.mode == 'batch':
+            self.scheduler = BatchSystemFactory(self.run, self.prof)
         node = self.prof['noeud'][0] or self.prof['serveur'][0]
         self.dict_info = {
             'sep'                   : sep,
@@ -190,19 +287,19 @@ class ASTER_CALCUL:
             'prof_content'          : self.prof.get_content(),
             'nomjob'                : self.prof['nomjob'][0],
             'nomjob_'               : self.flash('', ''),
-            'nomjob_p'              : self.flash('p', '$num_job'),
+            'nomjob_p'              : self.flash('export', '$num_job'),
             'as_run_cmd'            : " ".join(self.run.get_as_run_cmd(with_args=False)),
             'who'                   : self.run.system.getuser_host()[0],
             'opts'                  : opts,
             'remote_args'           : " ".join(self.run.get_as_run_args()),
         }
-        
+
         if self.prof['srv_dbg'][0] in YES_VALUES:
             self.dict_info['opts'] += ' --debug'
         if self.prof['srv_verb'][0] in YES_VALUES:
             self.dict_info['opts'] += ' --verbose'
-        
-        # rep_trav from profile or config(_nodename)
+
+        # rep_trav from profile or config(_nodename) / keep consistancy with job.py
         rep_trav = self.prof['rep_trav'][0]
         if rep_trav == '':
             rep_trav = get_nodepara(node, 'rep_trav', self.run['rep_trav'])
@@ -223,7 +320,9 @@ class ASTER_CALCUL:
         except Exception, msg:
             self.run.DBG('Error : unexpected "origine" value :', self.prof['origine'][0])
             client_vers = ''
-        if serv_vers != client_vers:
+        if client_vers == '':
+            ASTK_MESSAGE.append(msg_cli_version % serv_vers)
+        elif serv_vers != client_vers:
             ASTK_MESSAGE.append(msg_pb_version % (serv_vers, client_vers))
 
         ASTK_MESSAGE.append(msg_info % self.dict_info)
@@ -250,11 +349,7 @@ class ASTER_CALCUL:
         """Write btc file.
         """
         assert type(self.dict_info) is dict
-        
-        if self.prof['display'][0].strip() not in ("", ":0", ":0.0"):
-            str_display = 'DISPLAY=%s ; export DISPLAY' % self.prof['display'][0]
-        else:
-            str_display = ''
+
         aster_profile = osp.join(confdir, 'profile.sh')
         if osp.exists(aster_profile):
             str_aster_profile = '. %s' % aster_profile
@@ -267,12 +362,12 @@ class ASTER_CALCUL:
             m = 'b'
             # ne conserver que le numero (par ex: 12345@node12 => 12345)
             str_pid = """num_job=`echo $%s | awk '{inv=$0; sub("^[0-9]+","",inv); """ \
-                      """sub(inv,"",$0); print $0;}'`""" % self.run['batch_jid']
+                      """sub(inv,"",$0); print $0;}'`""" % self.scheduler.bjid
 
-        btc = r"""#!/bin/sh
+        btc = r"""#!/bin/bash
 
-%(str_display)s
 %(str_aster_profile)s
+# Do not change the following line (used and changed if consbtc=no)
 %(str_pid)s
 
 # copie du .export dans le flasheur et rep_trav
@@ -302,7 +397,6 @@ egrep -- '--- DIAGNOSTIC JOB :' %(rep_trav)s/%(nomjob)s.$num_job.fort.6.%(m)s |
 """
         dict_btc = self.dict_info.copy()
         dict_btc.update({
-            'str_display'        : str_display,
             'str_aster_profile'  : str_aster_profile,
             'str_pid'            : str_pid,
             'm'                  : m,
@@ -313,20 +407,34 @@ egrep -- '--- DIAGNOSTIC JOB :' %(rep_trav)s/%(nomjob)s.$num_job.fort.6.%(m)s |
         os.chmod(fbtc, 0755)
         time.sleep(0.5)
 
+    def change_btc(self, fbtc):
+        """Change a provided btc script file"""
+        # use the current jobid
+        content = open(fbtc, 'r').read()
+        exp = re.compile('^(num_job=.*)$', re.MULTILINE)
+        if self.mode == 'interactif':
+            pid = self.pid
+        else:
+            pid = self.scheduler.bjid
+        content = exp.sub('num_job=%s' % pid, content)
+        # update the file
+        open(fbtc, 'w').write(content)
+
+
     def soumbtc_batch(self, fbtc):
         """Run btc script in batch mode.
         """
-        from asrun.batch import BatchSystemFactory
-        scheduler = BatchSystemFactory(self.run, self.prof, fbtc)
-        iret, jobid, queue = scheduler.start()
+        self.scheduler.set_subpara(fbtc, change_input_script=True)
+        iret, jobid, queue = self.scheduler.start()
         if iret != 0:
             print3(_(u"Error during submitting job !"))
             iret = 5
         if jobid == '':
-            print3(_(u"Unexpected jobid"))
+            print3(_(u"Empty jobid"))
             iret = 5
         return iret, jobid, queue
 
+
     def soumbtc_interactif(self, fbtc):
         """Run btc in interactive mode.
         """
@@ -338,16 +446,14 @@ egrep -- '--- DIAGNOSTIC JOB :' %(rep_trav)s/%(nomjob)s.$num_job.fort.6.%(m)s |
         node = self.dict_info['node']
         dico = {
             'fbtc'   : fbtc,
-            'output' : self.flash('o'),
-            'error'  : self.flash('e'),
+            'output' : self.flash('output'),
+            'error'  : self.flash('error'),
         }
         # follow output or not
-        xterm = self.prof['xterm'][0]
-        if xterm == '' and self.prof['follow_output'][0] in YES_VALUES:
+        xterm = ''
+        if self.prof['follow_output'][0] in YES_VALUES:
             xterm   = self.run['terminal']
-            display = self.prof['display'][0] or ':0.0'
-            xterm = xterm.replace('@D', display)
-        
+
         if self.prof['depart'][0] != '' or xterm == '':
             cmd = cmd_batch % dico
         else:
@@ -360,10 +466,10 @@ egrep -- '--- DIAGNOSTIC JOB :' %(rep_trav)s/%(nomjob)s.$num_job.fort.6.%(m)s |
             if re.search('@E', xterm) == None:
                 xterm = xterm + ' -e @E'
             cmd = xterm.replace('@E', '%s "%s"' % (shell_cmd, cmd))
-        
+
         # run on another node
-        distant = not is_localhost(node)
-        
+        distant = not is_localhost2(node)
+
         if not distant:
             # check xterm command
             if xterm != '':
@@ -379,13 +485,21 @@ egrep -- '--- DIAGNOSTIC JOB :' %(rep_trav)s/%(nomjob)s.$num_job.fort.6.%(m)s |
                 print3(_(u"Connection failure to %s (from %s)") % (node, self.prof['serveur'][0]))
                 return 6, '', 'unknown'
 
-        kret, output = self.run.Shell(cmd, mach=node, bg=True)
+        # background is not possible if display forwarding is required
+        # (the connection must stay alive)
+        need_display = xterm != ''
+        background = (not need_display) \
+            or same_hosts2(self.prof['mclient'][0], node,
+                           self.prof['uclient'][0], self.prof['username'][0])
+        kret, output = self.run.Shell(cmd, mach=node, bg=background,
+                                    display_forwarding=need_display)
 
         return 0, self.jobid, 'interactif'
 
     def start(self, options=''):
         """Go !
         """
+        self.is_starting()
         self.run.DBG('Profile to run', self.prof.get_content(), all=True)
         # ----- copy and read .export, build dict for formatting
         self.build_dict_info(options)
@@ -396,12 +510,12 @@ egrep -- '--- DIAGNOSTIC JOB :' %(rep_trav)s/%(nomjob)s.$num_job.fort.6.%(m)s |
 
         jn = self.pid
         self.name = self.prof['nomjob'][0]
-        
+
         if self.run.get(self.mode) not in YES_VALUES:
             print3(ufmt(_(u"the configuration file (%s) does not allow mode='%s'"),
                         osp.join(confdir, "asrun"), self.mode))
             return 4, ""
-        
+
         # export file is not necessary in interactive mode
         if self.mode == 'batch':
             self.prof.WriteExportTo(self.prof.get_filename())
@@ -413,16 +527,16 @@ egrep -- '--- DIAGNOSTIC JOB :' %(rep_trav)s/%(nomjob)s.$num_job.fort.6.%(m)s |
             fbtc = get_tmpname(self.run, self.run['tmp_user'],
                                basename=self.prof['nomjob'][0] + '.u')
             os.environ['ASTK_MESSAGE'] = repr(self.dict_info['message'])
-            
+
             cmd = self.as_exec_ref + ' ' + self.prof.get_filename() + ' ' + fbtc
             iret, output = self.run.Shell(cmd, follow_output=True)
-            
+
             self.run.DBG('OUTPUT of as_exec_ref :', output, all=True)
-            
-            self.jobid, self.queue = parse_submission_result(output)
+
+            self.jobid, self.queue, self.stuyid = parse_submission_result(output)
             if osp.exists(fbtc):
-                jret = self.run.Rename(fbtc, self.flash('u'))
-            jret = self.run.Copy(self.flash('p'), self.prof.get_filename())
+                jret = self.run.Rename(fbtc, self.flash('script'))
+            jret = self.run.Copy(self.flash('export'), self.prof.get_filename())
             return iret, output
 
         # ----- consbtc ?
@@ -430,6 +544,7 @@ egrep -- '--- DIAGNOSTIC JOB :' %(rep_trav)s/%(nomjob)s.$num_job.fort.6.%(m)s |
         if self.prof['consbtc'][0] in NO_VALUES and not 'make_env' in self.prof['actions']:
             fbtc0 = self.prof.Get('D', 'btc')[0]['path']
             iret = self.run.Copy(fbtc, fbtc0)
+            self.change_btc(fbtc)
         else:
             self.consbtc(fbtc)
 
@@ -443,91 +558,73 @@ egrep -- '--- DIAGNOSTIC JOB :' %(rep_trav)s/%(nomjob)s.$num_job.fort.6.%(m)s |
             if iret == 0:
                 self.jobid = jobid
 
-            # copy fbtc into flasheur/
-            jret = self.run.Copy(self.flash('u'), fbtc)
+            # copy fbtc into flasheur/ (already removed if run in foreground)
+            if osp.exists(fbtc):
+                jret = self.run.Copy(self.flash('script'), fbtc, niverr='<A>_ALARM')
 
         # faut-il recopier le btc vers le client
         res_fbtc = self.prof.Get('R', 'btc')
         if len(res_fbtc) > 0:
             res_fbtc = res_fbtc[0]['path']
             self.run.Copy(res_fbtc, fbtc)
+            print3("BTCFILE=%s" % res_fbtc)
 
         return iret, ''
 
 
+    def flash(self, typ, num_job=None):
+        """If typ='o', return something like .../flasheur/nomjob.o1234"""
+        assert self.prof is not None
+        if num_job is None:
+            num_job = self.jobid
+        return flash_filename(magic.run['flasheur'], self.prof['nomjob'][0], num_job, typ)
+
+
     def get_state(self):
-        """Return current state of the job.
-        """
-        return Func_actu(self.run, self.jobid, self.name, self.mode)
-
-
-    def tail(self, nbline=50, expression=None):
-        """Return tail of current output of the job.
-        """
-        return Func_tail(self.run, self.jobid, self.name, self.mode, nbline, expression)
-
-
-    def wait(self, refresh_delay=1.):
-        """Wait for job completion.
-        """
-        state = '_'
-        while state != 'ENDED':
-            time.sleep(refresh_delay)
-            res = self.get_state()
-            state, diag = res[0:2]
-        self.diag = diag
+        """Return current state of the job."""
+        res = Func_actu(self.run, self.jobid, self.name, self.mode)
+        self.finish(res[0])
+        return res
 
 
     def get_diag(self):
-        """Return diagnostic of the execution based on the output file.
-        """
+        """Return diagnostic of the execution based on the output file."""
         res = self.get_state()
         state, diag = res[0:2]
         res = [diag, 0., 0., 0., 0.]
         self.diag = diag
-        if state == 'ENDED' and osp.exists(self.flash('o')):
-            txt = open(self.flash('o'), 'r').read()
+        if state == 'ENDED' and osp.exists(self.flash('output')):
+            txt = open(self.flash('output'), 'r').read()
             mat = re.search('%s +([0-9\.]+) +([0-9\.]+) +([0-9\.]+) +([0-9\.]+)' % 'Total', txt)
             if mat is not None:
                 res = res[:1] + [float(v) for v in mat.groups()]
         return res
 
+
+    def tail(self, nbline=50, expression=None):
+        """Return tail of current output of the job."""
+        return Func_tail(self.run, self.jobid, self.name, self.mode, nbline, expression)
+
+
     def kill(self):
-        """Kill the job (if it is running) and delete of its files.
-        """
+        """Kill the job (if it is running) and delete of its files."""
         Del(self.run, self.jobid, self.name, self.mode, signal='KILL')
 
-    def request(self, key):
-        """Return the value to request.
-        """
-        if key == 'cpu':
-            res = max(int(self.prof['ncpus'][0] or 1), int(self.prof['mpi_nbcpu'][0] or 1))
-        elif key == 'mem':
-            res = float(self.prof['memjob'][0] or 0.) / 1024.
-        else:
-            res = 0
-        return res
-
-    def on_host(self, serv, host):
-        """Change submission serv/host.
-        """
-        if serv is not None:
-            self.prof['serveur'] = serv
-        if host is not None:
-            self.prof['noeud'] = host
 
 
-
-class ASTER_TESTCASE(ASTER_CALCUL):
+class AsterCalcTestcase(AsterCalcul):
     """Derivation for a testcase.
     """
     def __init__(self, run, test, filename=None, prof=None, pid=None, **kwargs):
-        """Initializations
-        """
-        ASTER_CALCUL.__init__(self, run, filename, prof, pid)
+        """Initializations """
+        if prof is not None:
+            prof['parent'] = 'astout'
+        AsterCalcul.__init__(self, run, filename, prof, pid, differ_init=True)
         self.testcase = test
         self.param = kwargs.copy()
         self.prof = self.change_profile()
+        self.finalize_init()
+
 
     def change_profile(self):
         """Prepare profile object.
@@ -538,28 +635,34 @@ class ASTER_TESTCASE(ASTER_CALCUL):
                             basename=self.testcase + '.export', pid=self.pid)
         ptest.set_filename(fname)
         self.run.DBG('profile filename set to : ', fname)
-        del ptest['xterm']
         del ptest['follow_output']
         del ptest['rep_trav']
         del ptest['detr_rep_trav']
-        
+
         # update with the export to run the test
+        lunig = ptest.get_type('unig')
+        d_unig = None
+        if lunig:
+            unigest = lunig[0].path
+            conf = build_AsterConfig(self.run, ptest)
+            build = AsterBuild(self.run, conf)
+            d_unig = build.GetUnigest(unigest)
         prt = build_test_export(self.run, self.param['conf'], self.param['REPREF'],
                                 self.param['reptest'], self.testcase, self.param['resutest'],
-                                with_default=False)
+                                with_default=False, d_unig=d_unig)
         prt['nomjob'] = self.testcase
         ptest.update(prt)
-        
+
         # apply facmtps
         try:
-            ptest['tpsjob'] = int(ptest['tpsjob'][0])  * self.param['facmtps']
+            ptest['tpsjob'] = int(float(ptest['tpsjob'][0]))  * self.param['facmtps']
         except Exception:
             pass
         try:
-            ptest.args['tpmax'] = int(ptest.args['tpmax'])  * self.param['facmtps']
+            ptest.args['tpmax'] = int(float(ptest.args['tpmax']))  * self.param['facmtps']
         except Exception:
             pass
-        
+
         ptest.update_content()
         return ptest
 
@@ -574,7 +677,7 @@ class ASTER_TESTCASE(ASTER_CALCUL):
 
 
 
-class ASTER_PARAMETRIC(ASTER_CALCUL):
+class AsterCalcParametric(AsterCalcul):
     """Derivation for a parametric study :
         - change all "comm" files
         - change repe_out into resudir/calc_000i
@@ -584,7 +687,7 @@ class ASTER_PARAMETRIC(ASTER_CALCUL):
         """
         if prof is not None:
             prof['parent'] = 'parametric'
-        ASTER_CALCUL.__init__(self, run, filename, prof, pid)
+        AsterCalcul.__init__(self, run, filename, prof, pid, differ_init=True)
         self.label = label
         self.resudir = kwargs['resudir']
         try:
@@ -594,6 +697,8 @@ class ASTER_PARAMETRIC(ASTER_CALCUL):
         self.values   = kwargs['values'].copy()
         self.keywords = kwargs['keywords'].copy()
         self.prof = self.change_profile()
+        self.finalize_init()
+
 
     def change_profile(self):
         """Prepare profile object.
@@ -607,7 +712,6 @@ class ASTER_PARAMETRIC(ASTER_CALCUL):
         prof['nomjob']  = '%s-%s' % (self.prof['nomjob'][0], self.label)
         # delete unused entries
         assert prof.Get('R', typ='repe')
-        del prof['xterm']
         del prof['follow_output']
         del prof['rep_trav']
         del prof['detr_rep_trav']
@@ -659,7 +763,7 @@ class ASTER_PARAMETRIC(ASTER_CALCUL):
                 continue
             dicf['path'] = osp.join(self.resudir, self.label, osp.basename(dicf['path']))
         return prof
-        
+
     def change_comm_files(self, filename):
         """Change parameter definition in comm file.
         """
@@ -719,12 +823,29 @@ def init_profil_from(run, prof, keep_surch=True):
 def parse_submission_result(txt):
     """Decode a such string 'JOBID=  232564  QUEUE=  q4G_1h'.
     """
-    # must be consistency with i_serv.tcl
-    mat = re.search('JOBID *= *([0-9a-zA-Z\._\-]+) *QUEUE *= *(.*)', txt)
+    # must be consistent with i_serv.tcl
+    jobid, queue = '?', 'unknown'
+    mat = re.search('JOBID *= *([0-9a-zA-Z\._\-]+)', txt)
     if mat is not None:
-        jobid, queue = mat.groups()
-        queue = queue.split()[0]
-    else:
-        jobid, queue = '?', 'unkwown'
-    return jobid, queue
+        jobid = mat.group(1)
+    mat = re.search('QUEUE *= *([0-9a-zA-Z\._\-]+)', txt)
+    if mat is not None:
+        queue = mat.group(1)
+    studyid = jobid
+    mat = re.search('STUDYID *= *([0-9a-zA-Z\._\-]+)', txt)
+    if mat is not None:
+        studyid = mat.group(1)
+    return jobid, queue, studyid
 
+
+def parse_consbtc(txt):
+    """Decode the string to find the btc script filename."""
+    btc = None
+    mat = re.search('BTCFILE *= *(.*)$', txt, re.MULTILINE)
+    if mat is not None:
+        btc = mat.group(1)
+    return btc
+
+ASTER_CALCUL = bwc_deprecate_class('ASTER_CALCUL', AsterCalcul)
+ASTER_TESTCASE = bwc_deprecate_class('ASTER_TESTCASE', AsterCalcTestcase)
+ASTER_PARAMETRIC = bwc_deprecate_class('ASTER_PARAMETRIC', AsterCalcParametric)
