@@ -22,7 +22,6 @@
 Some utilities for asrun using only standard Python modules.
 """
 
-
 import sys
 import os
 import os.path as osp
@@ -59,8 +58,8 @@ class Enum(object):
         """Constructor"""
         self._dict_keys = {}
         for inum, key in enumerate(keys):
-            setattr(self, key, inum)
-            self._dict_keys[inum] = key
+            setattr(self, key, 2**inum)
+            self._dict_keys[2**inum] = key
 
     def exists(self, value):
         """Tell if value is in the enumeration"""
@@ -85,7 +84,6 @@ def _timefmt():
     else:
         return "%I:%M:%S"
 
-
 def now(datefmt=None, timefmt=None, sep=" "):
     if datefmt is None:
         datefmt = _datefmt()
@@ -100,6 +98,29 @@ def now(datefmt=None, timefmt=None, sep=" "):
         fmt += timefmt
     return time.strftime(fmt)
 
+def hms2s(vtime):
+    """Convert a time given in the format 'H:M:S' in seconds.
+    Raise ValueError if fails."""
+    secs = 0
+    stps = vtime.split(':')
+    m = 1
+    while len(stps) > 0:
+        secs += int(stps.pop()) * m
+        m = m * 60
+    if len(stps) > 3:
+        raise ValueError(u"invalid time value : '%s'" % vtime)
+    return secs
+
+def dhms2s(vtime, sep='-'):
+    """Convert a time given in the format 'D-H:M:S' in seconds."""
+    if type(vtime) in (list, tuple):
+        return sum([dhms2s(vt) for vt in vtime])
+    secs = 0
+    stps = vtime.split(sep)
+    secs = hms2s(stps.pop())
+    if len(stps) > 0:
+        secs += int(stps[0]) * 86400
+    return secs
 
 _encoding = None
 def get_encoding():
@@ -124,6 +145,19 @@ def get_language():
     return lang
 
 
+_val_pid = 0
+def get_unique_id(jobid, num=None):
+    """Return an identifier (as a pid) based on `jobid`.
+    Counter is incremented at each call.
+    """
+    global _val_pid
+    if num is None:
+        _val_pid += 1
+        num = _val_pid
+    assert type(num) in (int, long), u'get_unique_pid: integer argument required!'
+    return '%04d-%s' % (num, jobid)
+
+
 def _listsurcharge_u2mess(fsurch, lf):
     """Write the fortran subroutine "surchg.f".
         fsurch : subroutine filename
@@ -131,10 +165,10 @@ def _listsurcharge_u2mess(fsurch, lf):
     """
     header = """
       SUBROUTINE SURCHG(IFM)
-C --- IFM CONSERVE POUR LA COMPATIBILITE
+C --- IFM CONSERVE POUR LA COMPATIBILITE (DUMMY ARGUMENT)
       INTEGER IFM
-      INTEGER NBFILE
-      CHARACTER*16  VALK(%(nbfile)s)"""
+      CHARACTER*80  VALK(%(nbfile)s)
+      IFM = 1"""
     fmt_line = "      VALK(%(indice)d) = '%(subroutine)s%(separ)s'"
     fmt_call = "      CALL U2MESK('A','SUPERVIS_40',%(nbfile)s,VALK)"
     footer = "      END"
@@ -149,10 +183,10 @@ C --- IFM CONSERVE POUR LA COMPATIBILITE
         if i + 1 == len(l_file):
             sep = ''
         cont.append(fmt_line % { 'indice' : i + 1, 'subroutine' : filename, 'separ' : sep })
-    
+
     cont.append(fmt_call % { 'nbfile' : nbfile })
     cont.append(footer)
-    
+
     open(fsurch, 'w').write(os.linesep.join(cont))
     #print os.linesep.join(cont)
 
@@ -241,7 +275,7 @@ def get_list(ftest, unique=True):
     except (OSError, IOError):
         iret = 4
         return iret, ltest
-    
+
     lfich = [t for t in [re.sub(' *[#%]+.*', '', l).strip() for l in line] if t != '']
     if unique:
         # delete twins (by keeping ordering)
@@ -251,7 +285,7 @@ def get_list(ftest, unique=True):
                 ltest.append(t)
     else:
         ltest = lfich
-    
+
     return iret, ltest
 
 
@@ -268,14 +302,14 @@ def getpara(fpara, platform=None, others=[]):
     from asrun.common.i18n import _
     list_param = list_para_test + others
     dtrans = {
-        'mpi_nbnoeud' : 'nnoeud_mpi',
-        'mpi_nbcpu' : 'nproc_mpi',
+        'nnoeud_mpi' : 'mpi_nbnoeud',
+        'nproc_mpi' : 'mpi_nbcpu',
     }
-    supported_param = list_param + dtrans.values()
+    supported_param = list_param + dtrans.keys()
     iret, err = 0, ''
     dico = {}
     for k in list_param:
-        dico[k] = 0
+        dico[k] = '0'
 
     # ----- check that file exists
     if not os.path.exists(fpara):
@@ -287,7 +321,7 @@ def getpara(fpara, platform=None, others=[]):
     f = open(fpara, 'r')
     for str0 in f:
         line = str0.split()
-        while len(line)>0:
+        while len(line) > 0:
             w = line.pop(0)
             if w in supported_param:
                 dico[dtrans.get(w, w)] = line.pop(0)
@@ -327,14 +361,21 @@ def getpara(fpara, platform=None, others=[]):
         return iret, {}, err
     if dico['mem_aster'] == 0:
         dico['mem_aster'] = 100
-    
+
     if platform is not None:
         if re.search('64$', platform):
             facW = 8
         else:
             facW = 4
         dico['memjeveux'] = dico['mem_job'] / facW * dico['mem_aster'] / 100.
-    
+
+    # number of cpus, nodes must be >= 1
+    for key in ('ncpus', 'mpi_nbnoeud', 'mpi_nbcpu'):
+        val = dico[key]
+        if val.isdigit() and int(val) > 1:
+            pass
+        else:
+            dico[key] = '1'
     return iret, dico, err
 
 
@@ -382,6 +423,36 @@ def remove_empty_dirs(fromdir):
         except:
             pass
 
+def renametree(src, dst):
+    """Rename a entire tree 'src' to or into 'dst'."""
+    if not osp.exists(dst):
+        os.rename(src, dst)
+    elif osp.isfile(dst) != osp.isfile(src):
+        raise OSError("source and destination must be both files or directories")
+    elif osp.isfile(src):
+        # both are files
+        os.rename(src, dst)
+    else:
+        # src and dst are directories
+        names = os.listdir(src)
+        for name in names:
+            srcname = osp.join(src, name)
+            dstname = osp.join(dst, name)
+            renametree(srcname, dstname)
+        os.rmdir(src)
+
+def get_tmpname_base(dirname=None, basename=None, user=None, node=None, pid=None):
+    """Return a name for a temporary directory (*not created*)
+    of the form : 'dirname'/user@machine-'basename'.'pid'
+    *Only* basename is not compulsory in this variant.
+    """
+    basename = basename or 'tmpname-%.6f' % time.time()
+    if pid == "auto":
+        pid = "pid-%.6f" % time.time()
+    root, ext = osp.splitext(basename)
+    name = '%s-%s-' % (user, node) + root + '.' + str(pid) + ext
+    return osp.join(dirname, name)
+
 
 def check_joker(filename, joker):
     """Returns if 'filename' verifies 'joker' : ("toto.comm", "com?") returns True
@@ -431,6 +502,7 @@ try:
 except ImportError:
     MIMEText = None
 
+MIMETextClass = MIMEText
 
 def find_command(content, command):
     """Return the lowest index in content where command is found and
@@ -474,12 +546,12 @@ def re_search(content, **kargs):
     flag = re.MULTILINE | kargs.get('flag', re.MULTILINE)
     what = kargs.get('result', 'bool')
     pattern = kargs.get('pattern', '%s')
-    
+
     if kargs.get('string'):
         pattern = pattern % re.escape(kargs['string'])
-    
+
     expr = re.compile(pattern, flag)
-    
+
     if what == 'bool':
         res = expr.search(content) != None
     elif what == 'number':
@@ -490,3 +562,36 @@ def re_search(content, **kargs):
         res = None
     return res
 
+
+def get_plugin(uri):
+    """Load and return a python object (class, function...).
+    Its `uri` looks like "mainpkg.subpkg.module.object", this means
+    that "mainpkg.subpkg.module" is imported and "object" is
+    the plugin object to return.
+    """
+    path = uri.split('.')
+    modname = '.'.join(path[:-1])
+    if len(modname) == 0:
+        raise ImportError(u"invalid plugin name: %s" % uri)
+    try:
+        __import__(modname)
+        mod = sys.modules[modname]
+        plugin_object = getattr(mod, path[-1])
+    except (ImportError, AttributeError), err:
+        raise ImportError(u"plugin not found: %s\n"
+            "Check the plugin name in 'etc/codeaster/asrun' configuration file\n"
+            "and it would be search in PYTHONPATH : %s" % (uri, sys.path))
+    return plugin_object
+
+
+def unique_basename(pathname):
+    """Return a basename which takes all the pathname in account
+    (so it should be unique)."""
+    bname = sha1(pathname).hexdigest() + '.' + osp.basename(pathname)
+    return bname
+
+__reg_prefix__ = re.compile('^[a-f0-9]{%d}\.' % (sha1('a').digest_size * 2), re.I)
+
+def unique_basename_remove(bname):
+    """Remove the prefix added by unique_basename."""
+    return __reg_prefix__.sub('', bname)
