@@ -17,10 +17,11 @@
 #    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
 # ==============================================================================
 
-"""Definition of ASTER_CONFIG class.
+"""Definition of AsterConfig class.
 """
 
 import os
+import os.path as osp
 import re
 
 from asrun.installation import aster_root
@@ -28,14 +29,16 @@ from asrun.common.i18n import _
 from asrun.mystring import print3, split_endlines, ufmt
 from asrun.common.utils import get_absolute_dirname
 
+from asrun.backward_compatibility import bwc_deprecate_class
 
-class ASTER_CONFIG:
+
+class AsterConfig:
     """Class to read a configuration file of a Code_Aster version ('config.txt')
     and give easy access to the parameters.
     """
     def __init__(self, config_file, run=None):
         """config_file : filename of the 'config.txt' file to read
-        run   : ASTER_RUN object (optional)
+        run   : AsterRun object (optional)
         """
         # ----- initialisations
         self.config = {}
@@ -44,15 +47,14 @@ class ASTER_CONFIG:
         self.verbose = False
         self.debug = False
 
-        # ----- reference to ASTER_RUN object which manages the execution
+        # ----- reference to AsterRun object which manages the execution
         self.run = run
         if run != None:
             self.verbose = run['verbose']
             self.debug = run['debug']
 
         # ----- set optionnal/defaults values (ALWAYS AS LIST !)
-        # keep 'Eficas' by default (instead of 'Python') to support older version
-        reppy = 'Eficas'
+        reppy = 'Python'
         self.config = {
             # name of source directories
             'SRCFOR'          : ['bibfor'],
@@ -71,10 +73,10 @@ class ASTER_CONFIG:
             'BINCMDE'         : ['commande'],
             'BINELE'          : ['elements'],
             'BINPICKLED'      : ['cata_ele.pickled'],
-            'BINLIB_NODBG'    : ['lib_obj/lib_aster.lib'], # new values are
-            'BINLIB_DBG'      : ['lib_dbg/lib_aster.lib'], #     lib/libaster.a and libasterd.a
-            'BINLIBF_NODBG'   : ['lib_obj/ferm.lib'],      # and lib/libferm.a  and libfermd.a
-            'BINLIBF_DBG'     : ['lib_dbg/ferm.lib'],
+            'BINLIB_NODBG'    : ['lib/libaster.a'],
+            'BINLIB_DBG'      : ['lib/libasterd.a'],
+            'BINLIBF_NODBG'   : ['lib/libferm.a'],
+            'BINLIBF_DBG'     : ['lib/libfermd.a'],
             'BINOBJ_NODBG'    : ['obj'],
             'BINOBJF_NODBG'   : ['obj_f'],
             'BINOBJ_DBG'      : ['dbg'],
@@ -83,27 +85,27 @@ class ASTER_CONFIG:
             # name of bibpyt in execution directory
             'REPPY'           : [reppy],
             # modules to compile elements
-            'MAKE_SURCH_OFFI' : [os.path.join('Lecture_Cata_Ele', 'make_surch_offi.py')],
-            'MAKE_CAPY_OFFI'  : [os.path.join('Lecture_Cata_Ele', 'make_capy_offi.py')],
+            'MAKE_SURCH_OFFI' : [osp.join('Lecture_Cata_Ele', 'make_surch_offi.py')],
+            'MAKE_CAPY_OFFI'  : [osp.join('Lecture_Cata_Ele', 'make_capy_offi.py')],
             # command line arguments for aster executable
-            'REPOUT'          : [os.path.join(aster_root, 'outils')],
-            'REPMAT'          : [os.path.join(self.dirn, 'materiau')],
-            'REPDEX'          : [os.path.join(self.dirn, 'datg')],
-            'ARGPYT'          : [os.path.join('Execution', 'E_SUPERV.py')],
-            'ARGEXE'          : ['-eficas_path '+reppy],
+            'REPOUT'          : [osp.join('$ASTER_ROOT', 'outils')],
+            'REPMAT'          : [osp.join(self.dirn, 'materiau')],
+            'REPDEX'          : [osp.join(self.dirn, 'datg')],
+            'ARGPYT'          : [osp.join('Execution', 'E_SUPERV.py')],
+            'ARGEXE'          : ['-eficas_path ' + reppy],
         }
-        
+
         # ----- read config file
-        if not os.path.isfile(config_file):
+        if not osp.isfile(config_file):
             self._mess(ufmt(_(u'file not found : %s'), config_file), '<F>_FILE_NOT_FOUND')
 
         f = open(config_file, 'r')
         content = f.read()
         f.close()
         self.config.update(self._parse(content))
-        
+
         if self.debug:
-            print3('<DBG> <init> ASTER_CONFIG :')
+            print3('<DBG> <init> AsterConfig :')
             print3(self)
 
     def __repr__(self):
@@ -142,7 +144,7 @@ class ASTER_CONFIG:
         """
         l_res = []
         for val in self[key]:
-            l_res.extend([os.path.join(self.dirn, path) for path in val.split()])
+            l_res.extend([osp.join(self.dirn, path) for path in val.split()])
         return l_res
 
     def get_defines(self):
@@ -174,6 +176,7 @@ class ASTER_CONFIG:
                     typ = re.sub('^[ ]*', '', re.sub('[ ]*$', '', typ)).strip()
                     val = re.sub('^[ ]*', '', re.sub('[ ]*$', '', val)).strip()
                     if val != '':
+                        val = osp.expandvars(val)
                         if cfg.has_key(typ):
                             cfg[typ].append(val)
                         else:
@@ -189,3 +192,22 @@ class ASTER_CONFIG:
             open(fich, 'w').write(self._content)
         except IOError, msg:
             self._mess(ufmt(_(u'No write access to %s'), fich), '<F>_ERROR')
+
+ASTER_CONFIG = bwc_deprecate_class('ASTER_CONFIG', AsterConfig)
+
+def build_AsterConfig(run, prof=None):
+    """Build an AsterConfig object from the default config file or the one
+    referenced in the profile."""
+    from asrun.common_func  import get_tmpname
+    lconf = prof and prof.Get('D', typ='conf')
+    if not lconf:
+        ficconf = os.path.join(prof.get_version_path(), 'config.txt')
+    else:
+        ficconf = lconf[0]['path']
+        if run.IsRemote(ficconf):
+            ficconf = get_tmpname(run, run['tmp_user'], basename='config.txt')
+            run.ToDelete(ficconf)
+            kret = run.Copy(ficconf, lconf[0]['path'])
+        else:
+            ficconf = run.PathOnly(ficconf)
+    return AsterConfig(ficconf, run)
