@@ -27,19 +27,20 @@ import os
 import os.path as osp
 import re
 import cPickle
+import time
 from glob import glob
 
 from asrun.installation import aster_root
 from asrun.common.i18n import _
 from asrun.mystring     import print3, ufmt
-from asrun.config       import ASTER_CONFIG
-from asrun.build        import ASTER_BUILD, unigest2dict
+from asrun.config       import AsterConfig
+from asrun.build        import AsterBuild, unigest2dict
 from asrun.progress     import Progress
 
 
 class CataMessageError(Exception):
     """Error with a catalog of messages."""
-    
+
     def __init__(self, fcata, reason):
         self.reason = reason
         self.fcata = fcata
@@ -51,7 +52,7 @@ class CataMessageError(Exception):
 
 class MessageError(Exception):
     """Error on a message."""
-    
+
     def __init__(self, msgid, reason):
         self.reason = reason
         self.msgid = msgid
@@ -64,12 +65,14 @@ class MessageError(Exception):
 class MESSAGE_MANAGER(object):
     """Classe pour récupérer des informations sur le catalogue de messages.
     """
-    def __init__(self, repref, fort=('bibfor', 'bibf90'), pyt='bibpyt',
-                      cache_dict=None, verbose=True, force=False,
-                      ignore_comments=True, debug=False,
-                      surch_fort=[], surch_pyt=[], unig=None):
-        """Initialisations
-        """
+    def __init__(self, repref,
+                 fort=('bibfor', 'bibf90'),
+                 pyt='bibpyt',
+                 capy='catapy',
+                 cache_dict=None, verbose=True, force=False,
+                 ignore_comments=True, debug=False,
+                 surch_fort=[], surch_pyt=[], unig=None):
+        """Initialisations."""
         if type(fort) not in (list, tuple):
             fort = [fort,]
         if type(surch_fort) not in (list, tuple):
@@ -79,6 +82,7 @@ class MESSAGE_MANAGER(object):
         self.repref          = repref
         self.fort            = [osp.join(repref, rep) for rep in fort]
         self.pyt             = osp.join(repref, pyt)
+        self.capy            = osp.join(repref, capy)
         self.surch_fort      = [osp.abspath(d) for d in surch_fort]
         self.surch_pyt       = [osp.abspath(d) for d in surch_pyt]
         self.verbose         = verbose
@@ -87,19 +91,21 @@ class MESSAGE_MANAGER(object):
         self.message_dir     = 'Messages'
         self.cache_dict      = cache_dict or osp.join('/tmp', 'messages_dicts.pick')
         self.ignore_comments = ignore_comments
-        self.wrk_fort        = osp.join(os.getcwd(), 'F')
-        self.wrk_pyt         = osp.join(os.getcwd(), 'PY')
-        
+        self.wrk_fort        = osp.join(os.getcwdu(), 'F')
+        self.wrk_pyt         = osp.join(os.getcwdu(), 'PY')
+
         self.unig            = unig
-        # init ASTER_CONFIG, ASTER_BUILD objects
+        # init AsterConfig, AsterBuild objects
         fconf = osp.join(self.repref, 'config.txt')
-        self.conf_obj  = ASTER_CONFIG(fconf, run=None)
-        self.build_obj = ASTER_BUILD(run=None, conf=self.conf_obj)
+        self.conf_obj  = AsterConfig(fconf, run=None)
+        self.build_obj = AsterBuild(run=None, conf=self.conf_obj)
         if unig:
-            self.unig = unigest2dict(unig, self.conf_obj)
-        
+            # use with_fordepla=False not to mark moved files as removed
+            self.unig = unigest2dict(unig, self.conf_obj, with_fordepla=True)
+
         if self.verbose:
-            print3('Repertoires :\n   REF=%s\n   FORTRAN=%s\n   PYTHON=%s' % (self.repref, self.fort, self.pyt))
+            print3('Repertoires :\n   REF=%s\n   FORTRAN=%s\n   PYTHON=%s\n   CAPY=%s' \
+                % (self.repref, self.fort, self.pyt, self.capy))
 
         self.l_cata            = self._get_list_cata()
         self.l_src, self.l_pyt = self._get_list_src()
@@ -198,12 +204,12 @@ class MESSAGE_MANAGER(object):
         if error_occurs:
             raise CataMessageError(_(u"global"), _(u"errors occurred!"))
         all_msg = set(all_msg)
-        
+
         # messages jamais appelés
         used = set(self.d_msg_call.keys())
         unused =  list(all_msg.difference(used))
         unused.sort()
-        
+
         union = used.union(all_msg)
         not_found = list(used.difference(all_msg))
         not_found.sort()
@@ -277,7 +283,7 @@ class MESSAGE_MANAGER(object):
                 if self.verbose: print3('suppression :', f)
             if iunig > 0:
                 print3('%5d source(s) supprime(s).' % iunig)
-        
+
         s_f.difference_update(s_suppr)
         s_f.update(s_surch)
         l_f = [osp.abspath(f) for f in s_f]
@@ -291,9 +297,12 @@ class MESSAGE_MANAGER(object):
         s_pyt = set()
         s_pyt.update(glob(osp.join(self.pyt, '*.py')))
         s_pyt.update(glob(osp.join(self.pyt, '*', '*.py')))
+        s_pyt.update(glob(osp.join(self.capy, '*.capy')))
+        s_pyt.update(glob(osp.join(self.capy, '*', '*.capy')))
         d_py = {}
         for f in s_pyt:
-            key = self.build_obj.GetCModif('py', f)
+            typ = osp.splitext(f)[-1][1:]
+            key = self.build_obj.GetCModif(typ, f)
             assert d_py.get(key) is None, 'ERROR : %s  (old : %s)' % (key, d_py[key])
             d_py[key] = f
         # surcharge
@@ -301,6 +310,8 @@ class MESSAGE_MANAGER(object):
         for dsrc in self.surch_pyt:
             s_surch.update(glob(osp.join(dsrc, '*.py')))
             s_surch.update(glob(osp.join(dsrc, '*', '*.py')))
+            s_surch.update(glob(osp.join(dsrc, '*.capy')))
+            s_surch.update(glob(osp.join(dsrc, '*', '*.capy')))
         if len(s_surch) > 0:
             l_surch = list(s_surch)
             l_surch.sort()
@@ -309,7 +320,8 @@ class MESSAGE_MANAGER(object):
         # retirer des sources originaux ceux de la surcharge...
         s_suppr = set()
         for f in s_surch:
-            key = self.build_obj.GetCModif('py', f)
+            typ = osp.splitext(f)[-1][1:]
+            key = self.build_obj.GetCModif(typ, f)
             fexist = d_py.get(key)
             if fexist:
                 s_suppr.add(fexist)
@@ -317,12 +329,13 @@ class MESSAGE_MANAGER(object):
         # ... et ceux de l'unigest
         if self.unig:
             iunig = 0
-            for f in self.unig['py']:
-                iunig += 1
-                fabs = osp.abspath(osp.join(self.repref, f))
-                key = self.build_obj.GetCModif('py', fabs)
-                s_suppr.add(d_py.get(key, ''))
-                if self.verbose: print3('suppression :', fabs)
+            for typ in ('py', 'capy'):
+                for f in self.unig[typ]:
+                    iunig += 1
+                    fabs = osp.abspath(osp.join(self.repref, f))
+                    key = self.build_obj.GetCModif(typ, fabs)
+                    s_suppr.add(d_py.get(key, ''))
+                    if self.verbose: print3('suppression :', fabs)
             if iunig > 0:
                 print3('%5d module(s) python supprime(s).' % iunig)
 
@@ -343,11 +356,11 @@ class MESSAGE_MANAGER(object):
             return []
         ext = osp.splitext(fich)[-1]
         txt = clean_source(txt, ext, ignore_comments=self.ignore_comments, wrap=True)
-        
-        if osp.splitext(fich)[-1] != '.py':
-            expr  = 'CALL\s+(U2ME|UTEXC).*?\((.*?),[ \'\"]+(.*?)[\'\" ]+[,\)]+'
+
+        if osp.splitext(fich)[-1] not in ('.py', '.capy'):
+            expr  = 'CALL\s+(U2MES.|UTEXC[MP]+|UTPRIN)\s*\(([^,]*?), *[\'\"]+(.*?)[\'\"]+ *[,\)]+'
         else:
-            expr = '(UTMESS|GetText)\s*\((.*?).*?,[ \'\"]+(.*?)[\'\" ]+[,\)]+'
+            expr = '[\s\.:]+(UTMESS|GetText)\s*\(([^,]*?), *[\'\"]+(.*?)[\'\"]+ *[,\)]+'
         all_msg = re.findall(expr, txt)
         l_msg = []
         for found in all_msg:
@@ -355,7 +368,7 @@ class MESSAGE_MANAGER(object):
             if msg.startswith('FERMETUR_'):
                 pass
             elif re.search('^[A-Z0-9]+_[0-9]+$', msg) is None:
-                print3("Invalid message id ('%s') in file %s" % (msg, fich))
+                print3("Invalid message id ('%s') in file %s (%s)" % (msg, fich, sub))
             else:
                 mattyp = re.search('[\'\" ]+(.*?)[\+\'\" ]+', typ)
                 # variables and numbers (for exception) are allowed
@@ -364,10 +377,11 @@ class MESSAGE_MANAGER(object):
                    # may be '' for example STY//'+'
                     print3("Invalid message type (%s) for message '%s' in file %s" \
                         % (mattyp.group(1), msg, fich))
+                    print3("type = %s" % typ)
                 else:
                     l_msg.append(msg)
 #      l_msg = self.regexp.findall(txt)
-        
+
         # verif
         l_res = []
         for msg in l_msg:
@@ -375,7 +389,7 @@ class MESSAGE_MANAGER(object):
             assert len(spl) == 2, 'ERROR invalid : %s' % msg
             msg = '%s_%d' % (spl[0], int(spl[1]))
             l_res.append(msg)
-        
+
         return l_res
 
 
@@ -394,7 +408,7 @@ class MESSAGE_MANAGER(object):
             self.d_msg_call = cPickle.load(pick)
             pick.close()
 
-        else:   
+        else:
             self.d_msg_used = {}
             self.d_msg_call = {}
             l_all = self.l_src + self.l_pyt
@@ -411,11 +425,11 @@ class MESSAGE_MANAGER(object):
                     self.d_msg_used[key] = lm
                     for msg in lm:
                         self.d_msg_call[msg] = self.d_msg_call.get(msg, []) + [key,]
-            
+
             if self.verbose:
                 p.End()
                 #pprint.pprint(self.d_msg_used)
-            
+
             pick = open(self.cache_dict, 'w')
             cPickle.dump(self.d_msg_used, pick)
             cPickle.dump(self.d_msg_call, pick)
@@ -438,7 +452,7 @@ class MESSAGE_MANAGER(object):
         # catalogue objects
         old_f, old_num = self._id2filename(oldid)
         new_f = self._filename_cata(dest.lower())
-        
+
         # have these catalogues been already changed ?
         fmod = osp.join(self.wrk_pyt, osp.basename(old_f))
         if osp.isfile(fmod):
@@ -448,7 +462,7 @@ class MESSAGE_MANAGER(object):
         if osp.isfile(fmod):
             print3('from %s' % fmod)
             new_f = fmod
-            
+
         old_cata = CATA(old_f)
         new_cata = CATA(new_f)
         if self.verbose:
@@ -458,11 +472,11 @@ class MESSAGE_MANAGER(object):
         if new_num < 0:
             raise CataMessageError(new_f, _(u'no message id available in this catalog'))
         newid = self._filename2id(new_f, new_num)
-        
+
         # check message existence
         if old_cata[old_num] == None:
             raise MessageError(oldid, _(u'unknown message'))
-        
+
         new_cata[new_num] = old_cata[old_num]
         del old_cata[old_num]
 
@@ -475,7 +489,7 @@ class MESSAGE_MANAGER(object):
         content = new_cata.get_content()
         open(fout, 'w').write(content)
         print3('Nouveau fichier : %s' % fout)
-        
+
         # modify source using 'oldid' message
         l_src = self.d_msg_call.get(oldid, [])
         for f in l_src:
@@ -527,25 +541,25 @@ class MESSAGE_MANAGER(object):
 template_cata_header = """# -*- coding: utf-8 -*-
 #            CONFIGURATION MANAGEMENT OF EDF VERSION
 # ======================================================================
-# COPYRIGHT (C) 1991 - 2007  EDF R&D                  WWW.CODE-ASTER.ORG
-# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY  
-# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY  
-# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR     
-# (AT YOUR OPTION) ANY LATER VERSION.                                                  
-#                                                                       
-# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT   
-# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF            
-# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU      
-# GENERAL PUBLIC LICENSE FOR MORE DETAILS.                              
-#                                                                       
-# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE     
-# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,         
-#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.        
+# COPYRIGHT (C) 1991 - %s  EDF R&D                  WWW.CODE-ASTER.ORG
+# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
+# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
+# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
+# (AT YOUR OPTION) ANY LATER VERSION.
+#
+# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
+# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
+# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
+# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
+#
+# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
+# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
+#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
 # ======================================================================
 
 def _(x) : return x
 
-cata_msg = {"""
+cata_msg = {""" % time.strftime('%Y')
 
 template_cata_footer = """
 }
@@ -570,7 +584,7 @@ class CATA:
         self.fcata = fcata
         self.cata_msg = {}
         try:
-            d = {}
+            d = { '_' : lambda x: x, }
             execfile(fcata, d)
             self.cata_msg = d['cata_msg']
         except Exception, msg:
@@ -647,11 +661,13 @@ class CATA:
         def_args = {}
         for i in range(1,100):
             def_args['i%d' % i] = 99999999
-            def_args['r%d' % i] = 9.9999E99
+            def_args['r%d' % i] = 1.234e16   # not too big to avoid python issue1742669
             def_args['k%d' % i] = 'xxxxxx'
         def_args['ktout'] = "all strings !"
         error = []
         for num, msg in self.cata_msg.items():
+            if type(msg) is dict:
+                msg = msg['message']
             try:
                 txt = msg % def_args
             except:
@@ -692,10 +708,10 @@ def clean_source(content, ext, ignore_comments, wrap):
     """Nettoie un fichier source.
     """
     if ignore_comments:
-        assert ext in ('.f', '.F', '.py'), 'unknown type : %s' % str(ext)
+        assert ext in ('.f', '.F', '.py', '.capy'), 'unknown type : %s' % str(ext)
         if ext in ('.f', '.F'):
             reg_ign = re.compile('(^[A-Z!#].*$)', re.MULTILINE)
-        elif ext == '.py':
+        elif ext in ('.py', '.capy'):
             reg_ign = re.compile('(#.*$)', re.MULTILINE)
         content = reg_ign.sub('', content).expandtabs()
         if wrap and ext in ('.f', '.F'):
@@ -713,7 +729,7 @@ def GetMessageInfo(run, *args):
         fconf = osp.join(REPREF, 'config.txt')
     else:
         fconf = osp.abspath(run['config'])
-    conf = ASTER_CONFIG(fconf, run)
+    conf = AsterConfig(fconf, run)
 
     if run['nolocal']:
         run.Mess(_(u'This operation only works on local source files. "--nolocal" option ignored'))
@@ -729,13 +745,13 @@ def GetMessageInfo(run, *args):
     args = list(args)
     named_actions = ('check', 'move',)
     action = None
-    
+
     if len(args) > 0 and args[0] in named_actions:
         action = args.pop(0)
     elif len(args) == 0:
         run.Mess(_(u'You must choose an operation from %s or give a subroutine name or a message ID') \
                 % repr(named_actions), '<F>_INVALID_ARGUMENT')
-    
+
     # surcharge
     surch_fort = run.get('surch_fort', [])
     if surch_fort:
@@ -743,8 +759,8 @@ def GetMessageInfo(run, *args):
     surch_pyt = run.get('surch_pyt', [])
     if surch_pyt:
         surch_pyt = surch_pyt.split(',')
-    
-    msgman = MESSAGE_MANAGER(repref=REPREF, fort=bibfor, pyt=bibpyt, 
+
+    msgman = MESSAGE_MANAGER(repref=REPREF, fort=bibfor, pyt=bibpyt,
                             cache_dict=osp.join(run['cache_dir'], 'messages_dicts.pick'),
                             force=run['force'], verbose=run['verbose'],
                             debug=run['debug'],
@@ -753,7 +769,7 @@ def GetMessageInfo(run, *args):
                             unig      =run.get('unigest', None),
                             )
     msgman.read_cata()
-    
+
     if action == 'check':
         try:
             unused, not_found = msgman.check_cata()
@@ -772,7 +788,7 @@ def GetMessageInfo(run, *args):
             run.parser.error(
                     _(u"'--%s %s' requires 2 arguments") % (run.current_action, action))
         msgman.move(*args)
-    
+
     else:
         print3()
         fmt = '%12s : %s'
