@@ -25,12 +25,12 @@ import os.path as osp
 import time
 
 from asrun.common.i18n import _
-from asrun.mystring  import print3, ufmt
-from asrun.calcul    import ASTER_TESTCASE, ASTER_PARAMETRIC
+from asrun.mystring  import print3, ufmt, add_to_tail
+from asrun.calcul    import AsterCalcTestcase, AsterCalcParametric
 from asrun.thread    import Task, TaskAbort, Empty
 from asrun.repart    import NORESOURCE, ALLOCATED, OVERLIMIT
 
-fmt_resu = '%-12s %-18s %8.2f %8.2f %8.2f %8.2f'
+fmt_resu = '%-12s %-10s %-18s %8.2f %8.2f %8.2f %8.2f'
 
 
 def force_list(obj):
@@ -74,27 +74,33 @@ def check_opts(obj):
 class DistributionTask(Task):
     """Manage executions of several Code_Aster executions.
     items are couples (jobname, dict_options)
-    
+
     attributes (init during instanciation) :
-        run     : ASTER_RUN object
+        run     : AsterRun object
         hostrc  : ResourceManager object
         timeout : timeout for not allocated jobs
         info    : information level
     """
+    # Warning: do not change required parameters of DistribParametricTask
+    #          without updating MACR_RECAL source files.
     last_submit = -1.
     # declare attrs
     run = hostrc = prof = conf = test_result = None
-    info = nbitem = timeout = nbnook = nbmaxnook = 0
+    info = nbitem = timeout = run_timeout = nbnook = nbmaxnook = 0
     facmtps = 0.
     cpresok = False
-    REPREF = reptest = resutest = flashdir = ""
-
+    REPREF = reptest = resutest = flashdir = reptrav = ""
 
     def _mess_timeout(self, dt, timeout, job, refused):
-        """Emit a message when timeout occurs."""
+        """Emit a message when submission timeout occurs."""
         self.run.Mess(ufmt(_(u"no submission for last %.0f s (timeout %.0f s), job '%s' " \
             "cancelled after %d attempts."), dt, timeout, job, refused))
 
+    def _mess_running_timeout(self, dt, timeout, job):
+        """Emit a message when running timeout occured."""
+        self.run.Mess(ufmt(_(u"The job '%s' has been submitted since %.0f s and is "\
+            "not ended (timeout %.0f s). You can kill it and get other results."),
+            job, dt, timeout))
 
     def execute(self, items, **kwargs):
         """Function called for each group of items of the stack.
@@ -112,7 +118,7 @@ class DistributionTask(Task):
             moy_elaps = sum_elaps / max(1, len(l_resu))
             refresh_delay = max(2., moy_elaps/10.)
             self.pre_exec(result=l_resu, **kwargs)
-            
+
             # 1. try to start a new calculation
             rc_ok = True
             while rc_ok:
@@ -150,7 +156,7 @@ class DistributionTask(Task):
                         refresh_delay = 0.
                         if self.hostrc is not None:
                             serv = self.hostrc.GetConfig(host).get('serv')
-                        number = self.is_done(job)
+                        order = self.is_done(job)
                         calcul.on_host(serv, host)
                         jret, out = calcul.start()
                         self.last_submit = time.time()
@@ -160,11 +166,12 @@ class DistributionTask(Task):
                             if self.info >= 1:
                                 host = host or "'localhost'"
                                 self.run.Mess(ufmt(_(u'Starting execution of %s on %s (%d/%d - %s/%s)...'),
-                                    job, host, number, self.nbitem, calcul.jobid, calcul.queue),
+                                    job, host, order, self.nbitem, calcul.jobid, calcul.queue),
                                     'SILENT')
                             self.run.DBG('Starting execution // thread #%d   %s   %s   %s' \
                                     % (kwargs['threadid'], job, calcul.jobid, calcul.queue))
                             opts['submit_time'] = self.last_submit
+                            opts['order'] = order
                             run_calc.append([job, opts, itemid, calcul])
                     if result is not None:
                         l_resu.append(result)
@@ -176,16 +183,17 @@ class DistributionTask(Task):
                 time.sleep(refresh_delay)
             for job, opts, itemid, calcul in run_calc:
                 actu_time = time.time()
-                state, diag, node, tcpu, wrk, queue = calcul.get_state()
+                state = self.get_calcul_state(calcul)
                 # wait at least 3 secondes between submission and end
                 if state == 'ENDED' and actu_time - opts['submit_time'] > 3.:
                     if self.hostrc is not None:
                         self.hostrc.Free(job)
-                    res = list(calcul.get_diag())
+                    res = list(self.get_calcul_diag(calcul))
                     result = self.ended(job, opts, itemid, calcul, res)
                     sum_elaps += result[6]
                     l_resu.append(result)
                 else:
+                    self.run.DBG("waiting for %s : state %s" % (job, state))
                     next.append([job, opts, itemid, calcul])
             all_finished = len(next) == 0
             run_calc = next
@@ -206,7 +214,8 @@ class DistributionTask(Task):
         opts['refused'] =  opts.get('refused', 0) + 1
         result = self.failure(job, opts, itemid)
         if   status == OVERLIMIT:
-            self.run.Mess(ufmt(_(u"job '%s' exceeds hard limit, it will not be submitted"), job))
+            self.run.Mess(ufmt(_(u"job '%s' exceeds resources limit (defined through "
+                "hostfile), it will not be submitted."), job), '<A>_LIMIT_EXCEEDED')
         elif status == NORESOURCE:
             if self.last_submit > 0.:
                 dt = time.time() - self.last_submit
@@ -224,57 +233,74 @@ class DistributionTask(Task):
         return result
 
     def start_failed(self, job, opts, itemid, msg):
-        """Action when a job submitting failed.
-        """
+        """Action when a job submitting failed. """
         self.run.Mess(ufmt(_(u"'%s' not submitted. Error : %s"), job, msg), '<A>_NOT_SUBMITTED')
         return self.failure(job, opts, itemid)
 
+    def get_calcul_state(self, calcul):
+        """Function to retreive the state of a calculation."""
+        res = calcul.get_state()
+        if not calcul.is_ended():
+            dt = time.time() - calcul.start_time
+            if dt > self.run_timeout > 0:
+                self._mess_running_timeout(dt, self.run_timeout, calcul.name)
+        return res[0]
+
+    def get_calcul_diag(self, calcul):
+        """Function to retreive the diagnostic of the calculation."""
+        res = calcul.get_diag()
+        return res
 
     def pre_exec(self, **kwargs):
-        """Function called at the beginning of execute.
-        """
+        """Function called at the beginning of execute. """
 
     def post_exec(self, **kwargs):
         """Function called at the end of execute."""
 
     def create_calcul(self, job, opts, itemid, pid):
-        """Create a (derived) instance of ASTER_CALCUL.
-        """
+        """Create a (derived) instance of AsterCalcul. """
         raise NotImplementedError
 
+    def summary_line(self, job, opts, res):
+        """Return a summary line of the execution result."""
+        args = [job, '(%d/%d)' % (opts['order'], self.nbitem)]
+        args.extend(res)
+        line = ufmt(fmt_resu, *args)
+        return line
+
     # refused, start_failed, ended should return result in the same format
     def ended(self, job, opts, itemid, calcul, res):
-        """Call when a job is ended.
-        """
+        """Call when a job is ended."""
         raise NotImplementedError
 
     def result(self, *l_resu, **kwargs):
         """Function called after each task to treat results of execute.
         Arguments are 'execute' results + keywords args.
-        'result' is called thread-safely, so can store results in attributes.
-        """
+        'result' is called thread-safely, so can store results in attributes."""
         raise NotImplementedError
 
 
+
 class DistribTestTask(DistributionTask):
     """Manage executions of several Code_Aster testcases.
     items are couples (jobname=testcase name, options)
     attributes (init during instanciation) :
     IN :
-        run      : ASTER_RUN object
+        run      : AsterRun object
         hostrc   : ResourceManager object
-        prof     : ASTER_PROFIL object
-        conf     : ASTER_CONFIG object
+        prof     : AsterProfil object
+        conf     : AsterConfig object
         REPREF, reptest, resutest : directories
         flashdir : directory for .o, .e files...
         nbmaxnook, cpresok, facmtps : parameters
+        reptrav  : working directory
         info     : information level
     OUT :
         nbnook (indiced by threadid)
         test_result : list of (test, opts, diag, tcpu, tsys, ttot, telap)
     """
     def _mess_timeout(self, dt, timeout, job, refused):
-        """Emit a message when timeout occurs."""
+        """Emit a message when submission timeout occurs."""
         self.run.Mess(ufmt(_(u"no submission for last %.0f s " \
             "(timeout %.0f s, equal to the time requested by the main job, named 'astout'), " \
             "job '%s' cancelled after %d attempts."), dt, timeout, job, refused))
@@ -289,9 +315,9 @@ class DistribTestTask(DistributionTask):
             raise TaskAbort(reason, current_result)
 
     def create_calcul(self, job, opts, itemid, pid):
-        """Create a (derived) instance of ASTER_CALCUL.
+        """Create a (derived) instance of AsterCalcul.
         """
-        calcul = ASTER_TESTCASE(self.run, test=job, prof=self.prof, pid=pid,
+        calcul = AsterCalcTestcase(self.run, test=job, prof=self.prof, pid=pid,
                                         conf=self.conf, REPREF=self.REPREF,
                                         reptest=self.reptest, resutest=self.resutest,
                                         facmtps=self.facmtps)
@@ -300,26 +326,33 @@ class DistribTestTask(DistributionTask):
     def ended(self, job, opts, itemid, calcul, res):
         """Call when a job is ended.
         """
-        print3(fmt_resu % tuple([job,] + list(res)))
+        line = self.summary_line(job, opts, res)
+        # printing line is not thread safe but it's only printing!
+        print3(line)
+        add_to_tail(self.reptrav, line)
         # count nook for each thread
         gravity = self.run.GetGrav(calcul.diag)
-        if gravity == -9 or gravity >= self.run.GetGrav('NOOK'):
+        error = gravity == -9 or gravity >= self.run.GetGrav('NOOK')
+        if error:
             self.nbnook[opts['threadid']] += 1
+            add_to_tail(self.resutest, line, filename='NOOK')
+        add_to_tail(self.resutest, line, filename='RESULTAT')
         output_filename = _(u'no error or flashdir not defined')
         # keep result files if RESOK or test failed
-        if self.cpresok == 'RESNOOK' and gravity <= self.run.GetGrav('<A>'):
+        if self.cpresok == 'RESNOOK' and not error:
             calcul.clean_results()
         else:
             # copy output/error to flashdir
             if self.flashdir != None:
                 try:
-                    self.run.MkDir(self.flashdir, niverr='SILENT')
+                    if not osp.isdir(self.flashdir):
+                        self.run.MkDir(self.flashdir, niverr='SILENT')
                 except OSError:
                     pass
                 self.run.Copy(self.flashdir,
-                          calcul.flash('o'), calcul.flash('e'), calcul.flash('p'),
+                          calcul.flash('output'), calcul.flash('error'), calcul.flash('export'),
                           niverr='<A>_ALARM')
-                output_filename = osp.join(self.flashdir, osp.basename(calcul.flash('o')))
+                output_filename = osp.join(self.flashdir, osp.basename(calcul.flash('output')))
         result = [job, opts]
         result.extend(res)
         result.append(output_filename)
@@ -346,9 +379,9 @@ class DistribParametricTask(DistributionTask):
     items are couples (jobname=label, parameters values)
     attributes (init during instanciation) :
     IN :
-        run      : ASTER_RUN object
+        run      : AsterRun object
         hostrc   : ResourceManager object
-        prof     : ASTER_PROFIL object
+        prof     : AsterProfil object
         resudir  : directories
         flashdir : directory for .o, .e files...
         info     : information level
@@ -358,40 +391,46 @@ class DistribParametricTask(DistributionTask):
     """
     # declare attrs
     exec_result = keywords = resudir = None
-    
+
     def _mess_timeout(self, dt, timeout, job, refused):
-        """Emit a message when timeout occurs."""
+        """Emit a message when submission timeout occurs."""
         self.run.Mess(ufmt(_(u"no submission for last %.0f s " \
             "(timeout %.0f s, equal to 2 * the time requested by the main job), " \
             "job '%s' cancelled after %d attempts."), dt, timeout, job, refused))
 
     def create_calcul(self, job, opts, itemid, pid):
-        """Create a (derived) instance of ASTER_CALCUL.
+        """Create a (derived) instance of AsterCalcul.
         """
-        calcul = ASTER_PARAMETRIC(self.run, job, prof=self.prof, pid=pid,
-                                values=opts, keywords=self.keywords,
-                                resudir=self.resudir)
+        calcul = AsterCalcParametric(self.run, job, prof=self.prof, pid=pid,
+                                     values=opts, keywords=self.keywords,
+                                     resudir=self.resudir)
         return calcul
 
     def ended(self, job, opts, itemid, calcul, res):
-        """Call when a job is ended.
-        """
-        print3(fmt_resu % tuple([job,] + list(res)))
+        """Call when a job is ended."""
+        line = self.summary_line(job, opts, res)
+        # printing line is not thread safe but it's only printing!
+        print3(line)
+        add_to_tail(self.reptrav, line)
         # count nook for each thread
         gravity = self.run.GetGrav(calcul.diag)
-        if gravity == -9 or gravity >= self.run.GetGrav('NOOK'):
+        error = gravity == -9 or gravity >= self.run.GetGrav('NOOK')
+        if error:
             self.nbnook[opts['threadid']] += 1
+            add_to_tail(self.resudir, line, filename='NOOK')
+        add_to_tail(self.resudir, line, filename='RESULTAT')
         output_filename = _(u'no error or flashdir not defined')
         # copy output/error to flashdir
         if self.flashdir != None:
             try:
-                self.run.MkDir(self.flashdir, niverr='SILENT')
+                if not osp.isdir(self.flashdir):
+                    self.run.MkDir(self.flashdir, niverr='SILENT')
             except OSError:
                 pass
             self.run.Copy(self.flashdir,
-                       calcul.flash('o'), calcul.flash('e'), calcul.flash('p'),
+                       calcul.flash('output'), calcul.flash('error'), calcul.flash('export'),
                        niverr='<A>_ALARM')
-            output_filename = osp.join(self.flashdir, osp.basename(calcul.flash('o')))
+            output_filename = osp.join(self.flashdir, osp.basename(calcul.flash('output')))
         result = [job, opts]
         result.extend(res)
         result.append(output_filename)
