@@ -22,33 +22,35 @@ Give functions :
 - to compile code source, to create catalogue and to build
 an executable file,
 - to execute Code_Aster...
-Functions are called by an ASTER_RUN object.
+Functions are called by an AsterRun object.
 """
 
 import os
 import os.path as osp
-import sys
 from glob import glob
 
+from asrun.core         import magic
 from asrun.common.i18n  import _
 from asrun.mystring     import print3, ufmt
-from asrun.profil       import ASTER_PROFIL
-from asrun.build        import ASTER_BUILD
-from asrun.config       import ASTER_CONFIG
-from asrun.maintenance  import get_version
-from asrun.execution    import execute, copyfiles, build_export_from_files
+from asrun.profil       import AsterProfil
+from asrun.build        import AsterBuild
+from asrun.config       import build_AsterConfig
+from asrun.maintenance  import get_aster_version
+from asrun.execution    import execute, copyfiles, build_export_from_files, add_all_results
 from asrun.mpi          import MPI_INFO
 from asrun.astout       import RunAstout
 from asrun.parametric   import Parametric
+from asrun.multiple     import Multiple
 from asrun.common_func  import get_tmpname
-from asrun.common.utils import listsurcharge, NO_VALUES
+from asrun.common.utils import get_absolute_path, listsurcharge, get_plugin, \
+                               NO_VALUES
 from asrun.toolbox      import MakeShared
 
 
 def SetParser(run):
     """Configure the command-line parser, add options name to store to the list,
     set actions informations.
-    run : ASTER_RUN object which manages the execution
+    run : AsterRun object which manages the execution
     """
     acts_descr = {
         'run'       : {
@@ -58,7 +60,7 @@ def SetParser(run):
         },
         'quick'     : {
             'method' : StartNow,
-            'syntax' : '[options] file1 [file2 [...]]',
+            'syntax' : '[options] file1 [file2 [...]] [--surch_pyt=...] [--surch_fort=...]',
             'help'   : _(u'Start quickly an interactive execution using the files given in arguments')
         },
         'make_shared' : {
@@ -68,11 +70,22 @@ def SetParser(run):
                 'source files src1... srcN. Typically used to build a UMAT library.'),
         },
     }
+    opts_descr = {
+        'copy_all_results' : {
+            'args'   : ('--copy_all_results', ),
+            'kwargs' : {
+                'action'  : 'store_true',
+                'default' : False,
+                'dest'    : 'copy_all_results',
+                'help'    : _(u'copy all results in the current directory (for --quick action)')
+            }
+        },
+    }
     run.SetActions(
             actions_descr = acts_descr,
             actions_order = ['run', 'quick', 'make_shared'],
             group_options = False,
-            options_descr = {}
+            options_descr=opts_descr
     )
 
 
@@ -82,7 +95,7 @@ def RunAster(run, *args):
     """
     run.print_timer = True
 
-    prev = os.getcwd()
+    prev = os.getcwdu()
     # ----- check argument
     if len(args) != 1:
         run.parser.error(_(u"'--%s' requires one argument") % run.current_action)
@@ -94,34 +107,28 @@ def RunAster(run, *args):
 
     # 1.1. ----- check argument type
     # 1.1.1. ----- use profile from args
-    if isinstance(args[0], ASTER_PROFIL):
+    if isinstance(args[0], AsterProfil):
         prof = args[0].copy()
         prof.WriteExportTo(fprof)
+        forig = fprof
     else:
         # 1.1.2. ----- read profile from args
-        kret = run.Copy(fprof, args[0], niverr='<F>_PROFILE_COPY')
-        prof = ASTER_PROFIL(fprof, run)
+        forig = args[0]
+        kret = run.Copy(fprof, forig, niverr='<F>_PROFILE_COPY')
+        prof = AsterProfil(fprof, run)
+        if not run.IsRemote(forig):
+            export_fname = run.PathOnly(get_absolute_path(forig))
+            prof.absolutize_filename(export_fname)
     if not prof['mode'][0] in ('batch', 'interactif'):
         run.Mess(_(u"Unknown mode (%s), use 'interactif' instead") % \
                 repr(prof['mode'][0]), 'UNEXPECTED_VALUE')
         prof['mode'] = ['interactif']
+    run.DBG("Input export : %s" % fprof, prof)
 
+    # 1.2. get AsterConfig and AsterBuild objects
     REPREF = prof.get_version_path()
-
-    # 1.2. get ASTER_CONFIG and ASTER_BUILD objects
-    lconf = prof.Get('D', typ='conf')
-    if not lconf:
-        ficconf = os.path.join(REPREF, 'config.txt')
-    else:
-        ficconf = lconf[0]['path']
-        if run.IsRemote(ficconf):
-            ficconf = get_tmpname(run, run['tmp_user'], basename='config.txt')
-            run.ToDelete(ficconf)
-            kret = run.Copy(ficconf, lconf[0]['path'])
-        else:
-            ficconf = run.PathOnly(ficconf)
-    conf = ASTER_CONFIG(ficconf, run)
-    build = ASTER_BUILD(run, conf)
+    conf = build_AsterConfig(run, prof)
+    build = AsterBuild(run, conf)
     DbgPara = {
         'debug'     : { 'exe'    : conf['BIN_DBG'][0],
                       'suff'   : conf['BINOBJ_DBG'][0],
@@ -136,9 +143,16 @@ def RunAster(run, *args):
     # 1.3. set environment depending on version
     for f in conf.get_with_absolute_path('ENV_SH'):
         run.AddToEnv(f)
-    
+
     # 1.4. set mpi parameters
-    mpi = MPI_INFO(conf.get_defines())
+    klass = MPI_INFO
+    # allow to customize of the execution objects
+    if run.get('schema_execute'):
+        schem = get_plugin(run['schema_execute'])
+        run.DBG("calling plugin : %s" % run['schema_execute'])
+        klass = schem(prof)
+
+    mpi = klass(conf.get_defines())
     iret = mpi.set_cpuinfo(prof['mpi_nbnoeud'][0], prof['mpi_nbcpu'][0])
     if iret == 1:
         run.Mess(ufmt(_(u"%s is not a MPI version of Code_Aster. "
@@ -148,15 +162,21 @@ def RunAster(run, *args):
             % (prof['mpi_nbnoeud'][0], prof['mpi_nbcpu'][0]), '<F>_INVALID_PARAMETER')
 
     # 1.5. rep_trav from profile or from run[...]
-    mpi.set_rep_trav(run, prof['rep_trav'][0], prof['mode'][0])
-    reptrav = mpi.reptrav()
+    reptrav = mpi.set_rep_trav(prof['rep_trav'][0], prof['mode'][0])
+
+    # write reptrav in the export
+    prof['rep_trav'] = reptrav
+    prof.WriteExportTo(prof.get_filename())
+    #XXX overrides the original export
+    if forig != prof.get_filename():
+        run.Copy(forig, prof.get_filename(), niverr='<A>_ALARM')
+
     # add reptrav to LD_LIBRARY_PATH (to find dynamic libs provided by user)
     old = os.environ.get("LD_LIBRARY_PATH", "")
     os.environ["LD_LIBRARY_PATH"] = (reptrav + os.pathsep + old).strip(os.pathsep)
-    
-    # do not reinitialize rep_trav if 
+
+    # do not reinitialize rep_trav if
     if prof['prep_env'][0] not in NO_VALUES:
-        run.Delete(reptrav)
         run.MkDir(reptrav, chmod=0700)
     if prof['detr_rep_trav'][0] not in NO_VALUES:
         run.ToDelete(reptrav)
@@ -190,7 +210,7 @@ def RunAster(run, *args):
     # order of actions :
     list_actions = ['make_exec',  'make_cmde', 'make_ele',
                    'make_etude', 'make_dbg',  'make_env', 'astout',
-                   'distribution',
+                   'distribution', 'multiple',
                    'exec_crs',   'exec_crp']
 
     # 3. ==> Let's go !
@@ -208,11 +228,20 @@ def RunAster(run, *args):
         args_supp = ' -num_job '+jn+' -mode '+prof['mode'][0]
         prof.args['args'] += args_supp
 
+    # check if the version allows developments
+    if conf['DEVEL'][0] in NO_VALUES and \
+        ( 'make_exec' in prof['actions'] or \
+          'make_cmde' in prof['actions'] or \
+          'make_ele' in prof['actions'] ):
+        run.Mess(_(u'The configuration of this version does not allow '
+                    'user developments.'), '<F>_ERROR')
+
     #
     # 3.1. ----- make_exec
     #
     iret = 0
     if 'make_exec' in prof['actions']:
+        run.DBG(u'Start make_exec action')
         exetmp = os.path.join(reptrav, 'aster.exe')
         tit = _(u'Compilation of source files')
         run.Mess(tit, 'TITLE')
@@ -230,7 +259,7 @@ def RunAster(run, *args):
                 iret = max(iret, jret)
                 lf.extend(lbi)
         # liste des fichiers surchargés
-        vers = get_version(run, REPREF)
+        vers = get_aster_version(REPREF)
         vers = '.'.join(vers[:3])
         fsurch = os.path.join(repact, 'surchg.f')
         listsurcharge(vers, fsurch, lf)
@@ -249,6 +278,18 @@ def RunAster(run, *args):
         if run.IsDir(libferm):
             libferm = os.path.join(libferm, 'ferm.lib')
         lobj = glob(os.path.join(repobj, '*.o'))
+        # build an archive if there are more than NNN object files
+        if len(lobj) > 500:
+            run.timer.Stop(tit)
+            tit2 = _(u'Add object files to library')
+            run.timer.Start(tit2)
+            libtmp = osp.join(repobj, 'libsurch.a')
+            run.Copy(libtmp, libaster)
+            kret = build.Archive(repobj, libtmp, force=True)
+            lobj = []
+            libaster = libtmp
+            run.timer.Stop(tit2)
+            run.timer.Start(tit)
         kret = build.Link(exetmp, lobj, libaster, libferm, repact)
         run.timer.Stop(tit)
         run.CheckOK()
@@ -270,6 +311,7 @@ def RunAster(run, *args):
     # 3.2. ----- make_cmde
     #
     if 'make_cmde' in prof['actions']:
+        run.DBG(u'Start make_cmde action')
         tit = _(u"Compilation of commands catalogue")
         cmdetmp = os.path.join(reptrav, 'cata_commande')
         run.timer.Start(tit)
@@ -277,9 +319,12 @@ def RunAster(run, *args):
         run.MkDir(repact)
         kargs = { 'exe' : exetmp, 'cmde' : cmdetmp, }
         kargs['capy'] = [l['path'] for l in prof.Get('D', typ='capy')]
+        lfun = prof.Get('D', typ='unig')
+        if lfun:
+            kargs['unigest'] = build.GetUnigest(lfun[0]['path'])
         if prof.Get('D', typ='py'):
             kargs['py'] = [l['path'] for l in prof.Get('D', typ='py')]
-        jret = build.CompilCapy(REPREF, repact, **kargs)      #i18n=True, 
+        jret = build.CompilCapy(REPREF, repact, **kargs)      #i18n=True,
         run.timer.Stop(tit)
         run.CheckOK()
 
@@ -295,6 +340,7 @@ def RunAster(run, *args):
     # 3.3. ----- make_ele
     #
     if 'make_ele' in prof['actions']:
+        run.DBG(u'Start make_ele action')
         tit = _(u"Compilation of elements")
         eletmp = os.path.join(reptrav, 'elem.1')
         run.timer.Start(tit)
@@ -302,9 +348,6 @@ def RunAster(run, *args):
         run.MkDir(repact)
         kargs = { 'exe' : exetmp, 'cmde' : cmdetmp, 'ele' : eletmp, }
         kargs['cata'] = [l['path'] for l in prof.Get('D', typ='cata')]
-        lfun = prof.Get('D', typ='unig')
-        if lfun:
-            kargs['unigest'] = build.GetUnigest(lfun[0]['path'])
         if prof.Get('D', typ='py'):
             kargs['py'] = [l['path'] for l in prof.Get('D', typ='py')]
         jret = build.CompilEle(REPREF, repact, **kargs)
@@ -324,14 +367,22 @@ def RunAster(run, *args):
     #
     if 'make_env' in prof['actions'] or 'make_etude' in prof['actions'] or \
             'make_dbg' in prof['actions']:
+        run.DBG(u'Start make_etude/make_env/make_dbg action')
         os.chdir(reptrav)
         run.Mess(_(u'Code_Aster execution'), 'TITLE')
-        
+
         # 3.4.1. prepare reptrav to run Code_Aster (proc# = 0)
-        kargs = { 'exe'  : exetmp,
-                'cmde' : cmdetmp,
-                'ele'  : eletmp,
-                'lang' : prof['lang'][0], }
+        only_env = 'make_env' in prof['actions']
+        kargs = {
+            'exe'  : exetmp,
+            'cmde' : cmdetmp,
+            'ele'  : eletmp,
+            'lang' : prof['lang'][0],
+            'only_env' : only_env,
+        }
+        lfun = prof.Get('D', typ='unig')
+        if lfun:
+            kargs['unigest'] = build.GetUnigest(lfun[0]['path'])
         if prof.Get('D', typ='py'):
             kargs['py'] = [l['path'] for l in prof.Get('D', typ='py')]
         tit = _(u'Preparation of environment')
@@ -342,7 +393,7 @@ def RunAster(run, *args):
         else:
             run.Mess(_(u'... skipped (%s = no) !') % 'prep_env', 'SILENT')
         run.timer.Stop(tit)
-        
+
         # 3.4.2. copy datas (raise <E> errors if failed)
         tit = _(u'Copying datas')
         run.Mess(tit, 'TITLE')
@@ -351,7 +402,7 @@ def RunAster(run, *args):
             copyfiles(run, 'DATA', prof)
         else:
             run.Mess(_(u'... skipped (%s = no) !') % 'copy_data', 'SILENT')
-            print3(os.getcwd())
+            print3(os.getcwdu())
         run.timer.Stop(tit)
 
         # 3.4.3. execution
@@ -359,7 +410,7 @@ def RunAster(run, *args):
                 reptrav,
                 multiple       = False,
                 with_dbg       = 'make_dbg' in prof['actions'],
-                only_env       = 'make_env' in prof['actions'],
+                only_env       = only_env,
                 fpara          = None,
                 mpi_info       = mpi,
                 run=run, conf=conf, prof=prof, build=build, exe=exetmp)
@@ -370,7 +421,8 @@ def RunAster(run, *args):
             run.Mess(tit, 'TITLE')
             run.timer.Start(tit)
             if prof['copy_result'][0] not in NO_VALUES:
-                copyfiles(run, 'RESU', prof, copybase)
+                emit_alarm = prof['copy_result_alarm'][0] not in NO_VALUES
+                copyfiles(run, 'RESU', prof, copybase, emit_alarm)
             else:
                 run.Mess(_(u'... skipped (%s = no) !') % 'copy_result', 'SILENT')
             run.timer.Stop(tit)
@@ -379,10 +431,10 @@ def RunAster(run, *args):
             # 3.4.5. add .resu/.erre to output for testcases
             if os.path.exists('fort.15') and os.stat('fort.15').st_size > 0:
                 run.Mess(_(u'Content of RESU file'), 'TITLE')
-                run.FileCat('fort.8', sys.stdout)
+                run.FileCat('fort.8', magic.get_stdout())
                 run.Mess(_(u'Content of ERROR file'), 'TITLE')
-                run.FileCat('fort.9', sys.stdout)
-            
+                run.FileCat('fort.9', magic.get_stdout())
+
             run.CheckOK()
         os.chdir(prev)
 
@@ -390,18 +442,21 @@ def RunAster(run, *args):
     # 3.5. ----- astout
     #
     if 'astout' in prof['actions']:
+        run.DBG(u'Start astout action')
         kargs = { 'exe'  : exetmp,
                 'cmde' : cmdetmp,
                 'ele'  : eletmp,
            'numthread' : prof['numthread'][0],
         }
-        
+        os.chdir(reptrav)
         RunAstout(run, conf, prof, mpi_info=mpi, **kargs)
+        os.chdir(prev)
 
     #
     # 3.6. ----- distribution
     #
     if 'distribution' in prof['actions']:
+        run.DBG(u'Start distribution action')
         kargs = { 'exe'  : exetmp,
                 'cmde' : cmdetmp,
                 'ele'  : eletmp,
@@ -410,9 +465,17 @@ def RunAster(run, *args):
         Parametric(run, prof, mpi_info=mpi, **kargs)
 
     #
-    # 3.7. ----- exec_crs
+    # 3.7. ----- multiple
+    #
+    if 'multiple' in prof['actions']:
+        run.DBG(u'Start multiple action')
+        Multiple(run, prof, mpi_info=mpi, numthread=prof['numthread'][0])
+
+    #
+    # 3.8. ----- exec_crs
     #
     if 'exec_crs' in prof['actions']:
+        run.DBG(u'Start exec_crs action')
         repsrc = os.path.join(reptrav, 'sources')
         reptmp = os.path.join(reptrav, 'tmp')
         fmsg = os.path.join(reptrav, 'messages')
@@ -448,7 +511,7 @@ def RunAster(run, *args):
             f.close()
         else:
             run.Mess(_(u'messages file not found !'), '<A>_ALARM')
-        
+
         if iret == 0:
             diag = 'OK'
         else:
@@ -457,6 +520,7 @@ def RunAster(run, *args):
         run.timer.Stop(tit)
 
     if 'exec_crp' in prof['actions']:
+        run.DBG(u'Start exec_crp action')
         repsrc = os.path.join(reptrav, 'sources')
         reptmp = os.path.join(reptrav, 'tmp')
         fmsg = os.path.join(reptrav, 'messages')
@@ -498,7 +562,7 @@ def RunAster(run, *args):
             f.close()
         else:
             run.Mess(_(u'messages file not found !'), '<A>_ALARM')
-        
+
         if iret == 0:
             diag = 'OK'
         else:
@@ -517,8 +581,7 @@ def StartNow(run, *args):
     if not run["silent"]:
         run.Mess(_(u"This functionnality is still in a beta state of development "
                    u"and may be removed a future release, or may never be improved. "
-                   u"Please use --silent option to ignore this alarm the next time."),
-                   '<A>_ALARM')
+                   u"Please use --silent option to ignore this message the next time."))
     # ----- check argument
     if not run.get('aster_vers'):
         run.parser.error(_(u"You must define 'default_vers' in 'aster' configuration file or use '--vers' option."))
@@ -527,8 +590,8 @@ def StartNow(run, *args):
 
     # build export
     lf = [os.path.abspath(f) for f in args]
-    prof = build_export_from_files(run, lf)
-    
+    prof = build_export_from_files(run, lf, with_results=True)
+
     # development files
     surch_pyt = run.get('surch_pyt', [])
     if surch_pyt:
@@ -552,12 +615,22 @@ def StartNow(run, *args):
             'path' : exetmp, 'type' : 'exec',
             'ul'   : '0', 'isrep' : False, 'compr' : False,
         })
-    
+
     if exetmp is not None:
         prof['actions'] = prof['actions'] + ['make_exec']
+
+    # try to grab all results files
+    if run["copy_all_results"]:
+        resudir = os.getcwdu()
+        lcomm = prof.get_type('comm')
+        if len(lcomm) > 0:
+            jobname = osp.splitext(osp.basename(lcomm[0].path))[0]
+        else:
+            jobname = 'unamed'
+        add_all_results(prof, resudir, jobname)
+
     run.Mess(_(u"Profile used :"), 'TITLE')
     run.Mess(os.linesep + prof.get_content(), 'SILENT')
-    
     # execution
     RunAster(run, prof)
 
