@@ -32,9 +32,10 @@ from asrun.installation import confdir
 from asrun.common.i18n import _
 from asrun.mystring     import print3, ufmt, file_cleanCR
 from asrun.mpi          import MPI_INFO
-from asrun.profil       import ASTER_PROFIL
+from asrun.profil       import AsterProfil
 from asrun.common_func  import get_tmpname
-from asrun.common.utils import getpara, check_joker
+from asrun.common.utils import getpara, check_joker, hms2s
+from asrun.common.sysutils import on_64bits
 
 
 
@@ -42,7 +43,7 @@ def dict_typ_test(test):
     """supported file types + ['com?', '[0-9]*']
     """
     return {
-        'comm' : ('fort.1' ,  1), 
+        'comm' : ('fort.1' ,  1),
         'mail' : ('fort.20', 20),
         'mmed' : ('fort.20', 20),
         'med'  : ('fort.20', 20),
@@ -52,9 +53,23 @@ def dict_typ_test(test):
         'msup' : ('fort.19', 19),
         'para' : ('%s.para' % test, 0),
         'ensi' : ('DONNEES_ENSIGHT', 0),
-        'repe' : ('REPE_IN', 0)
+        'repe' : ('REPE_IN', 0),
     }
 
+def dict_typ_result():
+    """supported file types : resu, rmed
+    """
+    return {
+        'mess' : ('fort.6',   6),
+        'resu' : ('fort.8',   8),
+        'dat'  : ('fort.29', 29),
+        'rmed' : ('fort.80', 80),
+        'pos'  : ('fort.37', 37),
+        'base' : ('unused',   0),
+        'bhdf' : ('unused',   0),
+    }
+
+
 
 def execute(reptrav, multiple=False, with_dbg=False, only_env=False,
             follow_output=True, fpara=None, facmtps=1.,
@@ -83,8 +98,8 @@ def execute(reptrav, multiple=False, with_dbg=False, only_env=False,
     validbase = True
     if mpi_info is None:
         mpi_info = MPI_INFO()
-        mpi_info.set_rep_trav(run, reptrav)
-    
+        mpi_info.set_rep_trav(reptrav)
+
     interact = prof.args.has_key('interact') or \
               prof.args['args'].find('-interact') > -1
 
@@ -109,7 +124,7 @@ def execute(reptrav, multiple=False, with_dbg=False, only_env=False,
         elif k in ('memjeveux', 'memjeveux_stat', 'tpmax'):
             cmd_memtps[k] = v
         elif v.strip() == '' and k in drep.values():
-            run.Mess(_(u'empty value not allowed for "%s"') % k, '<A>_INVALID_PARAMETER')            
+            run.Mess(_(u'empty value not allowed for "%s"') % k, '<A>_INVALID_PARAMETER')
         else:
             cmd.append('-%s %s' % (k, v))
 
@@ -123,7 +138,7 @@ def execute(reptrav, multiple=False, with_dbg=False, only_env=False,
         ncpus = prof['ncpus'][0]
 
     # 4. ----- add parameters from prof or fpara if present
-    if re.search('64$', run['plate-forme']):
+    if on_64bits():
         facW = 8
     else:
         facW = 4
@@ -159,13 +174,13 @@ def execute(reptrav, multiple=False, with_dbg=False, only_env=False,
         except ValueError:
             run.Mess(_(u'incorrect value for memjeveux (%s) in profile') % \
                     prof.args['memjeveux'], '<E>_INCORRECT_PARA')
-        
+
         try:
             memjs = float(prof.args.get('memjeveux_stat', 0))
         except ValueError:
             run.Mess(_(u'incorrect value for memjeveux_stat (%s) in profile') % \
                     prof.args['memjeveux_stat'], '<E>_INCORRECT_PARA')
-    
+
     try:
         nbp = int(ncpus)
     except ValueError:
@@ -201,26 +216,21 @@ export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:.
         # set per version environment
         for f in conf.get_with_absolute_path('ENV_SH') + [tmp_profile]:
             run.Mess('   . %s' % f, 'SILENT')
-        
+
         cmd.insert(0, osp.join('.', osp.basename(exetmp)))
         # add memjeveux and tpmax
         cmd.extend(['-%s %s' % (k, v) for k, v in cmd_memtps.items()])
         cmdline = ' '.join(cmd)
-        
+
         # get pdb.py path
         pdbpy_cmd = "import os, sys ; " \
             "pdbpy = os.path.join(sys.prefix, 'lib', 'python' + sys.version[:3], 'pdb.py')"
         d = {}
         exec pdbpy_cmd in d
         pdbpy = d['pdbpy']
-        
-        if mpi_info.really():
-            if len(list_comm) > 1:
-                run.Mess(_("Running more than one comm file is not allowed in MPI mode. "
-                           "Only the first .comm will be started."), "<A>_ALARM")
-                list_comm = [list_comm[0], ]
 
-            cmdline = mpi_info.get_exec_command(run, cmdline,
+        if mpi_info.really():  #XXX and if not ? perhaps because of exit_code
+            cmdline = mpi_info.get_exec_command(cmdline,
                 add_tee=False,
                 env=conf.get_with_absolute_path('ENV_SH'))
 
@@ -228,19 +238,17 @@ export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:.
         k = 0
         for fcomm in list_comm:
             k += 1
-            cmd_2 = cmdline.replace('fort.1', fcomm)
-            run.Mess(_(u'Command line %d :') % k)
-            run.Mess(cmd_2, 'SILENT')
-            if mpi_info.really():
-                os.chdir(mpi_info.reptrav())
-                run.Rename(fcomm, 'fort.1')
-                break
+            run.Mess(_(u"Command line %d :") % k)
+            run.Mess(u"cp %s fort.1" % fcomm, 'SILENT')
+            run.Mess(cmdline, 'SILENT')
             # how to start the Python debugger
-            run.Mess(_(u'To start execution in the Python debugger you could type :'), 'SILENT')
-            pdb_cmd = cmd_2.replace(osp.join('.', osp.basename(exetmp)),
-                '%s %s' % (osp.join('.', osp.basename(exetmp)),
+            if not mpi_info.really():
+                run.Mess(_(u'To start execution in the Python debugger you could type :'), 'SILENT')
+                pdb_cmd = cmdline.replace(osp.join('.', osp.basename(exetmp)),
+                    '%s %s' % (osp.join('.', osp.basename(exetmp)),
                             ' '.join(pdbpy.splitlines())))
-            run.Mess(pdb_cmd, 'SILENT')
+                run.Mess(u"cp %s fort.1" % fcomm, 'SILENT')
+                run.Mess(pdb_cmd, 'SILENT')
 
         diag = 'OK'
 
@@ -278,7 +286,7 @@ export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:.
             update_cmd_memtps(cmd_memtps)
             cmd.extend(['-%s %s' % (k, v) for k, v in cmd_memtps.items()])
             pos_memtps = -1
-            
+
             cmd_args = ' '.join(cmd)
             ldbg.append('run ' + cmd_args)
             cmd = getdbgcmd(cmd_dbg, osp.basename(exetmp), '', ldbg, cmd_args)
@@ -295,13 +303,8 @@ export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:.
                 corefilesize = 'unlimited'
             run.SetLimit('core', corefilesize)
 
-        # 6.3. info if multiple .comm files
-        if len(list_comm) > 1 and mpi_info.really():
-            run.Mess(_("Running more than one comm file is not allowed in MPI mode. "
-                       "Only the first .comm will be started."), "<A>_ALARM")
-            list_comm = [list_comm[0], ]
-
-        if len(list_comm)>1:
+        # 6.3. if multiple .comm files, keep previous bases
+        if len(list_comm) > 1:
             run.Mess(_(u'%d command files') % len(list_comm))
             validbase = False
             BASE_PREC = osp.join(reptrav, 'BASE_PREC')
@@ -313,26 +316,26 @@ export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:.
         for fcomm in list_comm:
             k += 1
             os.chdir(mpi_info.reptrav())
-            run.Rename(fcomm, 'fort.1')
+            run.Copy('fort.1', fcomm)
 
             # start execution
             tit = _(u'Code_Aster run')
             run.timer.Start(tit)
-            
+
             # add memjeveux and tpmax at the right position
             cmd_i = cmd[:]
             update_cmd_memtps(cmd_memtps)
             if pos_memtps > -1:
                 for key, value in cmd_memtps.items():
                     cmd_i.insert(pos_memtps, '-%s %s' % (key, value))
-                
+
             if not ctest:
                 run.Mess(tit, 'TITLE')
                 run.Mess(_(u'Command line %d :') % k)
                 if not run['verbose']:
                     run.Mess(' '.join(cmd_i))
-            
-            cmd_exec = mpi_info.get_exec_command(run, ' '.join(cmd_i),
+
+            cmd_exec = mpi_info.get_exec_command(' '.join(cmd_i),
                 add_tee=add_tee,
                 env=conf.get_with_absolute_path('ENV_SH'))
 
@@ -345,7 +348,7 @@ export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:.
                 print3(exec_output)
 
             # mpirun does not include cpu/sys time of childrens, add it in timer
-            mpi_info.add_to_timer(run, exec_output, tit)
+            mpi_info.add_to_timer(exec_output, tit)
 
             run.timer.Stop(tit)
             if k < len(list_comm):
@@ -374,7 +377,7 @@ export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:.
                          glob(osp.join(BASE_PREC, 'bhdf.*')) + \
                          glob(osp.join(BASE_PREC, 'pick.*'))
                         if len(lbas) > 0:
-                            run.Copy(os.getcwd(), niverr='INFO', verbose=follow_output, *lbas)
+                            run.Copy(os.getcwdu(), niverr='INFO', verbose=follow_output, *lbas)
                         else:
                             run.Mess(_(u'no glob/bhdf base to restore'), '<A>_ALARM')
                     run.Mess(_(u'execution aborted (comm file #%d)') % k, diag_k)
@@ -390,8 +393,6 @@ export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:.
                          glob('bhdf.*') + \
                          glob('pick.*')
                         run.Copy(BASE_PREC, niverr='INFO', verbose=follow_output, *lbas)
-                        if mpi_info.really():
-                            run.Copy(reptrav,   niverr='INFO', verbose=follow_output, *lbas)
                     run.Mess(_(u'execution ended (comm file #%d)') % k, diag_k)
                 # the worst diagnostic
                 if run.GetGrav(diag) < run.GetGrav(diag_k):
@@ -433,7 +434,7 @@ export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:.
 
         if not ctest:
             # 6.7. content of reptrav
-            run.Mess(ufmt(_(u'Content of %s after execution'), os.getcwd()), 'TITLE')
+            run.Mess(ufmt(_(u'Content of %s after execution'), os.getcwdu()), 'TITLE')
             kret, out = run.Shell(cmd='ls -la . REPE_OUT')
             print3(out)
 
@@ -450,7 +451,7 @@ export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:.
     return diag, tcpu, tsys, ttot, validbase
 
 
-def copyfiles(run, mode, prof, copybase=True):
+def copyfiles(run, mode, prof, copybase=True, alarm=True):
     """Copy datas from profile into `pwd` (if mode=='DATA')
     or results from `pwd` into destination given by profile (if mode=='RESU').
     Aster bases are copied only if copybase is True.
@@ -465,7 +466,7 @@ def copyfiles(run, mode, prof, copybase=True):
     else:
         l_dico = prof.resu
         for df in l_dico:
-            copyfileR(run, df, copybase)
+            copyfileR(run, df, copybase, alarm)
 
 
 def copyfileD(run, df, icomm, ncomm):
@@ -500,7 +501,7 @@ def copyfileD(run, df, icomm, ncomm):
         # repe
         elif df['type'] == 'repe':
             dest = 'REPE_IN'
-        
+
     if dest is not None:
         # 3. --- copy
         kret = run.Copy(dest, df['path'], niverr='<E>_COPY_ERROR', verbose=True)
@@ -516,12 +517,13 @@ def copyfileD(run, df, icomm, ncomm):
 
         # clean text files if necessary
         if df['ul'] != 0 and run.IsTextFileWithCR(dest):
+            os.chmod(dest, 0644)
             file_cleanCR(dest)
             print3(ufmt(' ` ' + _(u'line terminators have been removed from %s'), dest))
     return icomm
 
 
-def copyfileR(run, df, copybase=True):
+def copyfileR(run, df, copybase=True, alarm=True):
     """Copy results from current directory into destination given by `df`.
     Aster bases are copied only if copybase is True.
     Raise only <E> if an error occurs run CheckOK() after.
@@ -529,7 +531,7 @@ def copyfileR(run, df, copybase=True):
     # list of files
     lf = []
     isdir = False
-    
+
     # 1. ----- files
     if df['ul'] != '0' or df['type'] in ('nom', 'para'):
         # if logical unit is set : fort.*
@@ -552,7 +554,8 @@ def copyfileR(run, df, copybase=True):
         elif df['type'] == 'bhdf' and copybase:
             lbas = glob('bhdf.*')
             if len(lbas) == 0:
-                run.Mess(_(u"No 'bhdf' found, saving 'glob' instead"), '<A>_COPY_BASE')
+                if alarm:
+                    run.Mess(_(u"No 'bhdf' found, saving 'glob' instead"), '<A>_COPY_BASE')
                 lbas = glob('glob.*')
             lf.extend(lbas)
             lf.extend(glob('pick.*'))
@@ -563,20 +566,19 @@ def copyfileR(run, df, copybase=True):
             else:
                 rep = 'REPE_OUT'
             lfrep = glob(osp.join(rep, '*'))
-            if len(lfrep) == 0:
+            if len(lfrep) == 0 and alarm:
                 run.Mess(ufmt(_(u"%s directory is empty !"), rep), '<A>_COPY_REPE')
             lf.extend(lfrep)
-        
+
     # 3. ----- compression
     kret = 0
     if df['compr']:
         lfnam = lf[:]
         lf = []
         for fnam in lfnam:
-            kret, f = run.Gzip(fnam, niverr='<E>_COMPRES_ERROR',
-                    verbose=True)
+            kret, f = run.Gzip(fnam, niverr='<E>_COMPRES_ERROR', verbose=True)
             lf.append(f)
-    
+
     # 4. ----- copy
     if kret == 0 and len(lf) > 0:
         # create destination
@@ -590,8 +592,14 @@ def copyfileR(run, df, copybase=True):
             kret = run.MkDir(osp.dirname(df['path']), '<E>_MKDIR_ERROR')
         # copy
         if kret == 0:
-            kret = run.Copy(df['path'], niverr='<E>_COPY_ERROR',
-                    verbose=True, *lf)
+            lfc = lf[:]
+            for fname in lfc:
+                if not osp.exists(fname):
+                    if alarm:
+                        run.Mess(ufmt(u"no such file or directory: %s", fname), '<A>_COPYFILE')
+                    lf.remove(fname)
+            if len(lf) > 0:
+                kret = run.Copy(df['path'], niverr='<E>_COPY_ERROR', verbose=True, *lf)
         # save base if failure
         if kret != 0:
             rescue = get_tmpname(run, basename='save_results')
@@ -602,7 +610,8 @@ def copyfileR(run, df, copybase=True):
                     verbose=True, *lf)
 
 
-def build_test_export(run, conf, REPREF, reptest, test, resutest=None, with_default=True):
+def build_test_export(run, conf, REPREF, reptest, test, resutest=None,
+                      with_default=True, d_unig=None):
     """Return a profile for a testcase.
     """
     REPTEST = osp.join(REPREF, conf['SRCTEST'][0])
@@ -613,11 +622,20 @@ def build_test_export(run, conf, REPREF, reptest, test, resutest=None, with_defa
         if run.IsRemote(rep):
             run.Mess(ufmt(_(u'reptest (%s) must be on exec host'), rep),
                     '<F>_INVALID_DIR')
+    lrm = []
+    if d_unig:
+        lrm = [osp.basename(f) for f in d_unig['test']]
+    lall = []
     lrep = [run.PathOnly(rep) for rep in lrep]
-    lf = []
     for r in lrep:
         f = osp.join(r, '%s.*' % test)
-        lf.extend(glob(f))
+        lall.extend(glob(f))
+    lf = []
+    for f in lall:
+        if osp.basename(f) in lrm:
+            run.Mess(ufmt(_(u'deleting %s (matches unigest)'), osp.basename(f)))
+        else:
+            lf.append(f)
     prof = build_export_from_files(run, lf, test, with_default=with_default)
     if resutest:
         for typ, ul in {'resu' : 8, 'mess' : 6, 'erre' : 9}.items():
@@ -629,15 +647,35 @@ def build_test_export(run, conf, REPREF, reptest, test, resutest=None, with_defa
     return prof
 
 
-def build_export_from_files(run, lf, root="", with_default=True):
+def build_export_from_files(run, lf, root="", with_default=True, with_results=False):
     """Build an export file from a list of files.
     """
-    prof = ASTER_PROFIL(run=run)
+    prof = AsterProfil(run=run)
     if with_default:
         prof.add_default_parameters()
-    dtyp = dict_typ_test(root)
+    ddat = build_dict_file(run, prof, dict_typ_test(root), lf, ['com?', '[0-9]*'])
+    dres = {}
+    if with_results:
+        dres = build_dict_file(run, prof, dict_typ_result(), lf)
+
+    for dicf, dr in ((ddat, 'D'), (dres, 'R')):
+        lcom_i = []
+        for f, dico in dicf.items():
+            if dico['type'] != 'comm' or osp.splitext(f)[-1] == '.comm':
+                prof.Set(dr, dico)
+            else:
+                lcom_i.append([f, dico])
+        lcom_i.sort()
+        for f, dico in lcom_i:
+            prof.Set(dr, dico)
+    return prof
+
+
+def build_dict_file(run, prof, dtyp, lf, opt_suffix=[]):
+    """Build the dictionnary of files 'lf' matching the types defined
+    in 'dtyp'."""
     dvu = {}
-    for typ0 in dtyp.keys() + ['com?', '[0-9]*']:
+    for typ0 in dtyp.keys() + opt_suffix:
         if typ0 == 'comm':    # always in com?
             continue
         for f in lf:
@@ -665,21 +703,25 @@ def build_export_from_files(run, lf, root="", with_default=True):
                 except ValueError:
                     ul = 0
             dvu[base] = { 'type'  : typ,
-                       'isrep' : False,
-                       'path'  : f,
-                       'ul'    : ul,
-                       'compr' : False }
-    lcom_i = []
-    for f, dico in dvu.items():
-        if dico['type'] != 'comm' or osp.splitext(f)[-1] == '.comm':
-            prof.Set('D', dico)
-        else:
-            lcom_i.append([f, dico])
-    lcom_i.sort()
-    for f, dico in lcom_i:
-        prof.Set('D', dico)
-    return prof
+                          'isrep' : False,
+                          'path'  : f,
+                          'ul'    : ul,
+                          'compr' : False }
+    return dvu
 
+def add_all_results(prof, dest, jobname, dtyp=None):
+    """Add all known results files to 'prof'."""
+    if dtyp is None:
+        dtyp = dict_typ_result
+    prof['copy_result_alarm'] = 'no'
+    for typ, value in dtyp().items():
+        if typ in ('base', 'bhdf'):
+            continue
+        path = osp.join(dest, "%s.%s" % (jobname, typ))
+        ul = value[1]
+        prof.Set('R',
+            { 'path' : path, 'ul' : ul, 'type' : typ,
+              'isrep' : False, 'compr' : False})
 
 def getdbgcmd(cmd_dbg, exe, core, lcmd, args):
     """Return the command line (as list) to run 'lcmd' in the debugger.
@@ -710,17 +752,18 @@ def fillpara(run, **kargs):
     form = 'tps_job %(tps_job)d mem_job %(mem_job)dMo '\
           'mem_aster %(mem_aster)d ncpus %(ncpus)d\n'
     dform = kargs.copy()
-    
+
     try:
         dform['tps_job']   = int(kargs['tps_job'])
         dform['mem_aster'] = int(kargs['mem_aster'][0] or '100')
         dform['mem_job']   = int(kargs['memjMo'] * 100. / dform['mem_aster'])
-        dform['memjeveux_stat'] = kargs['memjeveux_stat']
+        dform['memjeveux_stat'] = kargs.get('memjeveux_stat', 0)
         dform['ncpus']     = int(kargs['ncpus'])
 #      dform['nnoeud_mpi'] = int(kargs[''])
         open(kargs['filename'], 'w').write(form % dform)
     except Exception, msg:
-        run.DBG(kargs)
+        pass
+    run.DBG(kargs)
 
 
 def check_limits(run, mode, tps, mem, nbp, mpi_nbnoeud, mpi_nbcpu):
@@ -728,19 +771,9 @@ def check_limits(run, mode, tps, mem, nbp, mpi_nbnoeud, mpi_nbcpu):
     configuration file, False else.
     """
     # time
-    err = False
-    tpsmax = 0
-    stps = run[mode+'_tpsmax'].split(':')
     try:
-        m = 1
-        while len(stps)>0:
-            tpsmax += int(stps.pop())*m
-            m = m*60
+        tpsmax = hms2s(run[mode+'_tpsmax'])
     except ValueError:
-        err = True
-    if len(stps)>3:
-        err = True
-    if err:
         run.Mess(_(u'Incorrect value (%s) for %s') % \
                 (str(run[mode+'_tpsmax']), mode+'_tpsmax'), '<F>_CONFIG_ERROR')
     if tps > tpsmax:
@@ -764,20 +797,13 @@ def check_limits(run, mode, tps, mem, nbp, mpi_nbnoeud, mpi_nbcpu):
     if nbp > limit:
         run.Mess(_(u"""Requested number of processors (%s) is higher than the limit (%s)""") %\
                 (nbp, limit), '<E>_INCORRECT_PARA')
-    # mpi nbnoeud/nbcpu
+    # mpi nbcpu
+    para = mode + '_mpi_nbpmax'
     try:
-        limit = int(run.get('mpi_nbnoeudmax', 1))
+        limit = int(run.get(para, 1))
     except ValueError:
         run.Mess(_(u'Incorrect value (%s) for %s') % \
-                (str(run.get('mpi_nbnoeudmax', 1)), 'mpi_nbnoeudmax'), '<F>_CONFIG_ERROR')
-    if mpi_nbnoeud > limit:
-        run.Mess(_(u"""Requested number of MPI nodes (%s) is higher than the limit (%s)""") %\
-                (mpi_nbnoeud, limit), '<E>_INCORRECT_PARA')
-    try:
-        limit = int(run.get('mpi_nbcpumax', 1))
-    except ValueError:
-        run.Mess(_(u'Incorrect value (%s) for %s') % \
-                (str(run.get('mpi_nbcpumax', 1)), 'mpi_nbcpumax'), '<F>_CONFIG_ERROR')
+                (str(run.get(para, 1)), para), '<F>_CONFIG_ERROR')
     if mpi_nbcpu > limit:
         run.Mess(_(u"""Requested number of MPI processors (%s) is higher than the limit (%s)""") %\
                 (mpi_nbcpu, limit), '<E>_INCORRECT_PARA')
