@@ -25,7 +25,7 @@ This module gives functions to manipulate Code_Aster jobs :
 - kill and delete the job files,
 - search strings in output of a job,
 - purge 'flasheur' directory.
-The functions are called by an ASTER_RUN object.
+The functions are called by an AsterRun object.
 """
 
 import os
@@ -37,18 +37,21 @@ import re
 from asrun.common.i18n   import _
 from asrun.common.rcfile import get_nodepara
 from asrun.mystring      import print3, convert, ufmt
-from asrun.profil        import ASTER_PROFIL
+from asrun.profil        import AsterProfil
+from asrun.batch         import BatchSystemFactory
 from asrun.system        import shell_cmd
-from asrun.common_func   import get_tmpname
-from asrun.common.utils  import YES_VALUES
-from asrun.common.sysutils import is_localhost, is_localdisplay
+from asrun.common_func   import get_tmpname, flash_filename, edit_file, is_localhost2
+from asrun.common.utils  import YES_VALUES, dhms2s
+from asrun.common.sysutils import local_user
 from asrun.toolbox       import ConvertToHtml
+from asrun.backward_compatibility import bwc_edit_args
+from asrun.plugins.actions import ACTIONS
 
 
 def SetParser(run):
     """Configure the command-line parser, add options name to store to the list,
     set actions informations.
-    run : ASTER_RUN object which manages the execution
+    run : AsterRun object which manages the execution
     """
     acts_descr = {
         'info' : {
@@ -66,8 +69,8 @@ def SetParser(run):
         },
         'edit' : {
             'method' : Edit,
-            'syntax' : 'job_number job_name mode output|error DISPLAY',
-            'help'   : _(u'Opens output or error file on the provided display')
+            'syntax' : 'job_number job_name mode output|error',
+            'help'   : _(u'Opens output or error file')
         },
         'del' : {
             'method' : Del,
@@ -128,9 +131,9 @@ def Info(run, *args):
     """
     if len(args)>0:
         run.parser.error(_(u"'--%s' requires no argument") % run.current_action)
-    
+
     on_machref = run.get('rep_agla', 'local') != 'local'
-    
+
     # astk server version
     l_v = run.__version__.split('.')
     try:
@@ -139,11 +142,23 @@ def Info(run, *args):
         svers = run.__version__
     output = ["@SERV_VERS@", svers, "@FINSERV_VERS@"]
 
-    # all these parameters are necessary   
+    # all these parameters are necessary
     output.append("@PARAM@")
-    output.extend(['%s : %s' % (p, run[p]) for p in ('plate-forme', 'batch',
-        'batch_memmax', 'batch_tpsmax', 'batch_nbpmax', 'interactif',
-        'interactif_memmax', 'interactif_tpsmax', 'interactif_nbpmax')])
+    output.extend(['%s : %s' % (p, run[p]) \
+        for p in (
+            'protocol_exec', 'protocol_copyto', 'protocol_copyfrom', 'proxy_dir',
+            'plate-forme',
+            'batch',
+            'batch_memmax', 'batch_tpsmax', 'batch_nbpmax', 'batch_mpi_nbpmax',
+            'interactif',
+            'interactif_memmax', 'interactif_tpsmax', 'interactif_nbpmax',
+            'interactif_mpi_nbpmax',
+            )])
+    # and optionnal ones
+    for act in ACTIONS.keys():
+        p = 'schema_%s' % act
+        if run.get(p) is not None and run[p].strip() != '':
+            output.append('%s : %s' % (p, run[p]))
     output.append("@FINPARAM@")
 
     # Code_Aster versions
@@ -159,7 +174,7 @@ def Info(run, *args):
     output.append("@NOEUDS@")
     output.extend(['noeud : %s' % n for n in run.get('noeud', '').split()])
     output.append("@FINNOEUDS@")
-    
+
     # batch scheduler informations
     if run['batch'] in YES_VALUES:
         l_queue_group = run.get('batch_queue_group', '').split()
@@ -176,11 +191,11 @@ def Info(run, *args):
         output.extend(["@REX_URL@", run.get('rex_url', ''), "@FINREX_URL@"])
         output.extend(["@REX_REPFICH@", run.get('rep_rex', ''), "@FINREX_REPFICH@"])
         output.extend(["@MAIL_ATA@", run.get('mail_ata', ''), "@FINMAIL_ATA@"])
-        
+
         # read identAster file
-        f_ident = os.path.join(run['rep_identAster'], 'identAster')
-        if not os.path.isfile(f_ident):
-            run.Sortie(4)
+        f_ident = osp.join(run['rep_identAster'], 'identAster')
+        if not osp.isfile(f_ident):
+            run.Mess(ufmt(_(u'file not found : %s'), f_ident), '<F>_FILE_NOT_FOUND')
         f = open(f_ident, 'r')
         l_ident = [line.strip().replace('\t', ' ') \
                     for line in f if re.search('^[ ]*%', line)==None]
@@ -191,7 +206,7 @@ def Info(run, *args):
             if len(l_i) < 2:
                 continue
             d_user[l_i[1]] = l_i
-        
+
         # extract user info
         role = 'UA'
         details = ''
@@ -201,50 +216,27 @@ def Info(run, *args):
             if len(l) >= 6:
                 role = l[5].replace('/', ' ')
                 details = ('%s %s %s %s' % (l[0], l[1], l[2], l[3])).replace('/', ' ')
-        
+
         output.extend(["@ROLE@", role, "@FINROLE@"])
         if details:
             output.extend(["@INFOID@", details, "@FININFOID@"])
 
     # message of the day
     output.append("@MOTD@")
-    if run.get('motd', '') != '' and os.path.exists(run['motd']):
+    if run.get('motd', '') != '' and osp.exists(run['motd']):
         output.append(open(run['motd'], 'r').read())
     output.append("@FINMOTD@")
-    
+
     print3(os.linesep.join(output))
 
 
-def _sumtps(l_scpu):
-    """Return the sum in seconds of the times given as strings in `l_scpu`.
-    Format of each times in [dd:[hh:[mm:[ss]]]]
-    """
-    tot = 0
-    if len(l_scpu) < 1:
-        tot = '_'
-    for scpu in l_scpu:
-        try:
-            s = [int(s) for s in scpu.split(':')]
-        except ValueError:
-            s = []
-        if   len(s) == 4:
-            tot += 86400*s[0]+3600*s[1]+60*s[2]+s[3]
-        elif len(s) == 3:
-            tot += 3600*s[0]+60*s[1]+s[2]
-        elif len(s) == 2:
-            tot += 60*s[0]+s[1]
-        elif len(s) == 1:
-            tot += s[0]
-    return tot
-
-
 def Func_actu(run, *args):
     """Return state, diagnosis, node, cpu time and working directory of a job.
     """
     if len(args) != 3:
         run.parser.error(_(u"'--%s' takes exactly %d arguments (%d given)") % \
             (run.current_action, 3, len(args)))
-    
+
     njob, nomjob, mode = args
     # defaults
     etat  = '_'
@@ -257,127 +249,48 @@ def Func_actu(run, *args):
     on_machref = run.get('rep_agla', 'local') != 'local'
     # the real job id may differ
     jobid = str(njob)
-    
+    # astk profile
+    pr_astk = osp.join(run['flasheur'], '%s.p%s' % (nomjob, njob))
+    prof = None
+    if osp.isfile(pr_astk):
+        prof = AsterProfil(pr_astk, run)
+        wrk = prof['rep_trav'][0] or '_'
+
     # 1. get information about the job
     # 1.1. batch mode
     if mode == "batch":
         m = 'b'
-        # for LSF
-        if run['batch_nom'] == 'LSF':
-            cmd = ''
-            if run.get('batch_ini', '') != '':
-                cmd = '. %s ; ' % run['batch_ini']
-            cmd += '%s -uall' % run['batch_job']
-            jret, out = run.Shell(cmd)
-            mat = re.search('(^ *%s.*)' % njob, out, re.MULTILINE)
-            if mat == None:
-                etat = "ENDED"
-            else:
-                lin = mat.group(1).split()
-                if len(lin) >= 3:
-                    etat = lin[2]
-                    if etat != "PEND":
-                        node = re.split('[*@]+', lin[5])[-1]
-                if etat.find('SUSP')>-1:
-                    etat = 'SUSPENDED'
-        # for PBS
-        elif run['batch_nom'] == 'PBS':
-            cmd = ''
-            if run.get('batch_ini', '') != '':
-                cmd = '. %s ; ' % run['batch_ini']
-            cmd += '%s -f %s' % (run['batch_job'], njob)
-            jret, out = run.Shell(cmd)
-            # real job id
-            mjid = re.search('Job Id: *(\S+)', out, re.MULTILINE)
-            if mjid != None:
-                jobid = mjid.group(1)
-            # job state
-            metat = re.search(' job_state *= *(\S+)', out, re.MULTILINE)
-            if metat == None:
-                etat = "ENDED"
-            else:
-                etat = { 'R' : 'RUN', 'E'  :'RUN',
-                            'W' : 'PEND', 'T' : 'PEND', 'Q' : 'PEND',
-                            'S' : 'SUSPENDED', 'H' : 'SUSPENDED',
-                   }.get(metat.group(1), '?')
-            # queue
-            mqueue = re.search(' queue *= *(\S+)', out, re.MULTILINE)
-            if mqueue != None:
-                queue = mqueue.group(1)
-            # exec host
-            mnode = re.search(' exec_host *= *(\S+)/', out, re.MULTILINE)
-            if mnode != None:
-                node = mnode.group(1)
-        # for Sun Grid Engine
-        elif run['batch_nom'] == 'SunGE':
-            cmd = ''
-            if run.get('batch_ini', '') != '':
-                cmd = '. %s ; ' % run['batch_ini']
-            cmd += run['batch_job']
-            jret, out = run.Shell(cmd)
-            run.DBG(out, all=True)
-            mat = re.search('(^ *%s.*)' % njob, out, re.MULTILINE)
-            if mat == None:
-                etat = "ENDED"
-            else:
-                lin = mat.group(1).split()
-                if len(lin) >= 5:
-                    etat = lin[4]
-                    if re.search('[wh]+', etat) != None:
-                        etat = "PEND"
-                    elif re.search('[sST]+', etat) != None:
-                        etat = "SUSPENDED"
-                    else:
-                        etat = "RUN"
-                    if len(lin) >= 8:
-                        l_q = lin[7].split('@')
-                        queue = l_q[0]
-                        if len(l_q) > 1:
-                            node  = l_q[1].split('.')[0]
-                # because I don't known the running node on which to call 'ps'
-                if etat == 'RUN' and node in ('', '_'):
-                    cmd = ''
-                    if run.get('batch_ini', '') != '':
-                        cmd = '. %s ; ' % run['batch_ini']
-                    cmd += '%s -j %s' % (run['batch_job'], njob)
-                    jret, out = run.Shell(cmd)
-                    expr = re.compile('^usage *[0-9]* *: *cpu=([0-9:]+)', re.MULTILINE)
-                    l_field = expr.findall(out)
-                    tcpu = _sumtps(l_field)
-        else:
-            run.Mess(_(u'unknown batch scheduler'), '<F>_ERROR')
+        scheduler = BatchSystemFactory(run, prof)
+        etat, diag, node, tcpu, wrk, queue = scheduler.get_jobstate(njob, nomjob)
 
     # 1.2. interactive mode
     elif mode == "interactif":
         m = 'i'
-        # astk profile
-        pr_astk = os.path.join(run['flasheur'], '%s.p%s' % (nomjob, njob))
         # if it doesn't exist the job is ended
         etat = "ENDED"
-        if os.path.isfile(pr_astk):
-            prof = ASTER_PROFIL(pr_astk, run)
+        if prof is not None:
             node = prof['noeud'][0]
-    
     else:
         run.Mess(_(u'unexpected mode : %s') % mode, '<F>_UNEXPECTED_VALUE')
 
     # 2. query the process
     if node != '_':
-        jret, psout = run.Shell(run['ps_cpu'], mach=node)
+        if mode == "interactif" or tcpu == '_':
+            jret, psout = run.Shell(run['ps_cpu'], mach=node)
         # ended ?
         if mode == "interactif" and psout.find('btc.%s' % njob) > -1:
             etat = "RUN"
 
     # 3.1. the job is ended
     if etat == "ENDED":
-        fdiag = os.path.join(run['flasheur'], '%s.%s%s' % (nomjob, m, njob))
-        if os.path.isfile(fdiag):
+        fdiag = osp.join(run['flasheur'], '%s.%s%s' % (nomjob, m, njob))
+        if osp.isfile(fdiag):
             diag = open(fdiag, 'r').read().split(os.linesep)[0] or "?"
         if diag == '?' :
             diag = '<F>_SYSTEM'
             # try to find something in output
-            fout = os.path.join(run['flasheur'], '%s.o%s' % (nomjob, njob))
-            if os.path.isfile(fout):
+            fout = osp.join(run['flasheur'], '%s.o%s' % (nomjob, njob))
+            if osp.isfile(fout):
                 f = open(fout, 'r')
                 for line in f:
                     if line.find('--- DIAGNOSTIC JOB :')>-1:
@@ -385,14 +298,14 @@ def Func_actu(run, *args):
                     elif line.find('Cputime limit exceeded')>-1:
                         diag = '<F>_CPU_LIMIT_SYSTEM'
                 f.close()
-            # working directory
-            wrk = get_nodepara(node, 'rep_trav', run['rep_trav'])
             # copy fort.6 to '.o'
             if node != '_':
                 ftcp = get_tmpname(run, run['tmp_user'], basename='actu')
-                fort6 = os.path.join(wrk, '%s.%s.fort.6.%s' % (nomjob, njob, m))
+                # same name as in the btc script generated by calcul.py
+                wrk6 = get_nodepara(node, 'rep_trav', run['rep_trav'])
+                fort6 = osp.join(wrk6, '%s.%s.fort.6.%s' % (nomjob, njob, m))
                 jret = run.Copy(ftcp, '%s:%s' % (node, fort6), niverr='SILENT')
-                if os.path.isfile(ftcp):
+                if osp.isfile(ftcp):
                     txt = [os.linesep*2]
                     txt.append('='*48)
                     txt.append('===== Pas de diagnostic, recopie du fort.6 =====')
@@ -404,16 +317,12 @@ def Func_actu(run, *args):
                     f = open(fout, 'a')
                     f.write(os.linesep.join(txt))
                     f.close()
-    
     else:
     # 3.2. job is running
         if etat in ('RUN', 'SUSPENDED'):
             # working directory
-            wrk = get_nodepara(node, 'rep_trav', run['rep_trav'])
-            if not on_machref or mode == 'interactif':
+            if wrk == '_':
                 wrk = get_tmpname(run, basename=mode, node=node, pid=njob)
-            else:
-                wrk = os.path.join(wrk, str(jobid))
         if etat == 'RUN' and tcpu == '_':
             # tcpu may have been retrieved upper
             l_tcpu = []
@@ -422,59 +331,43 @@ def Func_actu(run, *args):
                     re.search('\-mode +%s' % mode, line) != None and \
                     (not on_machref or line.find(run['rep_agla'])<0):
                     l_tcpu.append(re.sub('\..*$', '', line.split()[0]).replace('-', ':'))
-            if len(l_tcpu)>0:
-                tcpu = _sumtps(l_tcpu)
-    
+            if len(l_tcpu) > 0:
+                tcpu = dhms2s(l_tcpu)
+
     # 4. return the result
     if node == "":
         node = "_"
     return etat, diag, node, tcpu, wrk, queue
-    
+
 
 def Actu(run, *args):
     """Return state, diagnosis, node, cpu time and working directory of a job.
     """
-    print3("ETAT=%s DIAG=%s EXEC=%s TCPU=%s REP_TRAV=%s QUEUE=%s" % Func_actu(run, *args))
+    print_actu_result(*Func_actu(run, *args))
 
 
 def Edit(run, *args):
     """Open output or error file of a job.
     """
-    if len(args) != 5:
-        run.parser.error(_(u"'--%s' takes exactly %d arguments (%d given)") % \
-            (run.current_action, 5, len(args)))
+    args = bwc_edit_args(args)
+    if len(args) < 4:
+        run.parser.error(_(u"'--%s' takes at least %d arguments (%d given)") % \
+            (run.current_action, 4, len(args)))
+
+    njob, nomjob, mode, typ = args
 
-    njob, nomjob, mode, typ, displ = args
-    d_ext = { 'output' : 'o', 'error' : 'e', 'export' : 'p',
-             'script' : 'u', 'diag'  : mode[0] }
-    ext = d_ext.get(typ, 'X')
-    
-    # get editor command line
-    edit = run.get('editor')
-    # backwards compatibility
-    if not edit:
-        edit = run['editeur']
-    
     # filename to edit
-    fich = os.path.join(run['flasheur'], '%s.%s%s' % (nomjob, ext, njob))
-    run.DBG('filename =', fich)
-    
-    if os.path.isfile(fich):
+    fname = flash_filename(run['flasheur'], nomjob, njob, typ, mode)
+    run.DBG('filename =', fname)
+
+    if osp.isfile(fname):
     # write file content to stdout
         if run.get('result_to_output'):
             run.PrintExitCode = False
-            print3(open(fich, "r").read())
+            print3(open(fname, "r").read())
         # edit the file
         else:
-            if re.search('@D', edit) != None:
-                edit = edit.replace('@D', displ)
-            elif edit.find('display') < 0 and not is_localdisplay(displ):
-                edit = '%s -display %s' % (edit, displ)
-            else:
-                pass
-
-            cmd = '%s %s' % (edit, fich)
-            kret, out = run.Shell(cmd, bg=True)
+            edit_file(run, fname)
     else:
         run.Sortie(4)
 
@@ -488,7 +381,7 @@ def Del(run, *args, **kwargs):
     elif len(args) > 4:
         run.parser.error(_(u"'--%s' takes at most %d arguments (%d given)") % \
             (run.current_action, 4, len(args)))
-    
+
     # 0. arguments
     njob, nomjob, mode = args[:3]
     if len(args) > 3:
@@ -498,29 +391,30 @@ def Del(run, *args, **kwargs):
     sent_signal = run['signal']
     if kwargs.get('signal'):
         sent_signal = kwargs['signal']
-    
-    use_batch_cmd = (mode == "batch" and sent_signal == 'KILL')
-        
+    delete_files = sent_signal == 'KILL'
+
+    use_batch_cmd = False
+    scheduler = None
+    if mode == 'batch':
+        scheduler = BatchSystemFactory(run)
+        use_batch_cmd = sent_signal == 'KILL' or scheduler.supports_signal()
+
     # 1. retrieve the job status
     etat, diag, node, tcpu, wrk, queue = Func_actu(run, njob, nomjob, mode)
     run.DBG(u"actu returns : etat/diag/node/tcpu/wrk/queue", (etat, diag, node, tcpu, wrk, queue))
-    
+
     # 2. send the signal
     if etat in ('RUN', 'SUSPENDED', 'PEND'):
         if use_batch_cmd:
-            if run['batch_nom'] in ('LSF', 'PBS', 'SunGE'):
-                cmd = ''
-                if run.get('batch_ini', '') != '':
-                    cmd = '. %s ; ' % run['batch_ini']
-                cmd += '%s %s' % (run['batch_kil'], njob)
-                iret, out = run.Shell(cmd)
-                if iret != 0:
-                    run.Sortie(4)
-            else:
-                run.Mess(_(u'unknown batch scheduler'), '<F>_ERROR')
+            iret = scheduler.signal_job(njob, sent_signal)
+            if iret != 0:
+                run.Sortie(4)
         else:
             numpr, psout = '', ''
             # get process id
+            if node == '_':
+                # try on localhost
+                node = ''
             if node != '_':
                 jret, psout = run.Shell(run['ps_pid'], mach=node)
                 exp = re.compile('^ *([0-9]+) +(.*)\-num_job +%s.*\-mode +%s' % (njob, mode),
@@ -534,20 +428,32 @@ def Del(run, *args, **kwargs):
                         numpr = int(numj)
                         run.DBG(u"Signal will be sent to process : %s" % numpr)
                         break
+                if numpr == '':
+                    # try to kill its as_run parent (but USR1 will not stop as_run)
+                    sent_signal = 'KILL'
+                    exp = re.compile('^ *([0-9]+) +(.*as_run.*\-\-num_job=+%s.*)' % njob, re.M)
+                    res = exp.findall(psout)
+                    res.reverse()  # the relevant process should be the last one
+                    run.DBG(u"as_run processes :", res)
+                    for numj, cmd in res:
+                        # python run by bin/as_run
+                        if cmd.find('python') > -1:
+                            numpr = int(numj)
+                            run.DBG(u"Signal will be sent to process : %s" % numpr)
+                            break
             if numpr != '':
-                if is_localhost(node):
+                if is_localhost2(node):
                     os.kill(numpr, getattr(signal, 'SIG%s' % sent_signal))
                 else:
                     iret, psout = run.Shell('kill -%s %s' % (sent_signal, numpr), mach=node)
             else:
                 run.DBG(u'<job.Del> process not found :' , psout, u'node = %s' % node, all=True)
-    
+
     # 3. delete files
-    if sent_signal == 'KILL':
-        l_fich = glob.glob(os.path.join(run['flasheur'], '%s.?%s' % (nomjob, njob)))
+    if delete_files:
+        l_fich = glob.glob(osp.join(run['flasheur'], '%s.?%s' % (nomjob, njob)))
         for f in l_fich:
             run.Delete(f)
-    
 
 
 def Func_tail(run, njob, nomjob, mode, nbline, expression=None):
@@ -559,6 +465,7 @@ def Func_tail(run, njob, nomjob, mode, nbline, expression=None):
         cmd = 'egrep -- \'%s\' ' % expression
     # retrieve the job status
     etat, diag, node, tcpu, wrk, queue = Func_actu(run, njob, nomjob, mode)
+    run.DBG(u"actu returns : etat/diag/node/tcpu/wrk/queue", (etat, diag, node, tcpu, wrk, queue))
     # fill output file
     s_out = ''
     if mode == 'batch' and run['batch_nom'] == 'SunGE':
@@ -566,7 +473,7 @@ def Func_tail(run, njob, nomjob, mode, nbline, expression=None):
                 "scheduler the running node.")
     if etat == 'RUN':
         # file to parse
-        fich = os.path.join(wrk, 'fort.6')
+        fich = osp.join(wrk, 'fort.6')
         run.DBG(ufmt(u'path to fort.6 : %s', fich))
         if node != '_':
             mach = node
@@ -579,7 +486,6 @@ def Func_tail(run, njob, nomjob, mode, nbline, expression=None):
     return etat, diag, s_out
 
 
-
 def Tail(run, *args):
     """Output the last part of fort.6 file or filter lines matching a pattern.
     """
@@ -589,30 +495,36 @@ def Tail(run, *args):
     elif len(args) > 6:
         run.parser.error(_(u"'--%s' takes at most %d arguments (%d given)") % \
             (run.current_action, 6, len(args)))
-    
+
     # arguments
     njob, nomjob, mode, fdest, nbline = args[:5]
     expression = None
     if len(args) > 5:
         expression = args[5]
-    
+
     etat, diag, s_out = Func_tail(run, njob, nomjob, mode, nbline, expression)
     if s_out == "":
         run.Mess(_(u'file is empty'))
 
-    print3("JOB=%s JOBID=%s ETAT=%s DIAG=%s" % (nomjob, njob, etat, diag))
+    print_tail_result(nomjob, njob, etat, diag)
     # exit if job isn't running
+    run.PrintExitCode = False
     if etat != 'RUN':
         run.Sortie(4)
-    
-    # send output file
-    if run.IsRemote(fdest):
-        ftmp = get_tmpname(run, run['tmp_user'], basename='tail')
-        open(ftmp, 'w').write(convert(s_out))
-        jret = run.Copy(fdest, ftmp)
+
+    if run.get('result_to_output') or fdest == 'None':
+        run.DBG(_(u'tail put to output'))
+        print3(s_out)
     else:
-        fdest = run.PathOnly(fdest)
-        open(fdest, 'w').write(convert(s_out))
+        # send output file
+        if run.IsRemote(fdest):
+            ftmp = get_tmpname(run, run['tmp_user'], basename='tail')
+            open(ftmp, 'w').write(convert(s_out))
+            jret = run.Copy(fdest, ftmp)
+        else:
+            fdest = run.PathOnly(fdest)
+            open(fdest, 'w').write(convert(s_out))
+            run.DBG(ufmt(u'output written to : %s', fdest))
 
 
 def Purge(run, *args):
@@ -621,32 +533,36 @@ def Purge(run, *args):
     if len(args) < 1:
         run.parser.error(_(u"'--%s' takes at least %d arguments (%d given)") % \
             (run.current_action, 1, len(args)))
-    
-    # 0. arguments
-    njob = len(args)
-    l_job = []
-    for j in args:
-        try:
-            l_job.append(int(j))
-        except ValueError:
-            run.parser.error(_(u'incorrect job number : %s') % j)
 
-    # 1. action
     l_keep = []
     l_f = os.listdir(run['flasheur'])
     for f in l_f:
         delete = True
-        for j in l_job:
-            if re.search('\.[a-z]?%d$' % j, f) != None:
+        for j in args:
+            if re.search('\.[a-z]?%s$' % j, f) != None:
                 delete = False
                 break
         if delete:
-            run.Delete(os.path.join(run['flasheur'], f), verbose=True)
+            run.Delete(osp.join(run['flasheur'], f), verbose=True)
+
+
+def print_actu_result(*args):
+    """Print the result of actu command."""
+    print3("ETAT=%s DIAG=%s EXEC=%s TCPU=%s REP_TRAV=%s QUEUE=%s" % args)
+
+
+def print_tail_result(*args):
+    """Print the result of tail command."""
+    print3("JOB=%s JOBID=%s ETAT=%s DIAG=%s" % args)
 
 
 def parse_actu_result(txt):
     """Decode output of the Actu function.
     """
+    if txt.strip() == '':
+        # it means that we cannot retreive the job status
+        resu = 'ENDED', '_', '_', '_', '_', '_'
+        return resu
     resu = ("_",) * 6
     expr = re.compile("ETAT=(.+) +DIAG=(.+) +EXEC=(.+) +TCPU=(.+) +REP_TRAV=(.+) +QUEUE=(.+)")
     mat = expr.search(txt)
