@@ -20,7 +20,7 @@
 """
 Tools to maintain a development version of Code_Aster, and useful for
 the developper.
-Methods are called by an ASTER_RUN object.
+Methods are called by an AsterRun object.
 """
 
 import os
@@ -32,29 +32,31 @@ import cPickle
 import traceback
 from optparse  import SUPPRESS_HELP
 from pprint    import pformat
-EnumTypes = (list, tuple)
+from functools import partial
 
 from asrun.installation import aster_root
 from asrun.common.i18n import _
 from asrun.mystring     import print3, ufmt
-from asrun.run          import RunAsterError
-from asrun.config       import ASTER_CONFIG
-from asrun.build        import ASTER_BUILD
+from asrun.core         import RunAsterError, magic
+from asrun.config       import AsterConfig
+from asrun.build        import AsterBuild
 from asrun.system       import local_host
 from asrun.update       import AstkUpdate, read_version_list, download_package
 from asrun.execution    import build_test_export
 from asrun.testlist     import TestList
 from asrun.mpi          import MPI_INFO
 from asrun.toolbox      import GetInfos
-from asrun.common_func  import get_tmpname, get_devel_param
+from asrun.common_func  import get_tmpname, get_devel_param, edit_file
 from asrun.dev          import GetMessageInfo, FreeSubroutines
 from asrun.common.utils import get_list, version2tuple, tuple2version, remove_empty_dirs
 
+from asrun.backward_compatibility import bwc_getop, bwc_get_version
+
 
 def SetParser(run):
     """Configure the command-line parser, add options name to store to the list,
     set actions informations.
-    run : ASTER_RUN object which manages the execution
+    run : AsterRun object which manages the execution
     """
     acts_descr = {
         'getop'        : {
@@ -62,6 +64,11 @@ def SetParser(run):
             'syntax' : '[options] commande[.capy]',
             'help'   : _(u'Return the main subroutine of a Code_Aster command')
         },
+        'showop'        : {
+            'method' : ShowOP,
+            'syntax' : '[options] commande[.capy]',
+            'help'   : _(u'Show the main subroutine of a Code_Aster command')
+        },
         'show'         : {
             'method' : Show,
             'syntax' : '[options] obj1 [obj2...]',
@@ -90,9 +97,9 @@ def SetParser(run):
         },
         'auto_update'  : {
             'method' : AutoUpdate,
-            'syntax' : '[--vers=...] [--force_upgrade] [--keep_increment] [--local]',
+            'syntax' : '[--vers=...] [--force_upgrade] [--keep_increment] [--local] [last_version]',
             'help'   : _(u'Download available updates from a server and apply them to the ' \
-                          'current development version')
+                          'current development version up to `last_version`.')
         },
         'astk_update'  : {
             'method' : AstkUpdate,
@@ -102,7 +109,12 @@ def SetParser(run):
         'getversion'   : {
             'method' : GetVersion,
             'syntax' : '[options]',
-            'help'   : _(u'Return current release number of the development version')
+            'help'   : _(u'Return current release number of the default version')
+        },
+        'getversion_path'   : {
+            'method' : GetVersionPath,
+            'syntax' : '[options]',
+            'help'   : _(u'Return the path of the default version')
         },
         'diag'         : {
             'method' : MakeDiag,
@@ -177,7 +189,7 @@ def SetParser(run):
                 'action'  : 'store',
                 'dest'    : 'aster_vers',
                 'metavar' : 'VERS',
-                'help'    : _(u'Code_Aster version to used (for get, show, getop)')
+                'help'    : _(u'Code_Aster version to used (for get, show, showop)')
             }
         },
         'version_dev' : {
@@ -361,16 +373,19 @@ def SetParser(run):
     title = _(u'Options for maintenance operations')
     run.SetActions(
             actions_descr=acts_descr,
-            actions_order=['show', 'get', 'diff', 'getop', 'get_export',
-                    'list', 'diag', 'messages', 'get_infos', 
-                    'getversion', 'make', 'update', 'auto_update', 'astk_update',
-                    'ctags', 'free_sub'],
+            actions_order=['show', 'get', 'diff', 'showop', 'get_export',
+                    'free_sub', 'list', 'diag', 'messages', 'get_infos',
+                    'getversion', 'getversion_path',
+                    'make', 'update', 'auto_update', 'astk_update',
+                    'ctags',
+                    # backward compatibility
+                    'getop'],
             group_options=True, group_title=title, actions_group_title=False,
             options_descr=opts_descr
     )
 
 
-def GetOP(run, *list_capy):
+def ShowOP(run, *list_capy):
     """Return the main subroutine of a command.
     """
     run.check_version_setting()
@@ -380,29 +395,47 @@ def GetOP(run, *list_capy):
                 _(u"'--%s' requires one or more arguments") % run.current_action)
     run.PrintExitCode = False
     ficconf = os.path.join(REPREF, 'config.txt')
-    conf = ASTER_CONFIG(ficconf, run)
+    conf = AsterConfig(ficconf, run)
     for capy in list_capy:
         capy = capy.replace('.capy', '')
-        nf = os.path.join(REPREF, conf['SRCCAPY'][0], 'commande', capy+'.capy')
+        nf = os.path.join(REPREF, conf['SRCCAPY'][0], 'commande', capy + '.capy')
         if not os.path.exists(nf):
             run.Mess(ufmt(_(u'file not found : %s'), nf), '<A>_ALARM')
             run.Mess(_(u"'--%s' search capy files on local machine") \
                     % run.current_action)
             break
         f = open(nf)
-        nop = re.search('op *= *([-_0-9a-zA-Z]+)', f.read())
+        txt = f.read()
+        name = None
+        nop = re.search('op *= *([-0-9]+)', txt)
         if nop:
             name = nop.group(1)
-            if name.isdigit():
-                num = '%04d' % int(name)
-                name = 'op'+num+'.f'
-            else:
+            fmt = 'op%04d.f'
+            if name[0] == '-':
+                fmt = 'ops%03d.f'
+                name = name[1:]
+            name = fmt % int(name)
+        else:
+            nop = re.search('op *= *OPS *\([\'\"]\w+\.(\w+)\.\w+[\'\"]\)', txt)
+            if nop:
+                name = nop.group(1)
+                print name
                 name = name+'.py'
+            else:
+                # for Code_Aster version <= 11.0.0
+                nop = re.search('op *= *(\w+)', txt)
+                if nop:
+                    name = nop.group(1) + '.py'
+        if name:
             Show(run, name)
         else:
             run.Mess(ufmt(_(u'op statement not found in %s'), capy), '<A>_ALARM')
 
 
+def GetOP(run, *list_capy):
+    bwc_getop(ShowOP, run, *list_capy)
+
+
 def Diff(run, *args):
     """Show the diff of a source file.
     """
@@ -459,7 +492,7 @@ def Get(run, *args, **kwargs):
         REPREF = run.get_version_path(run['aster_vers'])
         # répertoires
         ficconf = os.path.join(REPREF, 'config.txt')
-        conf  = ASTER_CONFIG(ficconf, run)
+        conf  = AsterConfig(ficconf, run)
         RC    = os.path.join(REPREF, conf['SRCC'][0])
         RFORT = os.path.join(REPREF, conf['SRCFOR'][0])
         RF90  = os.path.join(REPREF, conf['SRCF90'][0])
@@ -476,7 +509,7 @@ def Get(run, *args, **kwargs):
     if not get:
         rdest = run['cache_dir']
     else:
-        rdest = os.getcwd()
+        rdest = os.getcwdu()
     machdest = run.system.getuser_host()[1]
 
     fmt = '-- %-44s [%+30s]'
@@ -541,7 +574,7 @@ def Get(run, *args, **kwargs):
             or re.search('com[0-9]', ext) != None or ext.isdigit():
             rep = RTEST
             ct = 1
-        
+
         niverr_cp = '<E>_COPY_ERROR'
         if silent:
             niverr_cp = 'SILENT'
@@ -624,10 +657,7 @@ def Get(run, *args, **kwargs):
                         else:
                             s_edit = ' '.join(toedit)
                             seen.update(toedit)
-                            cmd = '%s %s' % (run['editor'], s_edit)
-                            if re.search('@D', cmd) != None:
-                                cmd = cmd.replace('@D', os.environ.get('DISPLAY', ':0'))
-                            jret, output = run.Shell(cmd, bg=True)
+                            edit_file(run, s_edit)
             else:
                 # fichier inexistant
                 run.DBG("search '%s' from '%s' : not found" % (obj, rep))
@@ -655,13 +685,13 @@ def GetVersion(run, *args, **kwargs):
     # ----- default keywords
     run.PrintExitCode = False
     silent = False
-    REPREF = run.get_version_path(run['aster_vers'])
+    vers = run['aster_vers']
     if kwargs.has_key('silent'):
         silent = kwargs['silent']
-    if kwargs.has_key('REPREF'):
-        REPREF = kwargs['REPREF']
-    
-    result = get_version(run, REPREF)
+    if kwargs.has_key('vers'):
+        vers = kwargs['vers']
+
+    result = get_aster_version(vers)
     iret = 0
     if len(result) != 5:
         iret = 4
@@ -673,19 +703,32 @@ def GetVersion(run, *args, **kwargs):
         run.Mess(_(u'Version %s %s - %s') % (typv, '.'.join(result[:3]), result[3]))
     return iret, result
 
+def GetVersionPath(run, *args, **kargs):
+    """Return the path of the default version."""
+    run.check_version_setting()
+    if len(args) > 0:
+        run.parser.error(_(u"'--%s' requires no argument") % run.current_action)
+    run.PrintExitCode = False
+    # ----- default keywords
+    run.PrintExitCode = False
+    vers = run['aster_vers']
+    repref = run.get_version_path(vers)
+    print3(repref)
 
-def get_version(run, REPREF):
-    """Return the Code_Aster version installed in REPREF.
+def get_aster_version(vers):
+    """Return the Code_Aster version named `vers`.
     """
-    ficconf = os.path.join(REPREF, 'config.txt')
+    run = magic.run
+    repref = run.get_version_path(vers)
+    ficconf = os.path.join(repref, 'config.txt')
     # if file don't exist, suppose default value
     if os.path.exists(ficconf):
-        conf = ASTER_CONFIG(ficconf, run)
+        conf = AsterConfig(ficconf, run)
         bibpyt = conf['SRCPY'][0]
     else:
         bibpyt = 'bibpyt'
     # ----- properties.py
-    f = os.path.join(REPREF, bibpyt, 'Accas', 'properties.py')
+    f = os.path.join(repref, bibpyt, 'Accas', 'properties.py')
     if not os.path.isfile(f):
         run.Mess(ufmt(_(u'file not found : %s'), f), '<F>_FILE_NOT_FOUND')
     mydict = {}
@@ -695,6 +738,9 @@ def get_version(run, REPREF):
     result.append(mydict.get('exploit', False))
     return result
 
+# for backward compatibility
+get_version = partial(bwc_get_version, get_aster_version)
+
 
 def MakeAster(run, *args):
     """Interface between Makefile and asrun.
@@ -706,7 +752,7 @@ def MakeAster(run, *args):
         fconf = os.path.join(REPREF, 'config.txt')
     else:
         fconf = os.path.abspath(run['config'])
-    conf = ASTER_CONFIG(fconf, run)
+    conf = AsterConfig(fconf, run)
 
     # check arguments
     target = 'all'
@@ -747,7 +793,6 @@ def MakeAster(run, *args):
                         (run.get_version_path(run['aster_vers']), local_host)
             run.Mess(errmsg, '<F>_BUILD_FAILED')
         else:
-            run.ExitOnFatalError = True
             subject = _(u'Built of %s ended successfully on %s') \
                         % (run.get_version_path(run['aster_vers']), local_host)
             mail.append(subject)
@@ -757,7 +802,7 @@ def MakeAster(run, *args):
             run.SendMail(dest=run['report_to'],
                 text = os.linesep.join(mail),
                 subject = subject)
-    
+
     elif target == 'clean':
         if len(param) == 0:
             l_clean = ['BIN_NODBG', 'BIN_DBG', 'BINCMDE', 'BINELE', 'BINPICKLED',
@@ -780,7 +825,7 @@ def MakeAster(run, *args):
                 for dobj in ('BINOBJ_NODBG', 'BINOBJ_DBG'):
                     ficrep = conf[dobj][0]
                     if ficrep != '':
-                        run.VerbStart(ufmt(_(u'remove %d object files of %s from %s'), len(lo), 
+                        run.VerbStart(ufmt(_(u'remove %d object files of %s from %s'), len(lo),
                                       dsrc, ficrep), verbose=True)
                         for fo in lo:
                             run.Delete(os.path.join(REPREF, ficrep, fo))
@@ -796,7 +841,7 @@ def _build_aster(run, conf, iupdate, REPREF, destdir, reptrav, lardv=None):
     """
     if lardv == None:
         lardv = []
-    build = ASTER_BUILD(run, conf)
+    build = AsterBuild(run, conf)
     DbgPara = {
         'debug'     : { 'exe'    : conf['BIN_DBG'][0],
                       'suff'   : conf['BINOBJ_DBG'][0],
@@ -816,7 +861,7 @@ def _build_aster(run, conf, iupdate, REPREF, destdir, reptrav, lardv=None):
     # ----- Initialize MPI_INFO object
     mpi_info = MPI_INFO(conf.get_defines())
     mpi_info.set_cpuinfo(1, 1)
-    mpi_info.set_rep_trav(run, reptrav)
+    reptrav = mpi_info.set_rep_trav(reptrav)
 
     # 1a. ----- perform update twice : debug and nodebug modes
     # first, remove object files from libaster
@@ -849,7 +894,7 @@ def _build_aster(run, conf, iupdate, REPREF, destdir, reptrav, lardv=None):
                 run.VerbIgnore(verbose=True)
             run.timer.Stop(tit)
             run.CheckOK()
-    
+
     # 1b. ----- perform update twice : debug and nodebug modes
     for mode in debug_mode:
         libaster = os.path.join(destdir, DbgPara[mode]['libast'])
@@ -859,7 +904,7 @@ def _build_aster(run, conf, iupdate, REPREF, destdir, reptrav, lardv=None):
             libaster = os.path.join(libaster, 'lib_aster.lib')
         if run.IsDir(libferm):
             libferm = os.path.join(libaster, 'ferm.lib')
-        run.Mess(_(u'Start build in %s mode') % mode, 'TITLE')      
+        run.Mess(_(u'Start build in %s mode') % mode, 'TITLE')
 
         # 1.2. compile updated files
         tit = _(u'Compilation in %s mode') % mode
@@ -891,7 +936,7 @@ def _build_aster(run, conf, iupdate, REPREF, destdir, reptrav, lardv=None):
         kret = build.Link(exec_name, [], libaster, libferm, reptrav)
         run.timer.Stop(tit)
         run.CheckOK()
-        
+
         # 1.4.2. testing binary
         # - executable is callable (no shared library unreferenced)
         # - required Python modules are present
@@ -912,7 +957,7 @@ except:
         run.VerbStart(ufmt(_(u'testing executable %s...'), exec_name), verbose=True)
         if run['verbose']:
             print3()
-        cmd_exec = mpi_info.get_exec_command(run, '%s "%s"' % (cmd, ''.join(cpyt)),
+        cmd_exec = mpi_info.get_exec_command('%s "%s"' % (cmd, ''.join(cpyt)),
             env=conf.get_with_absolute_path('ENV_SH'))
         kret, out = run.Shell(cmd_exec)
         # ----- avoid to duplicate output
@@ -978,8 +1023,8 @@ def Update(run, *args, **kwargs):
         fconf = os.path.join(REPREF, 'config.txt')
     else:
         fconf = os.path.abspath(run['config'])
-    conf   = ASTER_CONFIG(fconf, run)
-    build  = ASTER_BUILD(run, conf)
+    conf   = AsterConfig(fconf, run)
+    build  = AsterBuild(run, conf)
 
     # check arguments
     if len(args) < 1:
@@ -988,7 +1033,7 @@ def Update(run, *args, **kwargs):
 
     # id
     num_update = kwargs.get('num_update', 1)
-    
+
     run.PrintExitCode = False
     # set per version environment
     for f in conf.get_with_absolute_path('ENV_SH'):
@@ -1081,17 +1126,17 @@ def Update(run, *args, **kwargs):
 
     # 2. ----- start update
     # current version
-    i, vvv = GetVersion(run, silent=True, REPREF=REPREF)
+    i, vvv = GetVersion(run, silent=True, vers=run['aster_vers'])
     if i != 0:
         run.Mess(_(u'error occurs during getting release number'), '<F>_ERROR')
     old_vers = '.'.join(vvv[:3])
     # target version
-    i, vnext = GetVersion(run, silent=True, REPREF=repmaj)
+    i, vnext = GetVersion(run, silent=True, vers=repmaj)
     if i != 0:
         run.Mess(_(u'error occurs during getting release number'), '<F>_ERROR')
     new_vers = '.'.join(vnext[:3])
     run.Mess(_(u'Update version %s to %s') % (old_vers, new_vers), 'TITLE')
-    
+
     # 3. ----- copy files
     tit = _(u'Copy of updated files')
     run.timer.Start(tit)
@@ -1116,12 +1161,12 @@ def Update(run, *args, **kwargs):
             continue
         rep  = param[0]
         l_suff = param[1]
-        if not type(l_suff) in EnumTypes:
+        if not type(l_suff) in (list, tuple):
             l_suff = [l_suff,]
         for suff in l_suff:
             if rep == 'regexp':
                 # files by regular expression
-                expr = re.compile(l_suff[0])
+                expr = re.compile(suff)
                 l_files = [f for f in os.listdir('.') if expr.search(f)]
                 run.DBG('histor files :', os.listdir('.'), l_files)
                 dest = os.path.join(REPREF, repdest)
@@ -1153,9 +1198,9 @@ def Update(run, *args, **kwargs):
                         jret = run.Copy(dest, src)
                         run.VerbEnd(jret, verbose=True)
                         for fsrc in glob.glob(src):
-                            run.Mess(fmt_copy % os.path.join(dest, os.path.basename(fsrc)), 
+                            run.Mess(fmt_copy % os.path.join(dest, os.path.basename(fsrc)),
                             'SILENT')
-                
+
     run.timer.Stop(tit)
 
     # 4. ----- delete source files
@@ -1173,14 +1218,19 @@ def Update(run, *args, **kwargs):
 
     # 5. ----- update executable, libs and catalogues
     _build_aster(run, conf, True, REPREF, None, reptrav_built, lardv=lardv)
+    run.Delete(reptrav)
+    run.Delete(reptrav_built)
 
 
 def AutoUpdate(run, *args):
     """Try to download available updates and apply them.
     """
     run.check_version_setting()
+    if len(args) > 1:
+        run.parser.error(_(u"'--%s' requires at most one argument") % run.current_action)
+    maxv = None
     if len(args) > 0:
-        run.parser.error(_(u"'--%s' requires no argument") % run.current_action)
+        maxv = version2tuple(args[0])
 
     # check for astk update
     AstkUpdate(run)
@@ -1200,8 +1250,7 @@ def AutoUpdate(run, *args):
     os.chdir(reptrav)
 
     # ----- get current version number (ex. 9.1.0)
-    i, v = GetVersion(run, silent=True, \
-            REPREF=run.get_version_path(run['aster_vers']))
+    i, v = GetVersion(run, silent=True, vers=run['aster_vers'])
     if i != 0:
         run.Mess(_(u'error occurs during getting release number'), '<F>_ERROR')
     current = '%s.%s.%s' % (v[0], v[1], v[2])
@@ -1226,6 +1275,9 @@ def AutoUpdate(run, *args):
     todo = []
     next_release = None
     for vers_i, pkg, sha in lvers:
+        # ignore version more recent than maxv
+        if maxv and vers_i[:3] > maxv[:3]:
+            continue
         if vers_i[:3] > current:         # more recent
             if vers_i[:2] != current[:2]: # different release
                 newer.add(vers_i[:2])
@@ -1335,7 +1387,7 @@ def CopyVersion(run, version, dest, **kwargs):
         fconf = os.path.join(REPREF, 'config.txt')
     else:
         fconf = os.path.abspath(run['config'])
-    conf = ASTER_CONFIG(fconf, run)
+    conf = AsterConfig(fconf, run)
 
     title = ufmt(_(u'Copy %s to %s'), os.path.basename(version), os.path.basename(dest))
     run.timer.Start(title)
@@ -1359,13 +1411,13 @@ def CopyVersion(run, version, dest, **kwargs):
     objects = []
     for i in to_copy:
         objects.extend(conf[i])
-    
+
     # ----- add config.txt and "local" environment file(s)
     objects.append('config.txt')
     l_env = [f for f in conf['ENV_SH'] if not os.path.isabs(f)]
     objects.extend(l_env)
     objects = [os.path.join(REPREF, o) for o in objects]
-    
+
     # ----- copy !
     niverr = kwargs.get('niverr', None)
     run.MkDir(REPDEST, niverr=niverr)
@@ -1383,9 +1435,12 @@ def MakeDiag(run, *args):
         run.parser.error(_(u"'--%s' requires at most one argument") % run.current_action)
 
     REPREF = run.get_version_path(run['aster_vers'])
-    fconf = os.path.join(REPREF, 'config.txt')
-    conf   = ASTER_CONFIG(fconf, run)
-    build  = ASTER_BUILD(run, conf)
+    if not run.get('config'):
+        fconf = os.path.join(REPREF, 'config.txt')
+    else:
+        fconf = os.path.abspath(run['config'])
+    conf   = AsterConfig(fconf, run)
+    build  = AsterBuild(run, conf)
 
     run.PrintExitCode = False
     run.print_timer = False
@@ -1419,7 +1474,7 @@ def MakeDiag(run, *args):
         iret, list_tests = get_list(flist)
         if iret != 0:
             run.Mess(ufmt(_(u'error during reading file : %s'), flist), '<F>_ERROR')
-    
+
     # get the dict result
     dict_resu = get_diagnostic(run, build, l_dirs, list_tests)
     vlast = dict_resu['__global__']['version']
@@ -1429,7 +1484,7 @@ def MakeDiag(run, *args):
     nbnook = 0
     noresu = 0
     l_txt = []
-    
+
     for test in list_tests:
         t[0] += dict_resu[test]['tcpu']
         t[1] += dict_resu[test]['tsys']
@@ -1487,7 +1542,7 @@ def get_diagnostic(run, build, test_dirs, list_tests):
     """
     list_tests.sort()
     nbtest = len(list_tests)
-    
+
     # result dict which will be pickled (if we have sufficient permission)
     dict_resu = {
         '__global__' : {
@@ -1543,13 +1598,13 @@ def GenCtags(run, force=False):
     run.Mess(_(u'Generation of ctags database'), 'TITLE')
     run.check_version_setting()
     REPREF = run.get_version_path(run['aster_vers'])
-    
+
     ctags = run.Which('ctags')
     ftags = 'tags'
     if ctags is None:
         run.Mess(_(u"'ctags' is not in your PATH. ctags not generated."))
         return
-    
+
     style = run.get('ctags_style', '')
     if style == 'exuberant':
         # command line for Exuberant Ctags
@@ -1561,14 +1616,14 @@ def GenCtags(run, force=False):
         run.Mess(_(u"'Exuberant Ctags' and 'GNU Emacs ctags' styles are supported. "\
                  "You must define 'ctags_style' in the config file to generate ctags."))
         return
-    
+
     dcmd = { 'ctags' : ctags, 'ftags' : ftags }
     if not os.access(REPREF, os.W_OK):
         run.Mess(ufmt(_(u'no write access to %s'), REPREF), '<F>_ERROR')
-    
-    prev = os.getcwd()
+
+    prev = os.getcwdu()
     os.chdir(REPREF)
-    
+
     force = run['force'] or force or (not os.path.isfile(ftags))
     if force:
         run.Delete(ftags, verbose=True)
@@ -1589,7 +1644,7 @@ def GenCtags(run, force=False):
         run.CheckOK()
     else:
         run.Mess(_(u"ctags file already exists, use --force to overwrite it."))
-    
+
     os.chdir(prev)
 
 
@@ -1604,8 +1659,8 @@ def GetExport(run, *args):
     REPREF = run.get_version_path(run['aster_vers'])
     run.PrintExitCode = False
     ficconf = os.path.join(REPREF, 'config.txt')
-    conf = ASTER_CONFIG(ficconf, run)
-    
+    conf = AsterConfig(ficconf, run)
+
     prof = build_test_export(run, conf, REPREF, reptest=[], test=args[0])
     print3(prof.get_content())
 
@@ -1614,7 +1669,7 @@ def get_vers(fich, debug=False):
     """Extract the version which produced 'resu'/'erre' file.
     """
     vers = ''
-    exp = re.compile('CODE_ASTER.*VERSION.* ([0-9\.]+) ', re.MULTILINE)
+    exp = re.compile('Version.* ([0-9]+\.[0-9]+\.[0-9]+) ', re.M | re.I)
     if os.path.isfile(fich):
         l_v = exp.findall(open(fich, 'r').read())
         if len(l_v) > 0:
