@@ -33,7 +33,7 @@ import MySQLdb
 
 from asrun.common.i18n import _
 from asrun.mystring     import print3
-from asrun.system       import get_home_directory
+from asrun.common.sysutils import get_home_directory
 
 
 class MySQLError(Exception):
