@@ -22,7 +22,7 @@ Give methods to :
     - create a new issue in the tracker database,
     - close an issue,
     - extract messages from the tracker to write a histor file...
-Methods are called by an ASTER_RUN object.
+Methods are called by an AsterRun object.
 """
 
 import os
@@ -31,18 +31,18 @@ import re
 
 from asrun.installation import aster_root, confdir
 from asrun.common.i18n import _
-from asrun.mystring     import print3
-from asrun.mystring     import convert, cut_long_lines
+from asrun.mystring     import print3, ufmt, convert, cut_long_lines
 from asrun.maintenance  import GetVersion
 from asrun.common_func  import get_tmpname
 from asrun.common.sysutils import on_linux
 from asrun.common.utils import now
 
 
-try:   
+try:
     import asrun.myconnect as db
     from asrun.schema    import ReadDB, TYPE, USER, PRODUIT, STATUS, ISSUE, MSG, \
-                               ISSUE_MESSAGES, YES, NO, ReadDBError, WriteDBError
+                               ISSUE_MESSAGES, YES, NO, ReadDBError, WriteDBError, \
+                               mysql_date_fmt, ISSUE_NOSY
     from asrun.histor    import InitHistor
     imports_succeed = True
 except ImportError:
@@ -79,7 +79,7 @@ def parse_issue_file(content):
 def SetParser(run):
     """Configure the command-line parser, add options name to store to the list,
     set actions informations.
-    run : ASTER_RUN object which manages the execution
+    run : AsterRun object which manages the execution
     """
     acts_descr = {
         'create_issue' : {
@@ -106,7 +106,7 @@ def SetParser(run):
                 'action'  : 'store',
                 'default' : 'all',
                 'metavar' : 'STAT',
-                'choices' : ('all', 'resolu', 'valide_EDA', 'ferme'),
+                'choices' : ('all', 'resolu', 'valide_EDA', 'attente_doc', 'ferme'),
                 'dest'    : 'status',
                 'help'    : _(u'raise an error if issues are not in this status')
             }
@@ -152,7 +152,7 @@ def Creer(run, *args):
     on_machref = run.get('rep_agla', 'local') != 'local'
     if not on_machref:
         run.Mess(_(u'Only available on the AGLA machine'), '<F>_AGLA_ERROR')
-    
+
     # 0. check imports
     if not imports_succeed:
         run.Mess(_(u'Imports of REX interface failed.'), '<F>_IMPORT_ERROR')
@@ -179,7 +179,7 @@ def Creer(run, *args):
         c = db.CONNECT('REX', rcdir=confdir, verbose=run['debug'])
     except Exception, msg:
         run.Mess(msg, '<F>_DB_ERROR')
-        
+
     typ = ReadDB(TYPE, c)
     d_typ = {
         'AL'   : typ['anomalie'],
@@ -231,7 +231,7 @@ def Creer(run, *args):
         run.Mess(_(u"Version %s not found in database !") % d['VERSION'])
 
     # 4.3. fill fields
-    date_now = now(timefmt="")
+    date_now = now(datefmt=mysql_date_fmt, timefmt="")
     txtmsg = convert(d['TEXTE'], db_encoding)
     txtmsg = cut_long_lines(txtmsg, maxlen=100)
     issue = ISSUE({
@@ -249,18 +249,25 @@ def Creer(run, *args):
                 '_date'     : date_now,
                 '_summary'  : txtmsg[:255], } ,c)
     lien = ISSUE_MESSAGES({'linkid' : descr, 'nodeid' : issue}, c)
-    
+
     # 4.4. insert issue in database
     try:
         lien.write()
     except WriteDBError, msg:
         run.Mess(_(u'Insert issue failed'), '<F>_DB_ERROR')
-    
+
+    # 4.5. add user in nosy list
+    nosy = ISSUE_NOSY({'linkid' : user['id'], 'nodeid' : issue['id']}, c)
+    try:
+        nosy.write()
+    except WriteDBError, msg:
+        run.Mess(_(u'Add user to nosy list failed'), '<F>_DB_ERROR')
+
     # 5. get message and issue id
     numid = issue['id']
     msgid = descr['id']
     print3('INDEX=%s MESSAGE=%s' % (numid, msgid))
-    
+
     # 6. copy message file
     repid = str(int(msgid) / 1000)
     fmsg = osp.join(run['tmp_user'], 'msg%s' % msgid)
@@ -274,7 +281,7 @@ def Creer(run, *args):
     if iret != 0:
         run.Mess(_(u'Error message: %s') % output)
         run.Mess(_(u'Error occurs during copying message file'), '<F>_COPY')
-    
+
     # 7. study files
     if fichetude == 1:
         cmd = []
@@ -296,27 +303,27 @@ def Histor(run, *args):
     on_machref = run.get('rep_agla', 'local') != 'local'
     if not on_machref:
         run.Mess(_(u'Only available on the AGLA machine'), '<F>_AGLA_ERROR')
-    
+
     # 0. check imports
     if not imports_succeed:
         run.Mess(_(u'Imports of REX interface failed.'), '<F>_IMPORT_ERROR')
-    
+
     # 1. copy input file
     jn = run['num_job']
     ffich = get_tmpname(run, run['tmp_user'], basename='hist_input')
     kret = run.Copy(ffich, args[0], niverr='<F>_COPYFILE')
-    
+
     # 2. read input file
     hist_content = open(ffich, 'r').read()
     expr = re.compile('([0-9]+)', re.MULTILINE)
     l_nf = [int(n) for n in expr.findall(hist_content)]
-    
+
     # 3. open database connection
     try:
         c = db.CONNECT('REX', rcdir=confdir, verbose=run['debug'])
     except Exception, msg:
         run.Mess(msg, '<F>_DB_ERROR')
-    
+
     # 4. read issues from database
     prec_user = None
     histor = InitHistor(format=run['format'], url=run['rex_url'])
@@ -328,31 +335,28 @@ def Histor(run, *args):
         except ReadDBError:
             run.Mess(_(u'Unable to read issue %s') % numf, '<E>_UNKNOWN_ISSUE')
             continue
-        
+
         # 4.2.
         # 4.2.1. check status
         status = issue['_status'].GetPrimValue()
         if status != run['status'] and run['status'] != 'all':
             run.Mess(_(u"Status of issue %s is not '%s' (%s)") \
                     % (numf, run['status'], status), '<E>_UNEXPECTED_VALUE')
-        
-        # 4.2.2. alarm on impactDoc
-        if issue['_impactDoc'] != None:
-            run.Mess(_(u'modified documents : %s') % issue['_impactDoc'], \
-                    '<A>_DOCUMENTATION')
-        
+
+        # 4.2.2. alarm on impactDoc : no
+
         # 4.3.1. user header
         if numf == l_nf[0] or prec_user == None:
             prec_user = issue['_assignedto']
             histor.BeginUser(prec_user)
-        
+
         if issue['_assignedto'] != None and prec_user != None \
                 and issue['_assignedto']['id'] != prec_user['id']:
             if prec_user != None:
                 histor.EndUser(prec_user)
             prec_user = issue['_assignedto']
             histor.BeginUser(prec_user)
-        
+
         # 4.3.2. get message file of answer
         l_rep = []
         d_msg = issue.GetLinks()
@@ -382,18 +386,19 @@ def Histor(run, *args):
                 if len(l_msgid) > 1:
                     l_rep.append(_(u'message %d :') % (len(l_msgid) - i))
                 l_rep.append(txt)
-        
+
         # 4.3.3. add this issue to the histor object
         histor.AddIssue(issue, l_rep)
-        
+
         # 4.3.4. last issue
         if numf == l_nf[-1] and prec_user != None:
             histor.EndUser(prec_user)
-    
+
     # 5. copy histor file
     ffich = get_tmpname(run, run['tmp_user'], basename='hist_output')
     open(ffich, 'w').write(repr(histor))
     kret = run.Copy(args[1], ffich, niverr='<F>_COPYFILE')
+    run.Mess(_(u"Histor successfully generated."), 'OK')
 
 
 def Solder(run, *args):
@@ -403,7 +408,7 @@ def Solder(run, *args):
         run.parser.error(_(u"You must define 'default_vers' in 'aster' configuration file or use '--vers' option."))
     if len(args) != 1:
         run.parser.error(_(u"'--%s' requires one argument") % run.current_action)
-    
+
     on_machref = run.get('rep_agla', 'local') != 'local'
     if not on_machref:
         run.Mess(_(u'Only available on the AGLA machine'), '<F>_AGLA_ERROR')
@@ -413,50 +418,55 @@ def Solder(run, *args):
             raise KeyError
     except KeyError, msg:
         run.Mess(_(u'insufficient privileges to close issues !'), '<F>_AGLA_ERROR')
-    
+
     # 0. check imports
     if not imports_succeed:
         run.Mess(_(u'Imports of REX interface failed.'), '<F>_IMPORT_ERROR')
-    
+
     # 1. read histor file
     jn = run['num_job']
     ffich = get_tmpname(run, run['tmp_user'], basename='hist_input')
     kret = run.Copy(ffich, args[0], niverr='<F>_COPYFILE')
-    
+
     hist_content = open(ffich, 'r').read()
     expr = re.compile('RESTITUTION FICHE +([0-9]+)', re.MULTILINE)
     l_nf = [int(n) for n in expr.findall(hist_content)]
-    
+    if len(l_nf) == 0:
+        run.Mess(_(u'Incorrect file, no issue to close.'), '<F>_AGLA_ERROR')
+
     # 2. get version number and database fields to fill
-    REPREF = osp.join(aster_root, run['aster_vers'])
-    run.Mess(_(u'Close issues in :'))
-    iret, l_res = GetVersion(run, silent=False, REPREF=REPREF)
-    tagv = '.'.join(l_res[:3])
-    if l_res[4]:
-        typv = 'expl'
-        autr = 'dev'
+    typv = 'dev'
+    autr = 'expl'
+    # also accept a version number (ex. '2011.1') instead of 'NEW10'
+    if run['aster_vers'].replace('.', '').isdigit():
+        tagv = run['aster_vers']
+        run.Mess(ufmt(_(u'Close issues with version tag : %s'), tagv))
     else:
-        typv = 'dev'
-        autr = 'expl'
+        run.Mess(_(u'Close issues in :'))
+        iret, l_res = GetVersion(run, silent=False, vers=run['aster_vers'])
+        tagv = '.'.join(l_res[:3])
+        if l_res[4]:
+            typv = 'expl'
+            autr = 'dev'
     d_champ = {
         'a_corrige' : '_corrV' + typv,
         'v_correct' : '_verCorrV' + typv,
         'a_tester'  : '_corrV' + autr,
         'v_tester'  : '_verCorrV' + autr,
     }
-    
+
     # 3. open database connection
     try:
         c = db.CONNECT('REX', rcdir=confdir, verbose=run['debug'])
     except Exception, msg:
         run.Mess(msg, '<F>_DB_ERROR')
-    
+
     try:
         etat = ReadDB(STATUS, c)
         prod = ReadDB(PRODUIT, c)
     except ReadDBError:
         run.Mess(_(u'Unable to read status and product'), '<F>_DB_ERROR')
-    
+
     # 4. read issues from database
     req_status = 'valide_EDA'
     for numf in l_nf:
@@ -466,14 +476,14 @@ def Solder(run, *args):
         except ReadDBError:
             run.Mess(_(u'Unable to read issue %s') % numf, '<E>_UNKNOWN_ISSUE')
             continue
-        
+
         # 4.2. check issue values
         status = issue['_status'].GetPrimValue()
         if status != req_status:
             run.Mess(_(u"Status of issue %s is not '%s' (%s)") \
                     % (numf, req_status, status), '<A>_UNEXPECTED_VALUE')
             issue['_status'] = etat[req_status]
-        
+
         if issue['_produit'].GetPrimValue() == 'Code_Aster':
             if not issue[d_champ['a_corrige']] \
                 and issue['_type'].GetPrimValue() != 'aide utilisation':
@@ -483,17 +493,20 @@ def Solder(run, *args):
         else:
             issue[d_champ['a_corrige']] = NO
             issue[d_champ['a_tester']] = NO
-                        
+
         # 4.3. fill issue fields
         issue[d_champ['v_correct']] = tagv
-        
+
         # 4.4. close issue ?
         if not issue[d_champ['a_tester']] or not issue[d_champ['v_tester']] in ('', None):
-            issue['_status'] = etat['ferme']
+            new_status = etat['attente_doc']
+            if not issue['_impactDoc']:
+                new_status = etat['ferme']
+            issue['_status'] = new_status
             run.Mess(_(u'issue %s is closed') % numf)
         else:
             run.Mess(_(u'issue %s must be solved in V%s too') % (numf, autr))
-        
+
         # 4.5. write issue in database
         try:
             if not run['debug']:
@@ -505,11 +518,11 @@ def Solder(run, *args):
 
     # 5. delete files of *all* closed issues
     # list of closed issues
-    query = """SELECT id from _issue WHERE _status=%s;""" % etat['ferme']
+    query = """SELECT id from _issue WHERE _status=%s;""" % etat['ferme']['id']
     try:
         res = c.exe(query)
     except db.MySQLError, msg:
-        run.Mess(_("error executing a query to the database"), '<F>_DB_ERROR')
+        run.Mess(ufmt(_("error executing a query to the database\n %s"), query), '<F>_DB_ERROR')
     l_ferm = set([i[0] for i in res])
     # list of existing directories
     l_dirs = set([int(d) for d in os.listdir(osp.join(run['rep_rex'], "emise")) \
@@ -519,7 +532,7 @@ def Solder(run, *args):
     for d in dir_to_del:
         run.Delete(osp.join(run['rep_rex'], "emise", d))
         run.Mess(_(u"files of issue %s have been deleted.") % d)
-    
+
 
 
 
