@@ -18,10 +18,9 @@
 # ==============================================================================
 
 """
-Definition of ASTER_RUN class.
+Definition of AsterRun class.
 """
 
-
 import sys
 import os
 import os.path as osp
@@ -32,53 +31,52 @@ import traceback
 import pprint
 from ConfigParser import SafeConfigParser, NoOptionError
 from optparse import OptionGroup
+from warnings import warn
 
+from asrun.core          import magic, RunAsterError
 from asrun.common.i18n   import _
-from asrun.mystring      import print3, to_unicode, ufmt
+from asrun.mystring      import print3, convert_list, to_unicode, ufmt
 from asrun.parser        import (define_parser, default_options, get_option_value,
                                 SUPPRESS_HELP)
-from asrun.timer         import ASTER_TIMER
-from asrun.mystring      import convert
+from asrun.timer         import AsterTimer
 from asrun.common.rcfile import read_rcfile
 from asrun.installation  import aster_root, confdir, localedir
 from asrun.thread        import is_main_thread, TaskAbort
-from asrun.system        import get_home_directory, local_host, local_full_host, get_exec_name
-from asrun.common.utils  import get_subdirs
+from asrun.common.sysutils import get_home_directory, local_host, local_full_host, get_exec_name
+from asrun.common.utils  import get_subdirs, get_unique_id
+from asrun.common_func   import get_limits
+
+from asrun.backward_compatibility import bwc_config_rc, bwc_deprecate_class
 
 
-
-class RunAsterError(Exception):
-    """Base exception for ASTER_RUN
-    """
-
-class ASTER_RUN:
+class AsterRun:
     """Main class to manage execution.
     """
 
     def __init__(self, **kwargs):
-        """Initializations
-        """
+        """Initializations"""
         self.ExitOnFatalError = True
         self.PrintExitCode    = True
         self.__initialized    = False
-        self.__dbg_init       = time.time()
         self._val_pid         = 0
         self.rcdir = kwargs.get('rcdir')
         # ----- verification de la version de Python
-        if sys.hexversion < 0x020400F0:
-            print3(_(u"This script requires Python 2.4 or higher, sorry !"))
+        if sys.hexversion < 0x020600F0:
+            print3(_(u"This script requires Python 2.6 or higher, sorry !"))
             self.Sortie(4)
 
         # ----- package informations
         from asrun.__pkginfo__ import version, copyright
         self.program     = 'as_run'
         self.__version__ = version
-        
+
         # ----- options, liste des options du parser à stocker dans options
         self.options = default_options
         # always use _add_option to extend options_list
-        self.options_list = ['debug', 'verbose', 'silent', 'force', 'num_job', 'debug_stderr']
-        
+        self.options_list = ['debug', 'verbose', 'silent', 'force', 'num_job',
+            'debug_stderr', 'stdout', 'stderr', 'log_progress',
+            'remote_shell_protocol', 'remote_copy_protocol']
+
         # ----- informations about actions : module, method
         # always use set_action to define a new action
         self.actions_info = {}
@@ -89,19 +87,12 @@ class ASTER_RUN:
         self.user_vars = {
             'editor' :
                 {  'val'  : 'nedit',
-                    'help' : _(u'editor command') },
+                   'help' : _(u'editor command') },
             'devel_server_user' :
                 {  'val'  : '',
                     'help' : _(u'login on the development server') + os.linesep + \
                     _(u'# (name/ip address is usually set in /etc/codeaster/asrun)') },
-            'remote_copy_protocol' :
-                {  'val'  : 'SCP',
-                    'help' : _(u'remote protocol used to copy files and directories') },
-            'remote_shell_protocol' :
-                {  'val'  : 'SSH',
-                    'help' : _(u'remote protocol used for shell commands') },
         }
-        self._add_option(*self.user_vars.keys())
 
         # ----- formats
         self.fmt = {
@@ -135,28 +126,28 @@ class ASTER_RUN:
         # ----- parser
         self.parser = define_parser(self)
 
-        # ----- add user's preferences as options
-        for k, dV in self.user_vars.items():
-            self.parser.add_option('--'+k,
-                action='store', dest=k, default=None,
-                help=dV['help'].replace(os.linesep, ' ').replace('#', ''))
-
         # ----- user resource directory
         # if self.rcdir is set, it's supposed to start with ".astkrc_"
         # or to be an absolute directory name.
+        # if it's an absolute pathname it may be unusable on the remote host
         if self.rcdir is None:
             self.rcdir = get_option_value(sys.argv, "--rcdir", default=".astkrc")
         assert self.rcdir.startswith(".astkrc") or osp.abspath(self.rcdir) == self.rcdir, \
             u"absolute path or similar to '.astkrc_xxx' expected not: %s" % self.rcdir
 
         # ----- read master configuration file and user preferences
-        self.user_rc = osp.join(get_home_directory(), self.rcdir, 'config')
         self.config = {}
+        oldrc = osp.join(get_home_directory(), self.rcdir, 'config')
+        bwc_config_rc(oldrc, self._read_rc)
+
+        self.user_rc = osp.join(get_home_directory(), self.rcdir, 'prefs')
         self._read_rc(osp.join(confdir, 'asrun'), self.config, mcsimp=['noeud',])
         self._read_rc(osp.join(confdir, 'aster'), self.config, optionnal=True, mcsimp=['vers',])
         self._read_rc(osp.join(confdir, 'agla'),  self.config, optionnal=True)
         self._init_rc()
 
+        # add user plugins directory
+        sys.path.append(osp.join(get_home_directory(), self.rcdir))
         # ----- prepare tmp_user and cache_dir
         # no folder in cache_dir
         tmp_user = osp.join(self.config['rep_tmp'],
@@ -191,20 +182,18 @@ class ASTER_RUN:
             self.Sortie(4)
 
         # ----- init timer
-        self.timer = ASTER_TIMER()
+        self.timer = AsterTimer()
         self.LoadExtensions()
         self.CheckExtensions()
 
 
     def ParseArgs(self):
-        """Call the arguments parser, returns options and arguments.
-        """
+        """Call the arguments parser, returns options and arguments."""
         opts, args = self.parser.parse_args()
         # ----- all should have been initialized !
         self.__initialized = True
-        # exceptions
         os.environ['ASTER_ROOT'] = aster_root
-        
+
         # ----- stocke les options
         for option in self.options_list:
             self.options[option] = getattr(opts, option)
@@ -212,10 +201,9 @@ class ASTER_RUN:
         if opts.display != None:
             os.environ['DISPLAY'] = opts.display
             self.DBG(u'set DISPLAY to', opts.display)
-        
+
         # ----- migration
         if self.options['version_dev'] is not None:
-            from warnings import warn
             warn('--version_dev=... is deprecated. Use --vers=... instead.', DeprecationWarning, stacklevel=2)
             if self.options['aster_vers'] != self.options['version_dev']:
                 warn('--vers=%s will be overwritten by --version_dev into --vers=%s' \
@@ -227,36 +215,28 @@ class ASTER_RUN:
 
 
     def _init_rc(self):
-        """Read user preferences or create the file the first time
-        """
+        """Read user preferences or create the file the first time"""
         ficrc = self.user_rc
+        dname = osp.dirname(ficrc)
         # ----- check for ressources directory
         # safer as osp.exists for multiple executions
         try:
-            os.makedirs(osp.dirname(ficrc))
+            os.makedirs(dname)
         except OSError:
             pass
-        
+
         if not osp.isfile(ficrc):
             # ----- create ressources file
-            content = []
-            content.append(_(u"# as_run : user preferences file"))
-            content.append("")
-            content.append(_(u"# You can override here all values of $ASTER_ROOT/etc/codeaster/asrun"))
-            content.append("")
-            for var in self.user_vars.keys():
-                content.append("# %s" % self.user_vars[var]['help'])
-                content.append("%s : %s" % (var, self.user_vars[var]['val']))
-                content.append("")
-            txt = os.linesep.join(content)
-            open(ficrc, 'w').write(convert(txt))
-        # ----- read ressource file
+            from asrun.client import ClientConfig
+            client = ClientConfig(self.rcdir)
+            client.init_user_resource(osp.basename(ficrc))
+        # ----- read user ressource file
         self._read_rc(ficrc, self.config, mcsimp=['vers', 'noeud'])
+        self._check_config()
 
 
     def _read_rc(self, ficrc, destdict, optionnal=False, mcsimp=None):
-        """Read a ressource file and store variables to 'destdict'.
-        """
+        """Read a ressource file and store variables to 'destdict'."""
         if osp.isfile(ficrc):
             read_rcfile(ficrc, destdict, mcsimp=mcsimp)
         elif not optionnal:
@@ -264,10 +244,22 @@ class ASTER_RUN:
             self.Sortie(4)
 
 
+    def _check_config(self):
+        """Check configuration for deprecated fields or arguments."""
+        for key in ("editor", "editeur", "terminal"):
+            value = self.config.get(key, None)
+            # always used current display (eventually set by "ssh -X")
+            if value and value.find('@D') > -1:
+                warn("'%s' : the argument '@D' is deprecated. "
+                     "Remove it from your configuration file." % key,
+                     DeprecationWarning, stacklevel=2)
+                # remove display
+                self.config[key] = re.sub('\-+display[ =]+@D', '', value)
+
+
     def _add_action(self, action, info):
         """Set informations about an action
-        Prefer use this method over extend manually the dictionnary.
-        """
+        Prefer use this method over extend manually the dictionnary."""
         if not type(action) in (str, unicode):
             self.Mess(_(u'invalid type (expect %s not %s)') % \
                     ('string', type(action)), '<F>_PROGRAM_ERROR')
@@ -283,8 +275,7 @@ class ASTER_RUN:
 
     def _add_option(self, *args):
         """Add one or more option which will be available through getitem.
-        Prefer use this method over extend manually the list.
-        """
+        Prefer use this method over extending manually the list."""
         if not type(args) in (list, tuple):
             self.Mess(_(u'invalid type (expect %s not %s)') % \
                     ('string', type(args)), '<F>_PROGRAM_ERROR')
@@ -297,8 +288,7 @@ class ASTER_RUN:
 
 
     def LoadExtensions(self):
-        """Initialisations des extensions.
-        """
+        """Initialisations des extensions."""
         import asrun.maintenance
         import asrun.execute
         import asrun.services
@@ -314,8 +304,7 @@ class ASTER_RUN:
 
 
     def CheckExtensions(self):
-        """Initialisations des éventuelles extensions.
-        """
+        """Initialisations des éventuelles extensions."""
         user_extensions    = osp.join(get_home_directory(), self.rcdir, 'as_run.extensions')
         cwd_extensions     = 'as_run.extensions'
         config = SafeConfigParser()
@@ -352,8 +341,7 @@ class ASTER_RUN:
                 key   = option name,
                 value = {args, kwargs} passed to add_option parser method
             stored_options : list of options which should be available using
-                getitem (default to all "dest" from options_descr)
-        """
+                getitem (default to all "dest" from options_descr)"""
         # ----- default values
         # actions_order
         if dico.has_key('actions_order'):
@@ -424,8 +412,7 @@ class ASTER_RUN:
 
 
     def PrintConfig(self):
-        """Imprime la liste et la valeur des paramètres
-        """
+        """Imprime la liste et la valeur des paramètres"""
         self.parser.print_version()
         print3(os.linesep+_(u'Parameters (config attribute)')+' :')
         for key, val in self.config.items():
@@ -436,15 +423,13 @@ class ASTER_RUN:
 
 
     def ToDelete(self, path):
-        """Add 'path' to to_delete list.
-        """
+        """Add 'path' to to_delete list."""
         if not path in self.to_delete:
             self.to_delete.append(path)
 
 
     def DoNotDelete(self, path):
-        """Remove 'path' from to_delete list.
-        """
+        """Remove 'path' from to_delete list."""
         if path in self.to_delete:
             self.to_delete.remove(path)
 
@@ -452,8 +437,7 @@ class ASTER_RUN:
     def _clear_tmp(self):
         """Empty cache directory if necessary (remove files for which last access
         is older than 'deltat' seconds), and delete all directories
-        from 'self.to_delete'.
-        """
+        from 'self.to_delete'."""
         # to avoid to delete current directory (it may freeze some systems...)
         try:
             os.chdir(get_home_directory())
@@ -491,13 +475,13 @@ class ASTER_RUN:
     def __getitem__(self, key):
         """Méthode pour accéder facilement aux paramètres de configuration ou à la
         valeur d'une option.
-        Les options surchargent les paramètres de configuration.
-        """
+        Les options surchargent les paramètres de configuration."""
+        UNDEF = "__undefined__"
         if hasattr(self, '__'+key+'__'):
             return getattr(self, '__'+key+'__')
-        elif self.options.get(key) != None:
+        elif self.options.get(key, UNDEF) != UNDEF:
             return self.options[key]
-        elif self.config.get(key) != None:
+        elif self.config.get(key, UNDEF) != UNDEF:
             return self.config[key]
         elif self.__initialized:
             self.Mess(_(u"""'%s' does not exist in config or options.
@@ -507,8 +491,7 @@ class ASTER_RUN:
 
     def get(self, key, default=None):
         """Semblable à getitem avec la possibilité de retourner une valeur par défaut
-        si la valeur n'est pas trouvée.
-        """
+        si la valeur n'est pas trouvée."""
         if hasattr(self, '__'+key+'__'):
             return getattr(self, '__'+key+'__')
         elif self.options.get(key) != None:
@@ -521,18 +504,12 @@ class ASTER_RUN:
 
     def get_pid(self, num=None):
         """Return a jobid based on self['num_job'].
-        Counter is incremented at each call.
-        """
-        if num is None:
-            self._val_pid += 1
-            num = self._val_pid
-        assert type(num) in (int, long), _(u'get_pid: integer argument required!')
-        return '%04d-%s' % (num, self['num_job'])
+        Counter is incremented at each call."""
+        return get_unique_id(self['num_job'], num)
 
 
     def get_version_path(self, label, root=aster_root):
-        """Return full path to the version named 'label'.
-        """
+        """Return full path to the version named 'label'."""
         if not label:
             return None
         dv = {}
@@ -551,12 +528,11 @@ class ASTER_RUN:
         """Ajuste la configuration dynamiquement (car dépendance au contexte et
         pas seulement au fichier 'config').
         Paramètres déclenchants :
-            serv_as_node, only_serv_as_node
-        """
+            serv_as_node, only_serv_as_node, limits"""
         if not self.__initialized or not hasattr(self, 'GetHostName'):
-            print3(_(u'Warning : ASTER_RUN object not ready for dynamic adjustement !'))
+            print3(_(u'Warning : AsterRun object not ready for dynamic adjustement !'))
             return
-        
+
         # add localhost as a compute node
         if self.config.get('serv_as_node', False):
             l_node = self.config.get('noeud', '').split()
@@ -566,14 +542,24 @@ class ASTER_RUN:
                 l_node = [local_host,]
             self.config['noeud'] = ' '.join(l_node)
 
+        # hard limits
+        self.SetAutoLimit()
         # trace installation parameter
         self.DBG(u'installation dirs (aster_root, confdir, localedir, LANG)',
                     aster_root, confdir, localedir, os.environ.get('LANG', 'undefined'))
 
 
+    def SetAutoLimit(self):
+        """Set memory & cpu limits.
+        Must be called after the initialization of the system object."""
+        dlim_i = get_limits(self, 'interactif')
+        dlim_b = get_limits(self, 'batch')
+        self.config.update(dlim_i)
+        self.config.update(dlim_b)
+
+
     def GetGrav(self, cod):
-        """Return the severity behind 'cod' as a number to allow comparison
-        """
+        """Return the severity behind 'cod' as a number to allow comparison"""
         dgrav = { '?' : -9, '_' : -9, 'OK' : 0, 'A' : 1,
                 'NOOK' : 2, 'NO_TEST_RESU' : -1,
                 'S' : 4,
@@ -604,8 +590,7 @@ class ASTER_RUN:
             TITLE : add a separator.
         If cod='<F>' (without a description), if exists last <E> error is
         transformed into <F>, else <F>_ABNORMAL_ABORT.
-        If store is True, 'msg' is stored in print_on_exit dictionnary.
-        """
+        If store is True, 'msg' is stored in print_on_exit dictionnary."""
         # ----- gravite
         g0 = self.GetGrav(self.diag)
         g1 = self.GetGrav(cod)
@@ -635,9 +620,9 @@ class ASTER_RUN:
             if g1 == -9:
                 coderr = '<I> '+coderr
         print3(fmt % (coderr, msg))
-        sys.stdout.flush()
+        magic.get_stdout().flush()
         # ----- store in print_on_exit
-        if store:
+        if store or (not self.ExitOnFatalError and g1 >= self.GetGrav('<S>')):
             k = '?'
             msg2 = msg
             mat = re.search('<(.)>', cod)
@@ -646,10 +631,7 @@ class ASTER_RUN:
                 msg2 = self.fmt['msg+cod'] % (coderr, msg)
             elif cod in ('OK', 'NOOK'):
                 k = cod
-            if not self.print_on_exit.has_key(k):
-                self.print_on_exit[k] = [msg2]
-            else:
-                self.print_on_exit[k].append(msg2)
+            self.print_on_exit[k] = self.print_on_exit.get(k, []) + [msg2, ]
 
         # ----- diagnostic le plus défavorable
         if g1 > g0:
@@ -681,13 +663,12 @@ class ASTER_RUN:
 
 
     def Sortie(self, exit_code):
-        """Exit by printing diagnostic if defined or exit code if not null
-        """
+        """Exit by printing diagnostic if defined or exit code if not null"""
         # print important messages
         msg = self.get_important_messages()
         if len(msg) > 0:
             print3(msg)
-        
+
         # raise an exception to be catched by the Worker object
         if not is_main_thread():
             raise TaskAbort(msg)
@@ -710,19 +691,18 @@ class ASTER_RUN:
         if self.PrintExitCode or exit_code != 0:
             print3(self.fmt['exit'] % exit_code)
         self._clear_tmp()
+        self.DBG("exit %s" % exit_code)
         sys.exit(exit_code)
 
 
     def CheckOK(self, tole='<S>'):
-        """Exit one error more severe than 'tole' occurred.
-        """
+        """Exit one error more severe than 'tole' occurred."""
         if self.GetGrav(self.diag) >= self.GetGrav(tole):
             self.Sortie(4)
 
 
     def ReinitDiag(self, diag='?'):
-        """Reinitialize diagnostic (for example if a global diagnostic is stored)
-        """
+        """Reinitialize diagnostic (for example if a global diagnostic is stored)"""
         self.diag = diag
 
 
@@ -732,19 +712,22 @@ class ASTER_RUN:
             self.parser.error(_(u"You must define 'default_vers' in 'aster' configuration " \
                                 "file or use '--vers' option."))
 
+    def _get_msecs(self):
+        ct =  time.time()
+        return (ct - long(ct)) * 1000
+
 
     def _printDBG(self, *args, **kargs):
-        """Print debug information to stderr.
-        """
+        """Print debug information to stderr."""
         print_traceback = kargs.get('print_traceback', True)
-        file            = kargs.get('file', sys.stderr)
+        file            = kargs.get('file', magic.get_stderr())
         stack_id        = -3 - kargs.get('stack_id', 0)      # -1 : _printDBG, -2 : DBG ou ASSERT
         all             = kargs.get('all', False)
+        prefix = kargs.get('prefix', '')
         try:
             form = """
->>> %(time)012.3f  %(orig)s
-%(content)s
-%(traceback)s"""
+>>> %(time)12s  %(orig)s
+%(content)s%(traceback)s"""
             formTB = """
 Traceback:
 %s
@@ -757,13 +740,15 @@ Traceback:
                         ls.append(a)
                     else:
                         ls.append(pprint.pformat(a))
-                content = os.linesep.join(ls)
+                content = os.linesep.join(convert_list(ls))
                 if not all and len(content) > 800:
                     content = content[:800] + os.linesep + '[...]'
             except None:
                 content = str(args)
+            if prefix:
+                content = os.linesep.join([prefix + lin for lin in content.splitlines()])
             dinfo = {
-                'time'      : time.time() - self.__dbg_init,
+                'time'      : time.strftime('%H:%M:%S') + '.%03d' % self._get_msecs(),
                 'orig'      : stack[stack_id],
                 'content'   : content,
                 'traceback' : '',
@@ -782,74 +767,76 @@ Traceback:
 
 
     def DBG(self, *args, **kargs):
-        """Print debug information to stderr.
-        """
+        """Print debug information to stderr."""
         print_traceback = kargs.get('print_traceback', False)
-        file            = kargs.get('file', sys.stderr)
+        file            = kargs.get('file', magic.get_stderr())
         stack_id        = kargs.get('stack_id', 0)
         all             = kargs.get('all', False)
+        prefix = kargs.get('prefix', '')
         if ((not hasattr(file, "isatty") or not file.isatty()) or self['debug'] or self['verbose']) \
                 and (self['debug_stderr'] or file != sys.stderr):
-            self._printDBG(print_traceback=print_traceback, file=file, stack_id=stack_id, all=all, *args)
-
+            self._printDBG(print_traceback=print_traceback, file=file,
+                           stack_id=stack_id, all=all, prefix=prefix, *args)
 
     def ASSERT(self, condition):
-        """Print errors to stdout and stderr.
-        """
+        """Print errors to stdout and stderr."""
         if not condition:
-            for f in (sys.stdout, sys.stderr):
+            for f in (magic.get_stdout(), magic.get_stderr()):
                 self._printDBG(u'Assertion failed', print_traceback=True, file=f, stack_id=0)
 
-
     def get_as_run_cmd(self, with_args=True):
-        """Return as_run command line (type list).
-        """
+        """Return as_run command line (type list)."""
         cmd = [osp.join(aster_root, "bin", get_exec_name("as_run")),]
         if with_args:
             cmd.extend(self.get_as_run_args())
         return cmd
 
-
     def get_as_run_args(self):
-        """Return arguments for as_run command line (type list).
-        """
+        """Return arguments for as_run command line (type list)."""
         args = []
-        if self.rcdir != ".astkrc":
-            args.append("--rcdir=%s" % get_option_value(["--rcdir=%s" % self.rcdir], "--rcdir"))
+        args.extend(self.get_rcdir_arg())
         args.extend(self.get_remote_args())
         return args
 
+    def get_rcdir_arg(self):
+        """Return rcdir argument for as_run command line (type list)."""
+        args = []
+        if self.rcdir != ".astkrc":
+            args.append("--rcdir=%s" % get_option_value(["--rcdir=%s" % self.rcdir], "--rcdir"))
+        return args
 
     def get_remote_args(self):
-        """Return remote arguments for as_run command line.
-        """
+        """Return remote arguments for as_run command line."""
+        #XXX should be deprecated in future
         return ["--remote_shell_protocol=%s" % self['remote_shell_protocol'],
                 "--remote_copy_protocol=%s" % self['remote_copy_protocol'],]
 
 
 def AsRunFactory(*args, **kargs):
-    """Initialization of an ASTER_RUN object
-    """
-    #XXX pre 1.8 backward compatibility : astk_serv_root argument removed
+    """Initialization of an AsterRun object"""
+    # pre 1.8 backward compatibility : astk_serv_root argument removed
     if len(args) != 0:
-        from warnings import warn
         warn('AsRunFactory : astk_serv_root argument is deprecated', DeprecationWarning, stacklevel=2)
 
-    from asrun.system import ASTER_SYSTEM
-        
-    run = ASTER_RUN(**kargs)
+    from asrun.system import AsterSystem
+
+    run = AsterRun(**kargs)
     run.options['debug']   = False
     run.options['debug_stderr'] = True
     run.options['verbose'] = False
     run.options['force']   = False
     run.options['num_job'] = str(os.getpid())
+    run.options['log_progress'] = None
     for opt, value in kargs.items():
         if run.options.has_key(opt):
             run.options[opt] = value
         elif run.config.has_key(opt):
             run.config[opt] = value
-    run.system = ASTER_SYSTEM(run, **kargs)
+    run.system = AsterSystem(run, **kargs)
     run.print_timer = False
-
+    run.SetAutoLimit()
+    magic.run = run
+    magic.init_logger(filename=run['log_progress'], debug=run['debug'])
     return run
 
+ASTER_RUN = bwc_deprecate_class('ASTER_RUN', AsterRun)
