@@ -18,7 +18,7 @@
 # ==============================================================================
 
 """
-Definition of ASTER_SYSTEM class.
+Definition of AsterSystem class.
 """
 
 
@@ -27,14 +27,11 @@ import os.path as osp
 import sys
 import re
 import glob
-import platform
-import getpass
 
 from asrun.common.i18n import _
 from asrun.mystring import print3, convert, to_unicode, ufmt
-from asrun.common_func import get_tmpname_base
 from asrun.system_command import COMMAND as command
-from asrun.common.utils import get_encoding, now
+from asrun.common.utils import get_tmpname_base, get_encoding, now
 from asrun.common.sysutils import (
     FileName,
     on_linux,
@@ -42,14 +39,16 @@ from asrun.common.sysutils import (
     local_user,
     local_full_host,
     local_host,
-    is_localhost,
     get_hostname,
 )
+from asrun.common_func import is_localhost2
+from asrun.core import magic
 
+from asrun.backward_compatibility import bwc_deprecate_class
 
 # colors and tty...
 label = { 'OK' : '  OK  ', 'FAILED' : 'FAILED', 'SKIP' : ' SKIP ' }
-if hasattr(sys.stdout, "isatty") and sys.stdout.isatty():
+if on_linux() and hasattr(magic.get_stdout(), "isatty") and magic.get_stdout().isatty():
     label['OK']     = '\033[1;32m%s\033[m\017' % label['OK']
     label['FAILED'] = '\033[1;31m%s\033[m\017' % label['FAILED']
 #   label['SKIP']   = '\033[1;34m%s\033[m\017' % label['SKIP']
@@ -95,14 +94,6 @@ def env2dict(s):
     return d
 
 
-def get_exec_name(script):
-    """Change name of 'script' if it's necessary in the platform.
-    """
-    if on_windows() and not script.endswith(".bat"):
-        script += ".bat"
-    return script
-
-
 def get_command_line(cmd_in, bg, follow_output, separated_stderr,
                      output_filename, error_filename, var_exitcode):
     """Returns the command to run to redirect output/error, retreive exit code
@@ -127,10 +118,13 @@ def get_command_line(cmd_in, bg, follow_output, separated_stderr,
         else:
             new_cmd = command['not_follow_separ_stderr'] % values
     #print3(u"Command :", new_cmd)
+    # may happen if tmpdir has been deleted before another one, just before exit.
+    if not osp.isdir(osp.dirname(output_filename)):
+        new_cmd = command['foreground'] % values
     return new_cmd
 
 
-def _get_system_tmpdir():
+def get_system_tmpdir():
     """Returns a directory for temporary files
     """
     var = os.environ.get("TMPDIR") or os.environ.get('TEMP') or os.environ.get('TMP')
@@ -156,18 +150,14 @@ def split_path(path):
     return splitted
 
 
-def get_home_directory(user=""):
-    """Returns user home directory"""
-    return osp.expanduser("~%s" % user)
 
-
-class ASTER_SYSTEM:
+class AsterSystem:
     """Class to encapsultate "system" commands on local or remote machine.
     Available methods :
         'Copy', 'Shell', 'IsRemote', 'MkDir', 'Delete', 'Exists', 'IsDir',
         'Gzip', 'Gunzip', 'AddToEnv', 'GetHostName', 'PathOnly', 'Rename',
         'VerbStart, 'VerbEnd', 'VerbIgnore', 'SetLimit', 'Which',
-        'GetCpuInfo', 'FindPattern'
+        'GetCpuInfo', 'FindPattern', 'Symlink'
     Supported protocols :
     - for copy :
         LOCAL, RCP, SCP, RSYNC, HTTP
@@ -183,26 +173,24 @@ class ASTER_SYSTEM:
     RequiredParam = ['remote_copy_protocol', 'remote_shell_protocol']
 
     def __init__(self, run, **kargs):
-        """run : ASTER_RUN object
+        """run : AsterRun object
         OR just a dictionnary to define 'RequiredParam' + 'verbose', 'debug'
         """
         self.verbose = run['verbose']
         self.debug = run['debug']
-        self._tmpdir = _get_system_tmpdir()
+        self._tmpdir = get_system_tmpdir()
         # ----- fill param
         self.param = {}
         for key in self.RequiredParam:
             self.param[key] = run[key]
-        # ----- reference to ASTER_RUN object which manages the execution
+        # ----- reference to AsterRun object which manages the execution
         self.run = run
         # ----- check if protocols are supported
         if not self.param['remote_shell_protocol'] in ('LOCAL', 'RSH', 'SSH'):
-            self._mess(_(u'unknown shell protocol : %s') % self.param['remote_shell_protocol'],
-                    '<A>_ALARM')
+            self.param['remote_shell_protocol'] = 'SSH'
         if not self.param['remote_copy_protocol'] in ('LOCAL', 'RCP', 'SCP', 'RSYNC', 'HTTP'):
-            self._mess(_(u'unknown copy protocol : %s') % self.param['remote_copy_protocol'],
-                    '<A>_ALARM')
-        self._dbg('ASTER_SYSTEM param :', self.param)
+            self.param['remote_copy_protocol'] = 'SCP'
+        self._dbg('AsterSystem param :', self.param)
         # ----- cache cpuinfo, meminfo
         self._cpuinfo_cache = {}
         self._meminfo_cache = {}
@@ -213,7 +201,7 @@ class ASTER_SYSTEM:
                         for rep in self.run['shared_folders'].split(',')]
         # ----- add shortcuts - example: run.Copy instead of run.system.Copy
         if type(run) != dict:
-            for meth in ASTER_SYSTEM.__dict__.keys():
+            for meth in AsterSystem.__dict__.keys():
                 if meth[0] == meth[0].upper() and not hasattr(run, meth) \
                         and hasattr(self, meth) and callable(getattr(self, meth)):
                     setattr(run, meth, getattr(self, meth))
@@ -253,7 +241,7 @@ class ASTER_SYSTEM:
             if len(cmd) > Lm-2 or cmd.count(os.linesep) > 0:
                 pcmd = pcmd+os.linesep+' '*Lm
             print3(('%-'+str(Lm)+'s') % (pcmd, ), end=' ')
-            sys.stdout.flush()
+            magic.get_stdout().flush()
 
 
     def VerbEnd(self, iret, output='', verbose=None):
@@ -281,7 +269,8 @@ class ASTER_SYSTEM:
 
 
     def local_shell(self, cmd, bg=False, verbose=False, follow_output=False,
-                   alt_comment=None, interact=False, separated_stderr=False, **ignore_args):
+                   alt_comment=None, interact=False, separated_stderr=False,
+                   stack_id=3, **ignore_args):
         """Execute a command shell
             cmd           : command
             bg            : put command in background if True
@@ -298,7 +287,8 @@ class ASTER_SYSTEM:
         if len(cmd) > self.MaxCmdLen:
             self._mess((_(u'length of command shell greater '\
                     'than %d characters.') % self.MaxCmdLen), '<A>_ALARM')
-        self._dbg('cmd :', cmd, 'background : %s' % bg, 'follow_output : %s' % follow_output, all=True, stack_id=3)
+        self._dbg('cmd :', cmd, 'background : %s' % bg, 'follow_output : %s' % follow_output,
+                  all=True, stack_id=stack_id)
         self.VerbStart(alt_comment, verbose=verbose)
         if follow_output and verbose:
             print3(os.linesep+_(u'Command output :'))
@@ -323,7 +313,7 @@ class ASTER_SYSTEM:
             os.remove(ferr_name)
         except:
             pass
-        
+
         if follow_output:
             # repeat header message
             self.VerbStart(alt_comment, verbose=verbose)
@@ -337,7 +327,9 @@ class ASTER_SYSTEM:
                 iret = int(mat.group(1))
         self.VerbEnd(iret, output, verbose=verbose)
         if iret != 0:
-            self._dbg('ERROR : iret = %s' % iret, output, error, all=True)
+            self._dbg('ERROR : iret = %s' % iret, '+++ STANDARD OUTPUT:', output,
+                      '+++ STANDARD ERROR:', error, '+++ END', all=True,
+                      prefix="    ")
         if bg:
             iret = 0
         if not separated_stderr:
@@ -431,7 +423,7 @@ class ASTER_SYSTEM:
                 if proto == 'RCP':
                     cmd = 'rcp -r '+fsrc+' '+fdest
                 elif proto == 'SCP':
-                    cmd = 'scp -rBCq '+fsrc+' '+fdest
+                    cmd = 'scp -rBCq -o StrictHostKeyChecking=no '+fsrc+' '+fdest
                 elif proto == 'RSYNC':
                     if self.IsDir(f) and not self.Exists(dest):
                         self.MkDir(dest)
@@ -475,11 +467,33 @@ class ASTER_SYSTEM:
             iret = result[0]
         return iret
 
+    def Symlink(self, src, link_name, verbose=True):
+        """Create a symbolic link."""
+        if on_windows():
+            return self.Copy(link_name, src)
+
+        self.VerbStart(ufmt(_(u'adding a symbolic link %s to %s...'),
+                              link_name, src), verbose=verbose)
+        iret = 0
+        output = ''
+        try:
+            if osp.exists(link_name):
+                self.Delete(link_name)
+            os.symlink(src, link_name)
+        except OSError, output:
+            iret = 4
+            self._mess(ufmt(_(u'error occurs during creating a symbolic link' \
+                    ' from %s to %s'), src, link_name), '<E>_SYMLINK')
+        self.VerbEnd(iret, output, verbose=verbose)
+        return iret
 
     def Shell(self, cmd, mach='', user='', **opts):
         """Execute a command shell on local or remote machine.
         Options : see local_shell
         """
+        #TODO use list type
+        if type(cmd) in (list, tuple):
+            cmd = ' '.join(cmd)
         iret = 1
         # valeurs par défaut
         kargs = {
@@ -489,17 +503,18 @@ class ASTER_SYSTEM:
             'follow_output'   : False,
             'alt_comment'     : cmd,
             'timeout'         : None,
+            'display_forwarding' : False,
         }
         # surcharge par opts
         kargs.update(opts)
 
         proto = self.param['remote_shell_protocol']
-        
+
         # distant ?
         user = user or local_user
-        distant = not is_localhost(mach, user=user)
+        distant = not is_localhost2(mach, user=user)
         self._dbg('remote command (%s <> %s and %s <> %s) ? %s' % (mach, local_host, user, local_user, distant))
-        
+
         if not distant:
             result = self.local_shell(cmd, **kargs)
         else:
@@ -511,20 +526,20 @@ class ASTER_SYSTEM:
             cmd = '"%s"' % cmd
             if proto == 'RSH':
                 action = 'rsh -n -l '+user+' '+mach+' '+cmd
-            elif proto == 'SSH':
+            #elif proto == 'SSH': is the default
+            else:
                 options = "-n -o StrictHostKeyChecking=no -o BatchMode=yes"
-                if kargs.get("timeout"):
+                if kargs['timeout']:
                     options += " -o 'ConnectTimeout=%s'" % kargs["timeout"]
+                if kargs['display_forwarding']:
+                    options += " -X"
                 action = "ssh %(options)s -l %(user)s %(host)s %(command)s" % {
                     "user" : user,
                     "host" : mach,
                     "command" : cmd,
                     "options" : options,
                 }
-            if action != '':
-                result = self.local_shell(action, **kargs)
-            else:
-                self._mess(_(u'unknown shell protocol : %s') % proto, '<A>_ALARM')
+            result = self.local_shell(action, **kargs)
         return result
 
 
@@ -614,7 +629,7 @@ class ASTER_SYSTEM:
         NB : we suppose that host and host.domain are identical.
         """
         dico  = self.filename2dict(path)
-        return not is_localhost(dico["mach"])
+        return not is_localhost2(dico["mach"])
 
 
     def Exists(self, path):
@@ -676,13 +691,15 @@ class ASTER_SYSTEM:
                     return
         if text:
             f2.write(text)
-        
+
         if type(dest) != file:
             f2.close()
 
 
     def _check_filetype(self, path):
         """Return file type or '' on remote files."""
+        if on_windows():
+            return ''
         dico = self.filename2dict(path)
         out = ''
         if not self.IsRemote(path):
@@ -723,23 +740,21 @@ class ASTER_SYSTEM:
             }
             iret, out = self.Shell(cmd, dico['mach'], dico['user'], verbose=verbose)
         else:
+            dico['name'] = osp.expandvars(dico['name'])
             self.VerbStart(ufmt(_(u'creating directory %s'), dico['name']), verbose)
             # ----- it's not tested in remote
-            if self.IsDir(rep):
-                iret = 0
-                self.VerbIgnore(verbose)
-            else:
-                s = ''
-                try:
-                    if os.path.isdir(dico['name']):
-                        self._dbg(ufmt(_(u'mkdir skipped, directory already exists %s'),
-                                       dico['name']))
-                    else:
-                        os.makedirs(dico['name'])
-                        os.chmod(dico['name'], chmod)
-                except OSError, s:
+            s = ''
+            try:
+                os.makedirs(dico['name'])
+            except OSError, s:
+                # maybe simultaneously created by another process
+                if not os.path.isdir(dico['name']):
                     iret = 4
-                self.VerbEnd(iret, s, verbose)
+            try:
+                os.chmod(dico['name'], chmod)
+            except OSError, s:
+                self._mess(ufmt(_(u'can not change permissions on %s'), rep), '<A>_ALARM')
+            self.VerbEnd(iret, s, verbose)
         if iret != 0:
             self._mess(ufmt(_(u'can not create directory %s'), rep), niverr)
         return iret
@@ -758,18 +773,19 @@ class ASTER_SYSTEM:
             self._mess(ufmt(_(u'deleting this directory seems too dangerous. ' \
                     '%s has not been removed'), dico['name']), '<A>_ALARM')
             return
-        
+
         if remove_dirs:
             cmd = command['rm_dirs'] % { 'args' : dico['name'] }
         else:
             cmd = command['rm_file'] % { 'args' : dico['name'] }
-        
-        comment = ufmt(_(u'deleting %s'), rep)
+
+        tail = '.../'+'/'.join(rep.split('/')[-2:])
+        comment = ufmt(_(u'deleting %s'), tail)
         if self.IsRemote(rep):
             iret, out = self.Shell(cmd, dico['mach'], dico['user'],
-                                alt_comment=comment, verbose=verbose)
+                                alt_comment=comment, verbose=verbose, stack_id=4)
         else:
-            iret, out = self.Shell(cmd, alt_comment=comment, verbose=verbose)
+            iret, out = self.Shell(cmd, alt_comment=comment, verbose=verbose, stack_id=4)
         return
 
 
@@ -798,7 +814,7 @@ class ASTER_SYSTEM:
         for k, v in env_prof.items():
             if env_init.get(k, None) != v:
                 self._dbg('AddToEnv set : %s=%s' % (k, v))
-                os.environ[k] = v
+                os.environ[k] = convert(v)
         for k in [k for k in env_init.keys() if env_prof.get(k) is None]:
             self._dbg('unset %s ' % k, DBG=True)
             try:
@@ -861,7 +877,7 @@ class ASTER_SYSTEM:
     def Ping(self, mach='', timeout=2):
         """Return True if 'mach' is responding.
         """
-        if is_localhost(mach):
+        if is_localhost2(mach):
             return True
         iret, output = self.Shell(command['ping'] % { 'host' : mach, 'timeout' : timeout })
         return iret == 0
@@ -877,8 +893,8 @@ class ASTER_SYSTEM:
            + os.linesep   + '-'*len(sign) + os.linesep
         try:
             import smtplib
-            from asrun.common.utils import MIMEText
-            if MIMEText is None:
+            from asrun.common.utils import MIMETextClass
+            if MIMETextClass is None:
                 raise ImportError
         except ImportError:
             self._mess(_(u'Can not send mail from this machine'),
@@ -887,7 +903,7 @@ class ASTER_SYSTEM:
         dest = [s.strip() for s in dest.split(',')]
         mail_encoding = get_encoding()
         content = convert(to_unicode(text) + sign, mail_encoding)
-        msg = MIMEText(content, _charset=mail_encoding)
+        msg = MIMETextClass(content, _charset=mail_encoding)
         msg['Subject'] = convert(subject, mail_encoding)
         if author == None:
             author = '%s@%s' % (local_user, local_full_host)
@@ -965,22 +981,26 @@ class ASTER_SYSTEM:
 
 
 
-class ASTER_SYSTEM_MINIMAL(ASTER_SYSTEM):
-    """Fake ASTER_RUN to use easily ASTER_SYSTEM outside of asrun.
+class AsterSystemMinimal(AsterSystem):
+    """Fake AsterRun to use easily AsterSystem outside of asrun.
     """
     def __init__(self, **kargs):
         """Initialization
         """
-        opts = { 
+        opts = {
             'debug'   : False,
             'verbose' : False,
             'remote_copy_protocol' : 'SCP',
             'remote_shell_protocol' : 'SSH',
         }
         opts.update(kargs)
-        ASTER_SYSTEM.__init__(self, run=opts)
+        AsterSystem.__init__(self, run=opts)
 
 
+ASTER_SYSTEM = bwc_deprecate_class('ASTER_SYSTEM', AsterSystem)
+ASTER_SYSTEM_MINIMAL = bwc_deprecate_class('ASTER_SYSTEM_MINIMAL',
+                                           AsterSystemMinimal)
+
 if __name__ == '__main__':
-    system = ASTER_SYSTEM_MINIMAL()
-    
+    system = AsterSystemMinimal()
+
