@@ -32,7 +32,7 @@ from glob         import glob
 from asrun.installation import aster_root
 from asrun.common.i18n import _
 from asrun.mystring     import print3, ufmt
-from asrun.config       import ASTER_CONFIG
+from asrun.config       import AsterConfig
 from asrun.progress     import Progress
 from asrun.common.utils import getpara, get_list, list_para_test
 
@@ -121,12 +121,12 @@ class LISTE_CT(object):
         self.liste_ct.sort()
         if self.verbose:
             print3(ufmt(_(u'%6d testcases in the list.'), len(self.liste_ct)))
-    
+
     def filename(self, ct, ext):
         """Retourne le nom du fichier du cas-test.
         """
         return os.path.join(self.astest_dir, '%s.%s' % (ct, ext))
-    
+
     def read_para_ct(self, ct):
         """Lecture du fichier .para d'un cas-test
         """
@@ -170,7 +170,7 @@ class LISTE_CT(object):
         if len(txt) > 0:
             d['mcfact'] = '.*%s *= *_F *\(' % txt.pop()
         assert len(txt) == 0, '>>> trop de valeurs : %s' % txt0
-        
+
         expr = '%(cmde)s *\(%(mcfact)s%(mcsimp)s%(valeur)s' % d
         return expr
 
@@ -265,7 +265,7 @@ def TestList(run, *args):
         fconf = os.path.join(REPREF, 'config.txt')
     else:
         fconf = os.path.abspath(run['config'])
-    conf = ASTER_CONFIG(fconf, run)
+    conf = AsterConfig(fconf, run)
 
     l_ct = list(args)
     if len(l_ct) < 1 and not (run['all_test'] or run.get('test_list')):
@@ -278,16 +278,16 @@ def TestList(run, *args):
             l_ct.extend(l_lue)
         else:
             run.Mess(ufmt(_(u'error during reading file : %s'), run['test_list']), '<F>_ERROR')
-    
+
     astest_dir = run.get('astest_dir', os.path.join(REPREF, conf['SRCTEST'][0]))
-    
+
     run.PrintExitCode = False
     if run['debug']:
         run.DBG('astest_dir', astest_dir)
         run.DBG('filter', run.get('filter'))
         run.DBG('command', run.get('command'))
         run.DBG('search', run.get('search'))
-    
+
     filtre = LISTE_CT(astest_dir=astest_dir, all=run['all_test'], verbose=not run['silent'])
     filtre.add_ct(l_ct)
     res = []
@@ -309,9 +309,9 @@ def TestList(run, *args):
         for f in run['user_filter']:
             filtre.add_filter('user', f)
             res.append('%% user filter, file : %s' % f)
-        
+
     res.extend(filtre.build_list())
-    
+
     result = os.linesep.join(res)
     if run.get('output'):
         open(run['output'], 'w').write(result)
