@@ -1,26 +1,26 @@
-#@ MODIF as_timer Utilitai  DATE 07/04/2008   AUTEUR COURTOIS M.COURTOIS 
+#@ MODIF as_timer Utilitai  DATE 07/04/2008   AUTEUR COURTOIS M.COURTOIS
 # -*- coding: utf-8 -*-
 #            CONFIGURATION MANAGEMENT OF EDF VERSION
 # ======================================================================
 # COPYRIGHT (C) 1991 - 2007  EDF R&D                  WWW.CODE-ASTER.ORG
-# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY  
-# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY  
-# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR     
-# (AT YOUR OPTION) ANY LATER VERSION.                                                  
-#                                                                       
-# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT   
-# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF            
-# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU      
-# GENERAL PUBLIC LICENSE FOR MORE DETAILS.                              
-#                                                                       
-# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE     
-# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,         
-#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.        
+# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
+# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
+# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
+# (AT YOUR OPTION) ANY LATER VERSION.
+#
+# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
+# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
+# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
+# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
+#
+# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
+# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
+#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
 # ======================================================================
 # RESPONSABLE
 
 """
-Definition of ASTER_TIMER class.
+Definition of AsterTimer class.
 """
 
 
@@ -32,6 +32,7 @@ from asrun.mystring import convert
 from asrun.common.utils import sha1
 from asrun.common.sysutils import on_windows
 
+from asrun.backward_compatibility import bwc_deprecate_class
 
 # os.times : On windows, only the first two items are filled, the others are zero
 _id_cpu = 0
@@ -59,7 +60,7 @@ def _conv_hms(t):
     return h, m, s
 
 
-class ASTER_TIMER:
+class AsterTimer:
     """This class provides methods to easily measure time spent during
     different steps.
     Methods :
@@ -94,16 +95,16 @@ class ASTER_TIMER:
         self.add_total = add_total
         self.comment = ''
         self._comment_seen = {}
-        
+
         if not format in ('as_run', 'aster'):
             format = 'as_run'
-        
+
         if format == 'as_run':
             self.mintitr = 33
             self._fmtlig = u'   %%(name)-%(adjtitr)ds  %%(cpu_dt)9.2f  %%(sys_dt)9.2f  %%(cpu_sys)9.2f  %%(tot_dt)9.2f'
             self._fmtstr = u'   %%(title)-%(adjtitr)ds  %%(cpu)9s  %%(sys)9s  %%(cpu+sys)9s  %%(elapsed)9s'
             self._sepa   = ' ' + '-'*81
-            self.TotalKey = u'Total'  # DO NOT translate this label because it will be "grep'ed" by ASTER_CALCUL
+            self.TotalKey = u'Total'  # DO NOT translate this label because it will be "grep'ed" by AsterCalcul
             self.d_labels = {
                 'title'   : '',
                 'cpu'     : _(u'cpu'),
@@ -123,7 +124,7 @@ class ASTER_TIMER:
                 'sys'     : u'SYSTEM',
                 'cpu+sys' : u'USER+SYS',
                 'elapsed' : u'ELAPSED',
-            }      
+            }
 
         self.adjust_format()
         self.total_key = id(self)
@@ -170,7 +171,7 @@ class ASTER_TIMER:
             self.timers[timer]['sys_dt'] += sys_dt
         if to_total and timer != self.total_key:
             self.Add(self.total_key, cpu_dt, sys_dt)
-        
+
 
     def Stop(self, timer, hide=None):
         """Stop a timer
@@ -272,7 +273,7 @@ class ASTER_TIMER:
         self.StopAll()
         if self.add_total:
             self.Stop(self.total_key)
-        
+
         self.adjust_format()
         labels = self.fmtstr % self.d_labels
         out = ['']
@@ -297,4 +298,4 @@ class ASTER_TIMER:
         string = os.linesep.join(out)
         return string
 
-
+ASTER_TIMER = bwc_deprecate_class('ASTER_TIMER', AsterTimer)
