@@ -25,7 +25,7 @@ import os
 import os.path as osp
 
 from asrun.common.i18n import _
-from asrun.config       import ASTER_CONFIG
+from asrun.config       import AsterConfig
 from asrun.mystring     import print3, ufmt
 from asrun.common_func  import get_tmpname
 from asrun.contrib      import convert2html
@@ -77,7 +77,7 @@ class GetInfosTask(Task):
     """
     # declare attrs
     run = host_infos = silent = None
-    
+
     def execute(self, host, **kwargs):
         """Function called for each item of the stack
         (up to 'nbmaxitem' at each called).
@@ -90,7 +90,7 @@ class GetInfosTask(Task):
             mem = self.run.GetMemInfo('memtotal', mach=host) or 0
         return host, connect, cpu, mem
 
-        
+
     def result(self, host, connect, cpu, mem, **kwargs):
         """Function called after each task to treat results of 'execute'.
         Arguments are 'execute' results + keywords args.
@@ -110,18 +110,18 @@ class GetInfosTask(Task):
 # to be used by Code_Aster/UMAT testcases
 def make_shared(lib, srcfiles, conf=None, compiler_command=None):
     """Produce a shared library from a list of source files
-    using the provided command line or a ASTER_CONFIG object."""
+    using the provided command line or a AsterConfig object."""
     # command line
     cmd = []
     # using the provided command line
     if compiler_command is not None:
         cmd.append(compiler_command)
-    # using ASTER_CONFIG object
+    # using AsterConfig object
     elif conf is None:
         # supposed to exist in pwd
         assert osp.exists("config.txt"), \
-            "at least a command line, a ASTER_CONFIG object or config.txt file is required!"
-        conf = ASTER_CONFIG("config.txt")
+            "at least a command line, a AsterConfig object or config.txt file is required!"
+        conf = AsterConfig("config.txt")
     if conf is not None:
         cmd.append(conf['F90'][0])
         cmd.extend(conf['OPTF90_O'])
@@ -153,12 +153,12 @@ def MakeShared(run, *l_src):
     run.check_version_setting()
     REPREF = run.get_version_path(run['aster_vers'])
     ficconf = osp.join(REPREF, 'config.txt')
-    conf = ASTER_CONFIG(ficconf, run)
+    conf = AsterConfig(ficconf, run)
 
     # set per version environment
     for f in conf.get_with_absolute_path('ENV_SH'):
         run.AddToEnv(f)
-    
+
     make_shared(run['output'], l_src, conf)
 
 
