@@ -12,14 +12,15 @@ import os
 import unittest
 from glob      import glob
 from optparse  import OptionParser
+from pprint import pprint
 
 import common
 
 DEFAULT = [
-    "010_misc", "015_package", "020_object",
+    "010_misc", "015_package", "020_object", "030_server",
     "050_build_export", "060_thread", "065_system", "080_agla",
     "100_study", "110_server",
-    "200_client",
+    "200_client", "210_multi",
     "300_development", "310_study_dvp", "320_astout_dvp",
 ]
 if os.name != "posix":
@@ -55,12 +56,26 @@ if __name__ == "__main__":
             help="run all testcases")
     parser.add_option('--long', dest='long', action='store_true', default=False,
             help="run only long testcases")
+    parser.add_option('--vers', dest='vers', action='store', default=None,
+            help="use the specify version")
+    parser.add_option('--host', dest='host', action='store', default=None,
+            help="list of hosts (comma separated) to check remote features with "
+                 "their ASTER_ROOT directory. Example : "
+                 "--host=computer1:/opt/aster,server2:/aster "
+                 "(using a different username is not yet supported)")
     opts, l_args = parser.parse_args()
-    
-    common.init()
+
+    common.init(opts.host)
+    if opts.vers:
+        common.set_version(opts.vers)
     print 'Temporary directory :', common.tmpdir
+    print "Configuration for unittests :"
+    pprint(common.dict_conf)
+    if opts.host:
+        from data import available_hosts
+        print 'Available remote hosts : ', ', '.join(available_hosts.keys())
     print
-    
+
     if opts.all:
         l_args = ALL
     elif opts.long:
