@@ -35,11 +35,17 @@ def get_version_number_last(prefix):
    try:
       path = sysconfig.get_python_lib(prefix=prefix)
       cmd = [sys.executable, '-c',
-             "import sys ; sys.path.insert(0, '%s') ; import asrun ; print 'as_run', asrun.__version__" % path]
-      output = execute(cmd, env={'PYTHONPATH' : ''})
+             "import sys ; sys.path.insert(0, '%s') ; import asrun ; "\
+             "print 'as_run # %%s # %%s' %% (asrun.__version__, asrun.__file__)" % path]
+      output = execute(cmd)
       log.debug("get_version_number >= 1.8, output :\n%s", output)
-      mat = re.search('as_run +([0-9\-\.A-Za-z_]+)', output)
-      version = mat.group(1)
+      out = output.split('#')
+      if len(out) != 3:
+          return version
+      version, asrfile = [s.strip() for s in out[1:3]]
+      asrfile = re.sub("/asrun/__init__.py.*$", "", asrfile)
+      if asrfile.replace(path, '') != '':
+          raise Exception("unexpected asrun found :%s" % asrfile)
    except Exception, msg:
       log.debug("get_version_number >= 1.8, message : %s", str(msg))
    return version
@@ -52,7 +58,7 @@ def get_version_number_1_7_5(prefix):
       assert os.path.exists(os.path.join(path, '__pkginfo__.py')), 'module not found: %s' % path
       cmd = [sys.executable, '-c',
              "import sys ; sys.path.insert(0, '%s') ; import __pkginfo__ as p ; print 'as_run', p.version" % path]
-      output = execute(cmd, env={'PYTHONPATH' : ''})
+      output = execute(cmd)
       log.debug("get_version_number >= 1.7.5, output :\n%s", output)
       mat = re.search('as_run +([0-9\-\.A-Za-z_]+)', output)
       version = mat.group(1)
@@ -90,7 +96,7 @@ def get_parameters(version, where, prefix=None):
    # default values
    parameters = {
       'ASTER_ROOT'      : prefix,
-      'ASTER_VERSION'   : 'STA10.2',
+      'ASTER_VERSION'   : 'STA10.4',
       'SHELL_EXECUTION' : '/bin/bash',
       'WISH_EXE'        : 'wish',
       'PYTHON_EXE'      : sys.executable,
@@ -107,9 +113,8 @@ def get_parameters(version, where, prefix=None):
       log.info(str(error))
       pass
    parameters.update({
-      'ASTER_CONFDIR'  : os.path.join(prefix2etc(parameters['ASTER_ROOT']), 'codeaster'),
-      'TOOLS_DIR'      : os.path.join(parameters['ASTER_ROOT'], 'outils'),
-      'ASRUN_SITE_PKG' : sysconfig.get_python_lib(prefix=parameters['ASTER_ROOT']),
+      'TOOLS_DIR'      : os.path.join('$ASTER_ROOT', 'outils'),
+      'ASRUN_SITE_PKG' : sysconfig.get_python_lib(prefix='$ASTER_ROOT'),
    })
    if where:
       # SHELL_EXECUTION
@@ -138,7 +143,7 @@ def func_wish(param, where, version):
    files = ( ('0.0.0', os.path.join(where, 'ASTK', 'ASTK_CLIENT', 'bin', 'astk')),
              ('1.8.0', os.path.join(where, 'etc', 'codeaster', 'profile.sh')) )
    expressions = ( ('0.0.0', '(.*) +.*/ASTK/ASTK_CLIENT/lib/ASTK/astk.tcl'),
-                   ('1.8.0', 'WISHEXECUTABLE=(.*)') )   
+                   ('1.8.0', 'WISHEXECUTABLE=(.*)') )
    mat = re_search(get_function(files, version), get_function(expressions, version))
    check_and_store(param, mat, 'WISH_EXE')
 
