@@ -10,7 +10,6 @@ from distutils      import log, file_util, dir_util
 
 YES_VALUES = ['yes', 'oui', 'y', 'o']
 
-
 def execute(cmd, env=None):
    """Executes a command and returns its output."""
    p = Popen(cmd, stdout=PIPE, stderr=STDOUT, close_fds=True, env=env)
@@ -221,6 +220,9 @@ def chmod_scripts(files):
    if type(files) not in (list, tuple):
       files = [files,]
    for filename in files:
+      if not os.access(filename, os.W_OK):
+          log.warn("no sufficient permission to change '%s', skipped.", filename)
+          continue
       mode = ((os.stat(filename)[ST_MODE]) | 0555) & 07777
       log.info("changing mode of %s to %o", filename, mode)
       os.chmod(filename, mode)
@@ -228,7 +230,9 @@ def chmod_scripts(files):
 
 # unittest
 def _test_get_function():
-   ff = ( ('0.0.0', 'func < 1.7.5'), ('1.7.5', 'func >= 1.7.5'), ('1.8.0', 'func >= 1.8.0') )
-   for v in ('1.2.3', '1.7.4', '1.7.5', '1.7.99', '1.8', '1.8.01', 'unknown'):
+   ff = ( ('0.0.0', 'func < 1.7.5'), ('1.7.5', 'func >= 1.7.5'),
+          ('1.8.0', 'func >= 1.8.0'), ('1.10.0', 'func >= 1.10'), )
+   for v in ('1.2.3', '1.7.4', '1.7.5', '1.7.99', '1.8', '1.8.01',
+            '1.10', '1.11', 'unknown',):
       print v, '-->', get_function(ff, v)
 
